@@ -47,10 +47,6 @@ var tabPreviews = {
   },
 
   capture: function tabPreviews_capture(aTab, aShouldCache) {
-    // Bug 863512 - Make page thumbnails work in electrolysis
-    if (gMultiProcessBrowser)
-      return new Image();
-
     let browser = aTab.linkedBrowser;
     let uri = browser.currentURI.spec;
 
@@ -77,7 +73,7 @@ var tabPreviews = {
       aTab.__thumbnail_lastURI = uri;
     }
 
-    PageThumbs.captureToCanvas(aTab.linkedBrowser.contentWindow, canvas);
+    PageThumbs.captureToCanvas(browser, canvas);
     return canvas;
   },
 
