@@ -370,7 +370,11 @@ loop.webapp = (function($, _, OT, mozL10n) {
     },
 
     render: function() {
-      var callState = mozL10n.get("call_progress_" + this.state.callState + "_description");
+      var callStateStringEntityName = "call_progress_" + this.state.callState + "_description";
+      var callState = mozL10n.get(callStateStringEntityName);
+      document.title = mozL10n.get("standalone_title_with_status",
+                                   {clientShortname: mozL10n.get("clientShortname2"),
+                                    currentStatus: mozL10n.get(callStateStringEntityName)});
 
       return (
         PendingConversationView({
@@ -584,6 +588,9 @@ loop.webapp = (function($, _, OT, mozL10n) {
     },
 
     render: function() {
+      document.title = mozL10n.get("standalone_title_with_status",
+                                   {clientShortname: mozL10n.get("clientShortname2"),
+                                    currentStatus: mozL10n.get("status_conversation_ended")});
       return (
         React.DOM.div({className: "ended-conversation"}, 
           sharedViews.FeedbackView({
@@ -604,6 +611,7 @@ loop.webapp = (function($, _, OT, mozL10n) {
 
   var StartConversationView = React.createClass({displayName: 'StartConversationView',
     render: function() {
+      document.title = mozL10n.get("clientShortname2");
       return this.transferPropsTo(
         InitiateConversationView({
           title: mozL10n.get("initiate_call_button_label2"), 
@@ -620,6 +628,9 @@ loop.webapp = (function($, _, OT, mozL10n) {
     },
 
     render: function() {
+      document.title = mozL10n.get("standalone_title_with_status",
+                                   {clientShortname: mozL10n.get("clientShortname2"),
+                                    currentStatus: mozL10n.get("status_error")});
       return this.transferPropsTo(
         InitiateConversationView({
           title: mozL10n.get("call_failed_title"), 
@@ -712,6 +723,9 @@ loop.webapp = (function($, _, OT, mozL10n) {
           return WaitingConversationView({websocket: this._websocket});
         }
         case "connected": {
+          document.title = mozL10n.get("standalone_title_with_status",
+                                       {clientShortname: mozL10n.get("clientShortname2"),
+                                        currentStatus: mozL10n.get("status_in_conversation")});
           return (
             sharedViews.ConversationView({
               initiate: true, 
