@@ -1,22 +1,17 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-function test() {
-  TestRunner.run();
-}
+"use strict";
 
 /**
- * This test ensures that disabling features by flipping nsIDocShell.allow*
+ * These tests ensures that disabling features by flipping nsIDocShell.allow*
  * properties are (re)stored as disabled. Disallowed features must be
  * re-enabled when the tab is re-used by another tab restoration.
  */
-
-function runTests() {
-  // Create a tab that we're going to use for our tests.
-  let tab = gBrowser.selectedTab = gBrowser.addTab("about:mozilla");
+add_task(function docshell_capabilities() {
+  let tab = yield createTab();
   let browser = tab.linkedBrowser;
   let docShell = browser.docShell;
-  yield waitForLoad(browser);
 
   // Get the list of capabilities for docShells.
   let flags = Object.keys(docShell).filter(k => k.startsWith("allow"));
@@ -31,9 +26,12 @@ function runTests() {
   docShell.allowMetaRedirects = false;
 
   // Now reload the document to ensure that these capabilities
-  // are taken into account
+  // are taken into account.
   browser.reload();
-  yield whenBrowserLoaded(browser);
+  yield promiseBrowserLoaded(browser);
+
+  // Flush to make sure chrome received all data.
+  TabState.flush(browser);
 
   // Check that we correctly save disallowed features.
   let disallowedState = JSON.parse(ss.getTabState(tab));
@@ -44,7 +42,10 @@ function runTests() {
 
   // Reuse the tab to restore a new, clean state into it.
   ss.setTabState(tab, JSON.stringify({ entries: [{url: "about:robots"}] }));
-  yield waitForLoad(browser);
+  yield promiseTabRestored(tab);
+
+  // Flush to make sure chrome received all data.
+  TabState.flush(browser);
 
   // After restoring disallowed features must be available again.
   state = JSON.parse(ss.getTabState(tab));
@@ -53,7 +54,7 @@ function runTests() {
 
   // Restore the state with disallowed features.
   ss.setTabState(tab, JSON.stringify(disallowedState));
-  yield waitForLoad(browser);
+  yield promiseTabRestored(tab);
 
   // Check that docShell flags are set.
   ok(!docShell.allowImages, "images not allowed");
@@ -68,11 +69,10 @@ function runTests() {
 
   // Clean up after ourselves.
   gBrowser.removeTab(tab);
-}
+});
 
-function waitForLoad(aElement) {
-  aElement.addEventListener("load", function onLoad() {
-    aElement.removeEventListener("load", onLoad, true);
-    executeSoon(next);
-  }, true);
+function createTab() {
+  let tab = gBrowser.addTab("about:mozilla");
+  let browser = tab.linkedBrowser;
+  return promiseBrowserLoaded(browser).then(() => tab);
 }
