@@ -49,6 +49,8 @@ function ToolSidebar(tabbox, panel, uid, showTabstripe=true)
   if (!showTabstripe) {
     this._tabbox.setAttribute("hidetabs", "true");
   }
+
+  this._toolPanel.emit("sidebar-created", this);
 }
 
 exports.ToolSidebar = ToolSidebar;
@@ -70,12 +72,16 @@ ToolSidebar.prototype = {
 
     let tab = this._tabbox.tabs.appendItem();
     tab.setAttribute("label", ""); // Avoid showing "undefined" while the tab is loading
+    tab.setAttribute("id", "sidebar-tab-" + id);
+
+    let onIFrameLoaded = (event) => {
+      let doc = event.target;
+      let win = doc.defaultView;
+      tab.setAttribute("label", doc.title);
 
-    let onIFrameLoaded = () => {
-      tab.setAttribute("label", iframe.contentDocument.title);
       iframe.removeEventListener("load", onIFrameLoaded, true);
-      if ("setPanel" in iframe.contentWindow) {
-        iframe.contentWindow.setPanel(this._toolPanel, iframe);
+      if ("setPanel" in win) {
+        win.setPanel(this._toolPanel, iframe);
       }
       this.emit(id + "-ready");
     };
@@ -109,6 +115,27 @@ ToolSidebar.prototype = {
     this.emit("new-tab-registered", id);
   },
 
+  /**
+   * Remove an existing tab.
+   */
+  removeTab: function(id) {
+    let tab = this._tabbox.tabs.querySelector("tab#sidebar-tab-" + id);
+    if (!tab) {
+      return;
+    }
+
+    tab.remove();
+
+    let panel = this.getTab(id);
+    if (panel) {
+      panel.remove();
+    }
+
+    this._tabs.delete(id);
+
+    this.emit("tab-unregistered", id);
+  },
+
   /**
    * Select a specific tab.
    */
@@ -185,6 +212,8 @@ ToolSidebar.prototype = {
       this._tabbox.width = this._width;
     }
     this._tabbox.removeAttribute("hidden");
+
+    this.emit("show");
   },
 
   /**
@@ -193,6 +222,8 @@ ToolSidebar.prototype = {
   hide: function ToolSidebar_hide() {
     Services.prefs.setIntPref("devtools.toolsidebar-width." + this._uid, this._tabbox.width);
     this._tabbox.setAttribute("hidden", "true");
+
+    this.emit("hide");
   },
 
   /**
@@ -232,6 +263,8 @@ ToolSidebar.prototype = {
       this._telemetry.toolClosed(this._currentTool);
     }
 
+    this._toolPanel.emit("sidebar-destroyed", this);
+
     this._tabs = null;
     this._tabbox = null;
     this._panelDoc = null;
