@@ -8,7 +8,7 @@
  * of all the markers in the timeline data.
  */
 
-const {Cc, Ci, Cu, Cr} = require("chrome");
+const {Ci, Cu} = require("chrome");
 
 loader.lazyRequireGetter(this, "L10N",
   "devtools/timeline/global", true);
@@ -19,18 +19,19 @@ loader.lazyImporter(this, "setNamedTimeout",
   "resource:///modules/devtools/ViewHelpers.jsm");
 loader.lazyImporter(this, "clearNamedTimeout",
   "resource:///modules/devtools/ViewHelpers.jsm");
+loader.lazyRequireGetter(this, "EventEmitter",
+  "devtools/toolkit/event-emitter");
 
 const HTML_NS = "http://www.w3.org/1999/xhtml";
 
-const TIMELINE_IMMEDIATE_DRAW_MARKERS_COUNT = 30;
-const TIMELINE_FLUSH_OUTSTANDING_MARKERS_DELAY = 75; // ms
+const WATERFALL_SIDEBAR_WIDTH = 150; // px
 
-const TIMELINE_HEADER_TICKS_MULTIPLE = 5; // ms
-const TIMELINE_HEADER_TICKS_SPACING_MIN = 50; // px
-const TIMELINE_HEADER_TEXT_PADDING = 3; // px
+const WATERFALL_IMMEDIATE_DRAW_MARKERS_COUNT = 30;
+const WATERFALL_FLUSH_OUTSTANDING_MARKERS_DELAY = 75; // ms
 
-const TIMELINE_MARKER_SIDEBAR_WIDTH = 150; // px
-const TIMELINE_MARKER_BAR_WIDTH_MIN = 5; // px
+const WATERFALL_HEADER_TICKS_MULTIPLE = 5; // ms
+const WATERFALL_HEADER_TICKS_SPACING_MIN = 50; // px
+const WATERFALL_HEADER_TEXT_PADDING = 3; // px
 
 const WATERFALL_BACKGROUND_TICKS_MULTIPLE = 5; // ms
 const WATERFALL_BACKGROUND_TICKS_SCALES = 3;
@@ -38,6 +39,9 @@ const WATERFALL_BACKGROUND_TICKS_SPACING_MIN = 10; // px
 const WATERFALL_BACKGROUND_TICKS_COLOR_RGB = [128, 136, 144];
 const WATERFALL_BACKGROUND_TICKS_OPACITY_MIN = 32; // byte
 const WATERFALL_BACKGROUND_TICKS_OPACITY_ADD = 32; // byte
+const WATERFALL_MARKER_BAR_WIDTH_MIN = 5; // px
+
+const WATERFALL_ROWCOUNT_ONPAGEUPDOWN = 10;
 
 /**
  * A detailed waterfall view for the timeline data.
@@ -46,20 +50,23 @@ const WATERFALL_BACKGROUND_TICKS_OPACITY_ADD = 32; // byte
  *        The parent node holding the waterfall.
  */
 function Waterfall(parent) {
+  EventEmitter.decorate(this);
   this._parent = parent;
   this._document = parent.ownerDocument;
   this._fragment = this._document.createDocumentFragment();
   this._outstandingMarkers = [];
 
   this._headerContents = this._document.createElement("hbox");
-  this._headerContents.className = "timeline-header-contents";
+  this._headerContents.className = "waterfall-header-contents";
   this._parent.appendChild(this._headerContents);
 
   this._listContents = this._document.createElement("vbox");
-  this._listContents.className = "timeline-list-contents";
+  this._listContents.className = "waterfall-list-contents";
   this._listContents.setAttribute("flex", "1");
   this._parent.appendChild(this._listContents);
 
+  this.setupKeys();
+
   this._isRTL = this._getRTL();
 
   // Lazy require is a bit slow, and these are hot objects.
@@ -67,6 +74,10 @@ function Waterfall(parent) {
   this._blueprint = TIMELINE_BLUEPRINT;
   this._setNamedTimeout = setNamedTimeout;
   this._clearNamedTimeout = clearNamedTimeout;
+
+  // Selected row index. By default, we want the first
+  // row to be selected.
+  this._selectedRowIdx = 0;
 }
 
 Waterfall.prototype = {
@@ -75,18 +86,57 @@ Waterfall.prototype = {
    *
    * @param array markers
    *        A list of markers received from the controller.
-   * @param number timeStart
-   *        The delta time (in milliseconds) to start drawing from.
-   * @param number timeEnd
-   *        The delta time (in milliseconds) to end drawing at.
+   * @param number timeEpoch
+   *        The absolute time (in milliseconds) when the recording started.
+   * @param number startTime
+   *        The time (in milliseconds) to start drawing from.
+   * @param number endTime
+   *        The time (in milliseconds) to end drawing at.
    */
-  setData: function(markers, timeStart, timeEnd) {
+  setData: function(markers, timeEpoch, startTime, endTime) {
     this.clearView();
+    this._markers = markers;
 
-    let dataScale = this._waterfallWidth / (timeEnd - timeStart);
+    let dataScale = this._waterfallWidth / (endTime - startTime);
     this._drawWaterfallBackground(dataScale);
-    this._buildHeader(this._headerContents, timeStart, dataScale);
-    this._buildMarkers(this._listContents, markers, timeStart, timeEnd, dataScale);
+
+    // Label the header as if the first possible marker was at T=0.
+    this._buildHeader(this._headerContents, startTime - timeEpoch, dataScale);
+    this._buildMarkers(this._listContents, markers, startTime, endTime, dataScale);
+    this.selectRow(this._selectedRowIdx);
+  },
+
+  /**
+   * Keybindings.
+   */
+  setupKeys: function() {
+    let pane = this._document.querySelector("#timeline-pane");
+    pane.parentNode.parentNode.addEventListener("keydown", e => {
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_UP) {
+        e.preventDefault();
+        this.selectNearestRow(this._selectedRowIdx - 1);
+      }
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_DOWN) {
+        e.preventDefault();
+        this.selectNearestRow(this._selectedRowIdx + 1);
+      }
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_HOME) {
+        e.preventDefault();
+        this.selectNearestRow(0);
+      }
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_END) {
+        e.preventDefault();
+        this.selectNearestRow(this._listContents.children.length);
+      }
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_PAGE_UP) {
+        e.preventDefault();
+        this.selectNearestRow(this._selectedRowIdx - WATERFALL_ROWCOUNT_ONPAGEUPDOWN);
+      }
+      if (e.keyCode === Ci.nsIDOMKeyEvent.DOM_VK_PAGE_DOWN) {
+        e.preventDefault();
+        this.selectNearestRow(this._selectedRowIdx + WATERFALL_ROWCOUNT_ONPAGEUPDOWN);
+      }
+    }, true);
   },
 
   /**
@@ -110,7 +160,7 @@ Waterfall.prototype = {
    */
   recalculateBounds: function() {
     let bounds = this._parent.getBoundingClientRect();
-    this._waterfallWidth = bounds.width - TIMELINE_MARKER_SIDEBAR_WIDTH;
+    this._waterfallWidth = bounds.width - WATERFALL_SIDEBAR_WIDTH;
   },
 
   /**
@@ -118,29 +168,29 @@ Waterfall.prototype = {
    *
    * @param nsIDOMNode parent
    *        The parent node holding the header.
-   * @param number timeStart
+   * @param number startTime
    *        @see Waterfall.prototype.setData
    * @param number dataScale
    *        The time scale of the data source.
    */
-  _buildHeader: function(parent, timeStart, dataScale) {
+  _buildHeader: function(parent, startTime, dataScale) {
     let container = this._document.createElement("hbox");
-    container.className = "timeline-header-container";
+    container.className = "waterfall-header-container";
     container.setAttribute("flex", "1");
 
     let sidebar = this._document.createElement("hbox");
-    sidebar.className = "timeline-header-sidebar theme-sidebar";
-    sidebar.setAttribute("width", TIMELINE_MARKER_SIDEBAR_WIDTH);
+    sidebar.className = "waterfall-sidebar theme-sidebar";
+    sidebar.setAttribute("width", WATERFALL_SIDEBAR_WIDTH);
     sidebar.setAttribute("align", "center");
     container.appendChild(sidebar);
 
     let name = this._document.createElement("label");
-    name.className = "plain timeline-header-name";
+    name.className = "plain waterfall-header-name";
     name.setAttribute("value", this._l10n.getStr("timeline.records"));
     sidebar.appendChild(name);
 
     let ticks = this._document.createElement("hbox");
-    ticks.className = "timeline-header-ticks";
+    ticks.className = "waterfall-header-ticks waterfall-background-ticks";
     ticks.setAttribute("align", "center");
     ticks.setAttribute("flex", "1");
     container.appendChild(ticks);
@@ -148,18 +198,18 @@ Waterfall.prototype = {
     let offset = this._isRTL ? this._waterfallWidth : 0;
     let direction = this._isRTL ? -1 : 1;
     let tickInterval = this._findOptimalTickInterval({
-      ticksMultiple: TIMELINE_HEADER_TICKS_MULTIPLE,
-      ticksSpacingMin: TIMELINE_HEADER_TICKS_SPACING_MIN,
+      ticksMultiple: WATERFALL_HEADER_TICKS_MULTIPLE,
+      ticksSpacingMin: WATERFALL_HEADER_TICKS_SPACING_MIN,
       dataScale: dataScale
     });
 
     for (let x = 0; x < this._waterfallWidth; x += tickInterval) {
-      let start = x + direction * TIMELINE_HEADER_TEXT_PADDING;
-      let time = Math.round(timeStart + x / dataScale);
+      let start = x + direction * WATERFALL_HEADER_TEXT_PADDING;
+      let time = Math.round(startTime + x / dataScale);
       let label = this._l10n.getFormatStr("timeline.tick", time);
 
       let node = this._document.createElement("label");
-      node.className = "plain timeline-header-tick";
+      node.className = "plain waterfall-header-tick";
       node.style.transform = "translateX(" + (start - offset) + "px)";
       node.setAttribute("value", label);
       ticks.appendChild(node);
@@ -173,23 +223,25 @@ Waterfall.prototype = {
    *
    * @param nsIDOMNode parent
    *        The parent node holding the markers.
-   * @param number timeStart
+   * @param number startTime
    *        @see Waterfall.prototype.setData
    * @param number dataScale
    *        The time scale of the data source.
    */
-  _buildMarkers: function(parent, markers, timeStart, timeEnd, dataScale) {
-    let processed = 0;
+  _buildMarkers: function(parent, markers, startTime, endTime, dataScale) {
+    let rowsCount = 0;
+    let markerIdx = -1;
 
     for (let marker of markers) {
-      if (!isMarkerInRange(marker, timeStart, timeEnd)) {
+      markerIdx++;
+      if (!isMarkerInRange(marker, startTime, endTime)) {
         continue;
       }
       // Only build and display a finite number of markers initially, to
       // preserve a snappy UI. After a certain delay, continue building the
       // outstanding markers while there's (hopefully) no user interaction.
-      let arguments_ = [this._fragment, marker, timeStart, dataScale];
-      if (processed++ < TIMELINE_IMMEDIATE_DRAW_MARKERS_COUNT) {
+      let arguments_ = [this._fragment, marker, startTime, dataScale, markerIdx, rowsCount];
+      if (rowsCount++ < WATERFALL_IMMEDIATE_DRAW_MARKERS_COUNT) {
         this._buildMarker.apply(this, arguments_);
       } else {
         this._outstandingMarkers.push(arguments_);
@@ -204,7 +256,7 @@ Waterfall.prototype = {
     // Otherwise prepare flushing the outstanding markers after a small delay.
     else {
       this._setNamedTimeout("flush-outstanding-markers",
-        TIMELINE_FLUSH_OUTSTANDING_MARKERS_DELAY,
+        WATERFALL_FLUSH_OUTSTANDING_MARKERS_DELAY,
         () => this._buildOutstandingMarkers(parent));
     }
 
@@ -224,6 +276,7 @@ Waterfall.prototype = {
     }
     this._outstandingMarkers.length = 0;
     parent.appendChild(this._fragment);
+    this.selectRow(this._selectedRowIdx);
   },
 
   /**
@@ -233,18 +286,24 @@ Waterfall.prototype = {
    *        The parent node holding the marker.
    * @param object marker
    *        The { name, start, end } marker in the data source.
-   * @param timeStart
+   * @param startTime
    *        @see Waterfall.prototype.setData
    * @param number dataScale
    *        @see Waterfall.prototype._buildMarkers
+   * @param number markerIdx
+   *        Index of the marker in this._markers
+   * @param number rowIdx
+   *        Index of current row
    */
-  _buildMarker: function(parent, marker, timeStart, dataScale) {
+  _buildMarker: function(parent, marker, startTime, dataScale, markerIdx, rowIdx) {
     let container = this._document.createElement("hbox");
-    container.className = "timeline-marker-container";
+    container.setAttribute("markerIdx", markerIdx);
+    container.className = "waterfall-marker-container";
 
     if (marker) {
       this._buildMarkerSidebar(container, marker);
-      this._buildMarkerWaterfall(container, marker, timeStart, dataScale);
+      this._buildMarkerWaterfall(container, marker, startTime, dataScale, markerIdx);
+      container.onclick = () => this.selectRow(rowIdx);
     } else {
       this._buildMarkerSpacer(container);
       container.setAttribute("flex", "1");
@@ -254,6 +313,83 @@ Waterfall.prototype = {
     parent.appendChild(container);
   },
 
+  /**
+   * Select first row.
+   */
+  resetSelection: function() {
+    this.selectRow(0);
+  },
+
+  /**
+   * Select a marker in the waterfall.
+   *
+   * @param number idx
+   *        Index of the row to select. -1 clears the selection.
+   */
+  selectRow: function(idx) {
+    // Unselect
+    let prev = this._listContents.children[this._selectedRowIdx];
+    if (prev) {
+      prev.classList.remove("selected");
+    }
+
+    this._selectedRowIdx = idx;
+
+    let row = this._listContents.children[idx];
+    if (row && !row.hasAttribute("is-spacer")) {
+      row.focus();
+      row.classList.add("selected");
+      let markerIdx = row.getAttribute("markerIdx");
+      this.emit("selected", this._markers[markerIdx]);
+      this.ensureRowIsVisible(row);
+    } else {
+      this.emit("unselected");
+    }
+  },
+
+  /**
+   * Find a valid row to select.
+   *
+   * @param number idx
+   *        Index of the row to select.
+   */
+  selectNearestRow: function(idx) {
+    if (this._listContents.children.length == 0) {
+      return;
+    }
+    idx = Math.max(idx, 0);
+    idx = Math.min(idx, this._listContents.children.length - 1);
+    let row = this._listContents.children[idx];
+    if (row && row.hasAttribute("is-spacer")) {
+      if (idx > 0) {
+        return this.selectNearestRow(idx - 1);
+      } else {
+        return;
+      }
+    }
+    this.selectRow(idx);
+  },
+
+  /**
+   * Scroll waterfall to ensure row is in the viewport.
+   *
+   * @param number idx
+   *        Index of the row to select.
+   */
+  ensureRowIsVisible: function(row) {
+    let parent = row.parentNode;
+    let parentRect = parent.getBoundingClientRect();
+    let rowRect = row.getBoundingClientRect();
+    let yDelta = rowRect.top - parentRect.top;
+    if (yDelta < 0) {
+      parent.scrollTop += yDelta;
+    }
+    yDelta = parentRect.bottom - rowRect.bottom;
+    if (yDelta < 0) {
+      parent.scrollTop -= yDelta;
+    }
+  },
+
   /**
    * Creates the sidebar part of a marker in this view.
    *
@@ -266,20 +402,32 @@ Waterfall.prototype = {
     let blueprint = this._blueprint[marker.name];
 
     let sidebar = this._document.createElement("hbox");
-    sidebar.className = "timeline-marker-sidebar theme-sidebar";
-    sidebar.setAttribute("width", TIMELINE_MARKER_SIDEBAR_WIDTH);
+    sidebar.className = "waterfall-sidebar theme-sidebar";
+    sidebar.setAttribute("width", WATERFALL_SIDEBAR_WIDTH);
     sidebar.setAttribute("align", "center");
 
     let bullet = this._document.createElement("hbox");
-    bullet.className = "timeline-marker-bullet";
+    bullet.className = "waterfall-marker-bullet";
     bullet.style.backgroundColor = blueprint.fill;
     bullet.style.borderColor = blueprint.stroke;
     bullet.setAttribute("type", marker.name);
     sidebar.appendChild(bullet);
 
     let name = this._document.createElement("label");
-    name.className = "plain timeline-marker-name";
-    name.setAttribute("value", blueprint.label);
+    name.setAttribute("crop", "end");
+    name.setAttribute("flex", "1");
+    name.className = "plain waterfall-marker-name";
+
+    let label;
+    if (marker.causeName) {
+      label = this._l10n.getFormatStr("timeline.markerDetailFormat",
+                                      blueprint.label,
+                                      marker.causeName);
+    } else {
+      label = blueprint.label;
+    }
+    name.setAttribute("value", label);
+    name.setAttribute("tooltiptext", label);
     sidebar.appendChild(name);
 
     container.appendChild(sidebar);
@@ -292,29 +440,32 @@ Waterfall.prototype = {
    *        The container node representing the marker.
    * @param object marker
    *        @see Waterfall.prototype._buildMarker
-   * @param timeStart
+   * @param startTime
    *        @see Waterfall.prototype.setData
    * @param number dataScale
    *        @see Waterfall.prototype._buildMarkers
    */
-  _buildMarkerWaterfall: function(container, marker, timeStart, dataScale) {
+  _buildMarkerWaterfall: function(container, marker, startTime, dataScale) {
     let blueprint = this._blueprint[marker.name];
 
     let waterfall = this._document.createElement("hbox");
-    waterfall.className = "timeline-marker-waterfall";
+    waterfall.className = "waterfall-marker-item waterfall-background-ticks";
+    waterfall.setAttribute("align", "center");
     waterfall.setAttribute("flex", "1");
 
-    let start = (marker.start - timeStart) * dataScale;
+    let start = (marker.start - startTime) * dataScale;
     let width = (marker.end - marker.start) * dataScale;
     let offset = this._isRTL ? this._waterfallWidth : 0;
 
     let bar = this._document.createElement("hbox");
-    bar.className = "timeline-marker-bar";
+    bar.className = "waterfall-marker-bar";
     bar.style.backgroundColor = blueprint.fill;
     bar.style.borderColor = blueprint.stroke;
+    // Save border color. It will change when marker is selected.
+    bar.setAttribute("borderColor", blueprint.stroke);
     bar.style.transform = "translateX(" + (start - offset) + "px)";
     bar.setAttribute("type", marker.name);
-    bar.setAttribute("width", Math.max(width, TIMELINE_MARKER_BAR_WIDTH_MIN));
+    bar.setAttribute("width", Math.max(width, WATERFALL_MARKER_BAR_WIDTH_MIN));
     waterfall.appendChild(bar);
 
     container.appendChild(waterfall);
@@ -328,11 +479,11 @@ Waterfall.prototype = {
    */
   _buildMarkerSpacer: function(container) {
     let sidebarSpacer = this._document.createElement("spacer");
-    sidebarSpacer.className = "timeline-marker-sidebar theme-sidebar";
-    sidebarSpacer.setAttribute("width", TIMELINE_MARKER_SIDEBAR_WIDTH);
+    sidebarSpacer.className = "waterfall-sidebar theme-sidebar";
+    sidebarSpacer.setAttribute("width", WATERFALL_SIDEBAR_WIDTH);
 
     let waterfallSpacer = this._document.createElement("spacer");
-    waterfallSpacer.className = "timeline-marker-waterfall";
+    waterfallSpacer.className = "waterfall-marker-item waterfall-background-ticks";
     waterfallSpacer.setAttribute("flex", "1");
 
     container.appendChild(sidebarSpacer);
