@@ -8,15 +8,18 @@
 #include "nsStructuredCloneContainer.h"
 
 #include "nsCOMPtr.h"
-#include "nsIScriptContext.h"
+#include "nsIGlobalObject.h"
 #include "nsIVariant.h"
 #include "nsIXPConnect.h"
 #include "nsServiceManagerUtils.h"
 #include "nsContentUtils.h"
 #include "jsapi.h"
+#include "jsfriendapi.h"
 #include "js/StructuredClone.h"
+#include "xpcpublic.h"
 
 #include "mozilla/Base64.h"
+#include "mozilla/dom/ScriptSettings.h"
 
 using namespace mozilla;
 
@@ -39,22 +42,31 @@ nsStructuredCloneContainer::~nsStructuredCloneContainer()
 }
 
 nsresult
-nsStructuredCloneContainer::InitFromJSVal(JS::Handle<JS::Value> aData,
-                                          JSContext* aCx)
+nsStructuredCloneContainer::InitFromJSVal(JS::Handle<JS::Value> aData)
 {
   NS_ENSURE_STATE(!mData);
-  NS_ENSURE_ARG_POINTER(aCx);
-
-  // Make sure that we serialize in the right context.
-  MOZ_ASSERT(aCx == nsContentUtils::GetCurrentJSContext());
-  JS::Rooted<JS::Value> jsData(aCx, aData);
-  bool success = JS_WrapValue(aCx, &jsData);
-  NS_ENSURE_STATE(success);
 
   uint64_t* jsBytes = nullptr;
-  success = JS_WriteStructuredClone(aCx, jsData, &jsBytes, &mSize,
-                                    nullptr, nullptr,
-                                    JS::UndefinedHandleValue);
+  bool success = false;
+  if (aData.isPrimitive()) {
+    // |aData| is a primitive, so the structured clone algorithm won't run
+    // script and we can just use AutoJSAPI.
+    dom::AutoJSAPI jsapi;
+    jsapi.Init();
+    success = JS_WriteStructuredClone(jsapi.cx(), aData, &jsBytes, &mSize,
+                                      nullptr, nullptr,
+                                      JS::UndefinedHandleValue);
+  } else {
+    // |aData| is an object and the structured clone algorithm can run script as
+    // part of the "own" "deep clone" sub-steps, so we need an AutoEntryScript.
+    // http://www.whatwg.org/specs/web-apps/current-work/#internal-structured-cloning-algorithm
+    nsIGlobalObject* nativeGlobal =
+      xpc::NativeGlobal(js::GetGlobalForObjectCrossCompartment(&aData.toObject()));
+    dom::AutoEntryScript aes(nativeGlobal);
+    success = JS_WriteStructuredClone(aes.cx(), aData, &jsBytes, &mSize,
+                                      nullptr, nullptr,
+                                      JS::UndefinedHandleValue);
+  }
   NS_ENSURE_STATE(success);
   NS_ENSURE_STATE(jsBytes);
 
@@ -64,7 +76,7 @@ nsStructuredCloneContainer::InitFromJSVal(JS::Handle<JS::Value> aData,
     mSize = 0;
     mVersion = 0;
 
-    JS_ClearStructuredClone(jsBytes, mSize);
+    JS_ClearStructuredClone(jsBytes, mSize, nullptr, nullptr);
     return NS_ERROR_FAILURE;
   }
   else {
@@ -73,7 +85,7 @@ nsStructuredCloneContainer::InitFromJSVal(JS::Handle<JS::Value> aData,
 
   memcpy(mData, jsBytes, mSize);
 
-  JS_ClearStructuredClone(jsBytes, mSize);
+  JS_ClearStructuredClone(jsBytes, mSize, nullptr, nullptr);
   return NS_OK;
 }
 
