@@ -121,7 +121,13 @@ protected:
     void get(const char* aKey, double& aRet)      { aRet = getFloat(aKey); }
     void get(const char* aKey, const char*& aRet) { aRet = get(aKey); }
     void get(const char* aKey, int& aRet)         { aRet = getInt(aKey); }
-    void get(const char* aKey, bool& aRet)        { aRet = strcmp(get(aKey), FALSE); }
+
+    void
+    get(const char* aKey, bool& aRet)
+    {
+      const char* value = get(aKey);
+      aRet = value ? strcmp(value, TRUE) == 0 : false;
+    }
 
     void remove(const char* aKey)                 { android::CameraParameters::remove(aKey); }
 
