@@ -16,6 +16,7 @@
 #include "nsIDOMEventListener.h"
 #include "nsTObserverArray.h"
 
+class nsIDocShell;
 class nsIDOMEvent;
 class nsIEventListenerInfo;
 class nsIScriptContext;
@@ -151,6 +152,8 @@ inline EventListenerFlags AllEventsAtSystemGroupCapture()
 
 class EventListenerManager MOZ_FINAL
 {
+  ~EventListenerManager();
+
 public:
   struct Listener
   {
@@ -209,8 +212,7 @@ public:
     }
   };
 
-  EventListenerManager(dom::EventTarget* aTarget);
-  virtual ~EventListenerManager();
+  explicit EventListenerManager(dom::EventTarget* aTarget);
 
   NS_INLINE_DECL_CYCLE_COLLECTING_NATIVE_REFCOUNTING(EventListenerManager)
 
@@ -419,6 +421,8 @@ protected:
                               nsIDOMEvent* aDOMEvent,
                               dom::EventTarget* aCurrentTarget);
 
+  nsIDocShell* GetDocShellForTarget();
+
   /**
    * Compile the "inline" event listener for aListener.  The
    * body of the listener can be provided in aBody; if this is null we
