@@ -22,10 +22,9 @@ public:
                nsPresContext* aPresContext,
                WidgetPointerEvent* aEvent);
 
-  virtual JSObject* WrapObject(JSContext* aCx,
-                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE
+  virtual JSObject* WrapObjectInternal(JSContext* aCx) MOZ_OVERRIDE
   {
-    return PointerEventBinding::Wrap(aCx, aScope, this);
+    return PointerEventBinding::Wrap(aCx, this);
   }
 
   static already_AddRefed<PointerEvent>
@@ -34,6 +33,11 @@ public:
               const PointerEventInit& aParam,
               ErrorResult& aRv);
 
+  static already_AddRefed<PointerEvent>
+  Constructor(EventTarget* aOwner,
+              const nsAString& aType,
+              const PointerEventInit& aParam);
+
   int32_t PointerId();
   int32_t Width();
   int32_t Height();
@@ -44,6 +48,8 @@ public:
   void GetPointerType(nsAString& aPointerType);
 };
 
+void ConvertPointerTypeToString(uint16_t aPointerTypeSrc, nsAString& aPointerTypeDest);
+
 } // namespace dom
 } // namespace mozilla
 
