@@ -49,6 +49,7 @@ if (!window.runTest) {
   {
     SimpleTest.waitForExplicitFinish();
 
+    allowIndexedDB();
     if (limitedQuota) {
       denyUnlimitedQuota();
     }
@@ -66,10 +67,11 @@ if (!window.runTest) {
 
 function finishTest()
 {
-  resetUnlimitedQuota();
-  resetExperimental();
   resetArchiveReader();
+  resetExperimental();
   resetTesting();
+  resetUnlimitedQuota();
+  resetIndexedDB();
   SpecialPowers.notifyObserversInParentProcess(null, "disk-space-watcher",
                                                "free");
 
@@ -197,6 +199,16 @@ function removePermission(type, url)
   SpecialPowers.removePermission(type, url);
 }
 
+function allowIndexedDB(url)
+{
+  addPermission("indexedDB", true, url);
+}
+
+function resetIndexedDB(url)
+{
+  removePermission("indexedDB", url);
+}
+
 function allowUnlimitedQuota(url)
 {
   addPermission("indexedDB-unlimited", true, url);
