@@ -21,7 +21,6 @@
 #include "ArrayCluster.h"
 #include "Client.h"
 #include "PersistenceType.h"
-#include "StoragePrivilege.h"
 
 #define QUOTA_MANAGER_CONTRACTID "@mozilla.org/dom/quota/manager;1"
 
@@ -47,7 +46,7 @@ class CheckQuotaHelper;
 class CollectOriginsHelper;
 class FinalizeOriginEvictionRunnable;
 class GroupInfo;
-class GroupInfoPair;
+class GroupInfoTriple;
 class OriginClearRunnable;
 class OriginInfo;
 class OriginOrPatternString;
@@ -58,13 +57,16 @@ struct SynchronizedOp;
 struct OriginParams
 {
   OriginParams(PersistenceType aPersistenceType,
-               const nsACString& aOrigin)
+               const nsACString& aOrigin,
+               bool aIsApp)
   : mOrigin(aOrigin)
   , mPersistenceType(aPersistenceType)
+  , mIsApp(aIsApp)
   { }
 
   nsCString mOrigin;
   PersistenceType mPersistenceType;
+  bool mIsApp;
 };
 
 class QuotaManager MOZ_FINAL : public nsIQuotaManager,
@@ -149,10 +151,6 @@ public:
     LockedRemoveQuotaForOrigin(aPersistenceType, aGroup, aOrigin);
   }
 
-  void
-  RemoveQuotaForPattern(PersistenceType aPersistenceType,
-                        const nsACString& aPattern);
-
   already_AddRefed<QuotaObject>
   GetQuotaObject(PersistenceType aPersistenceType,
                  const nsACString& aGroup,
@@ -275,7 +273,8 @@ public:
 
   void
   OriginClearCompleted(PersistenceType aPersistenceType,
-                       const OriginOrPatternString& aOriginOrPattern);
+                       const nsACString& aOrigin,
+                       bool aIsApp);
 
   void
   ResetOrClearCompleted();
@@ -306,12 +305,16 @@ public:
   GetStoragePath(PersistenceType aPersistenceType) const
   {
     if (aPersistenceType == PERSISTENCE_TYPE_PERSISTENT) {
-      return mPersistentStoragePath;
+      return mPermanentStoragePath;
     }
 
-    NS_ASSERTION(aPersistenceType == PERSISTENCE_TYPE_TEMPORARY, "Huh?");
+    if (aPersistenceType == PERSISTENCE_TYPE_TEMPORARY) {
+      return mTemporaryStoragePath;
+    }
 
-    return mTemporaryStoragePath;
+    MOZ_ASSERT(aPersistenceType == PERSISTENCE_TYPE_DEFAULT);
+
+    return mDefaultStoragePath;
   }
 
   uint64_t
@@ -331,51 +334,50 @@ public:
   GetInfoFromURI(nsIURI* aURI,
                  uint32_t aAppId,
                  bool aInMozBrowser,
-                 PersistenceType aPersistenceType,
                  nsACString* aGroup,
                  nsACString* aOrigin,
-                 StoragePrivilege* aPrivilege,
                  bool* aIsApp,
                  bool* aHasUnlimStoragePerm);
 
   static nsresult
   GetInfoFromPrincipal(nsIPrincipal* aPrincipal,
-                       PersistenceType aPersistenceType,
                        nsACString* aGroup,
                        nsACString* aOrigin,
-                       StoragePrivilege* aPrivilege,
                        bool* aIsApp,
                        bool* aHasUnlimStoragePerm);
 
   static nsresult
   GetInfoFromWindow(nsPIDOMWindow* aWindow,
-                    PersistenceType aPersistenceType,
                     nsACString* aGroup,
                     nsACString* aOrigin,
-                    StoragePrivilege* aPrivilege,
                     bool* aIsApp,
                     bool* aHasUnlimStoragePerm);
 
   static void
   GetInfoForChrome(nsACString* aGroup,
                    nsACString* aOrigin,
-                   StoragePrivilege* aPrivilege,
                    bool* aIsApp,
                    bool* aHasUnlimStoragePerm);
 
+  static bool
+  IsOriginWhitelistedForPersistentStorage(const nsACString& aOrigin);
+
   static bool
   IsTreatedAsPersistent(PersistenceType aPersistenceType,
-                        const nsACString& aOrigin,
                         bool aIsApp);
 
   static bool
   IsTreatedAsTemporary(PersistenceType aPersistenceType,
-                       const nsACString& aOrigin,
                        bool aIsApp)
   {
-    return !IsTreatedAsPersistent(aPersistenceType, aOrigin, aIsApp);
+    return !IsTreatedAsPersistent(aPersistenceType, aIsApp);
   }
 
+  static bool
+  IsFirstPromptRequired(PersistenceType aPersistenceType,
+                        const nsACString& aOrigin,
+                        bool aIsApp);
+
   static bool
   IsQuotaEnforced(PersistenceType aPersistenceType,
                   const nsACString& aOrigin,
@@ -455,6 +457,12 @@ private:
   nsresult
   MaybeUpgradeIndexedDBDirectory();
 
+  nsresult
+  MaybeUpgradePersistentStorageDirectory();
+
+  nsresult
+  MaybeUpgradeStorageArea();
+
   nsresult
   InitializeRepository(PersistenceType aPersistenceType);
 
@@ -515,27 +523,22 @@ private:
 
   static PLDHashOperator
   RemoveQuotaForTemporaryStorageCallback(const nsACString& aKey,
-                                         nsAutoPtr<GroupInfoPair>& aValue,
+                                         nsAutoPtr<GroupInfoTriple>& aValue,
                                          void* aUserArg);
 
   static PLDHashOperator
   RemoveQuotaCallback(const nsACString& aKey,
-                      nsAutoPtr<GroupInfoPair>& aValue,
+                      nsAutoPtr<GroupInfoTriple>& aValue,
                       void* aUserArg);
 
-  static PLDHashOperator
-  RemoveQuotaForPatternCallback(const nsACString& aKey,
-                                nsAutoPtr<GroupInfoPair>& aValue,
-                                void* aUserArg);
-
   static PLDHashOperator
   GetOriginsExceedingGroupLimit(const nsACString& aKey,
-                                GroupInfoPair* aValue,
+                                GroupInfoTriple* aValue,
                                 void* aUserArg);
 
   static PLDHashOperator
   GetAllTemporaryStorageOrigins(const nsACString& aKey,
-                                GroupInfoPair* aValue,
+                                GroupInfoTriple* aValue,
                                 void* aUserArg);
 
   static PLDHashOperator
@@ -545,7 +548,7 @@ private:
 
   static PLDHashOperator
   GetInactiveTemporaryStorageOrigins(const nsACString& aKey,
-                                     GroupInfoPair* aValue,
+                                     GroupInfoTriple* aValue,
                                      void* aUserArg);
 
   // TLS storage index for the current thread's window.
@@ -553,7 +556,7 @@ private:
 
   mozilla::Mutex mQuotaMutex;
 
-  nsClassHashtable<nsCStringHashKey, GroupInfoPair> mGroupInfoPairs;
+  nsClassHashtable<nsCStringHashKey, GroupInfoTriple> mGroupInfoTriples;
 
   // A map of Windows to the corresponding quota helper.
   nsRefPtrHashtable<nsPtrHashKey<nsPIDOMWindow>,
@@ -565,6 +568,7 @@ private:
 
   LiveStorageTable mPersistentLiveStorageTable;
   LiveStorageTable mTemporaryLiveStorageTable;
+  LiveStorageTable mDefaultLiveStorageTable;
 
   // Maintains a list of synchronized operatons that are in progress or queued.
   nsAutoTArray<nsAutoPtr<SynchronizedOp>, 5> mSynchronizedOps;
@@ -583,8 +587,9 @@ private:
 
   nsString mIndexedDBPath;
   nsString mStoragePath;
-  nsString mPersistentStoragePath;
+  nsString mPermanentStoragePath;
   nsString mTemporaryStoragePath;
+  nsString mDefaultStoragePath;
 
   uint64_t mTemporaryStorageLimit;
   uint64_t mTemporaryStorageUsage;
