@@ -77,8 +77,6 @@ public:
     HRESULT hr = mDT->mDevice->CreateTexture2D(&desc, nullptr, byRef(tmpTexture));
     if (FAILED(hr)) {
       gfxCriticalError() << "[D2D] CreateTexture2D failure " << size << " Code: " << hexa(hr);
-      // Crash debug builds but try to recover in release builds.
-      MOZ_ASSERT(false);
       return;
     }
     mDT->mDevice->CopyResource(tmpTexture, mDT->mTexture);
@@ -94,8 +92,6 @@ public:
 
     if (FAILED(hr)) {
       gfxCriticalError() << "[D2D] CreateSharedBitmap failure " << size << " Code: " << hexa(hr);
-      // Crash debug builds but try to recover in release builds.
-      MOZ_ASSERT(false);
       return;
     }
 
@@ -1386,7 +1382,7 @@ DrawTargetD2D::Init(ID3D10Texture2D *aTexture, SurfaceFormat aFormat)
   mFormat = aFormat;
 
   if (!mTexture) {
-    gfxDebug() << "No valid texture for Direct2D draw target initialization.";
+    gfxCriticalError() << "No valid texture for Direct2D draw target initialization.";
     return false;
   }
 
@@ -2375,7 +2371,8 @@ DrawTargetD2D::CreateBrushForPattern(const Pattern &aPattern, Float aAlpha)
       RefPtr<DrawTargetD2D> dt = new DrawTargetD2D();
       if (!dt->Init(samplingRect.Size(),
                     source->GetFormat())) {
-        MOZ_ASSERT("Invalid sampling rect size!");
+        // FIXME: Uncomment assertion, bug 1068195
+        // MOZ_ASSERT(false, "Invalid sampling rect size!");
         return nullptr;
       }
 
@@ -2420,7 +2417,9 @@ DrawTargetD2D::CreateBrushForPattern(const Pattern &aPattern, Float aAlpha)
 
         bitmap = CreatePartialBitmapForSurface(dataSurf, mTransform, mSize, pat->mExtendMode, mat, mRT, &sourceRect);
         if (!bitmap) {
-          return nullptr;
+          RefPtr<ID2D1SolidColorBrush> colBrush;
+          mRT->CreateSolidColorBrush(D2D1::ColorF(0, 0), byRef(colBrush));
+          return colBrush.forget();
         }
       }
       break;
