@@ -3,22 +3,30 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <new>      // Workaround for bug in VS10; see bug 981264.
+#include <cmath>
+#include <utility>
+
 #include "gfxDrawable.h"
 #include "gfxPlatform.h"
 #include "gfxUtils.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/RefPtr.h"
 
-#include "ClippedImage.h"
+#include "ImageRegion.h"
 #include "Orientation.h"
 #include "SVGImageContext.h"
 
+#include "ClippedImage.h"
 
 namespace mozilla {
 
 using namespace gfx;
 using layers::LayerManager;
 using layers::ImageContainer;
+using std::make_pair;
+using std::modf;
+using std::pair;
 
 namespace image {
 
@@ -26,30 +34,28 @@ class ClippedImageCachedSurface
 {
 public:
   ClippedImageCachedSurface(TemporaryRef<SourceSurface> aSurface,
-                            const nsIntSize& aViewportSize,
-                            const SVGImageContext* aSVGContext,
+                            const nsIntSize& aSize,
+                            const Maybe<SVGImageContext>& aSVGContext,
                             float aFrame,
                             uint32_t aFlags)
     : mSurface(aSurface)
-    , mViewportSize(aViewportSize)
+    , mSize(aSize)
     , mFrame(aFrame)
     , mFlags(aFlags)
   {
     MOZ_ASSERT(mSurface, "Must have a valid surface");
     if (aSVGContext) {
-      mSVGContext.construct(*aSVGContext);
+      mSVGContext.emplace(*aSVGContext);
     }
   }
 
-  bool Matches(const nsIntSize& aViewportSize,
-               const SVGImageContext* aSVGContext,
+  bool Matches(const nsIntSize& aSize,
+               const Maybe<SVGImageContext>& aSVGContext,
                float aFrame,
                uint32_t aFlags)
   {
-    bool matchesSVGContext = (!aSVGContext && mSVGContext.empty()) ||
-                             *aSVGContext == mSVGContext.ref();
-    return mViewportSize == aViewportSize &&
-           matchesSVGContext &&
+    return mSize == aSize &&
+           mSVGContext == aSVGContext &&
            mFrame == aFrame &&
            mFlags == aFlags;
   }
@@ -59,25 +65,23 @@ public:
   }
 
 private:
-  RefPtr<SourceSurface>              mSurface;
-  const nsIntSize                    mViewportSize;
-  Maybe<SVGImageContext>             mSVGContext;
-  const float                        mFrame;
-  const uint32_t                     mFlags;
+  RefPtr<SourceSurface>  mSurface;
+  const nsIntSize        mSize;
+  Maybe<SVGImageContext> mSVGContext;
+  const float            mFrame;
+  const uint32_t         mFlags;
 };
 
 class DrawSingleTileCallback : public gfxDrawingCallback
 {
 public:
   DrawSingleTileCallback(ClippedImage* aImage,
-                         const nsIntRect& aClip,
-                         const nsIntSize& aViewportSize,
-                         const SVGImageContext* aSVGContext,
+                         const nsIntSize& aSize,
+                         const Maybe<SVGImageContext>& aSVGContext,
                          uint32_t aWhichFrame,
                          uint32_t aFlags)
     : mImage(aImage)
-    , mClip(aClip)
-    , mViewportSize(aViewportSize)
+    , mSize(aSize)
     , mSVGContext(aSVGContext)
     , mWhichFrame(aWhichFrame)
     , mFlags(aFlags)
@@ -90,21 +94,24 @@ public:
                           const GraphicsFilter& aFilter,
                           const gfxMatrix& aTransform)
   {
+    MOZ_ASSERT(aTransform.IsIdentity(),
+               "Caller is probably CreateSamplingRestrictedDrawable, "
+               "which should not happen");
+
     // Draw the image. |gfxCallbackDrawable| always calls this function with
     // arguments that guarantee we never tile.
-    mImage->DrawSingleTile(aContext, aFilter, aTransform, aFillRect, mClip,
-                           mViewportSize, mSVGContext, mWhichFrame, mFlags);
+    mImage->DrawSingleTile(aContext, mSize, ImageRegion::Create(aFillRect),
+                           mWhichFrame, aFilter, mSVGContext, mFlags);
 
     return true;
   }
 
 private:
-  nsRefPtr<ClippedImage> mImage;
-  const nsIntRect        mClip;
-  const nsIntSize        mViewportSize;
-  const SVGImageContext* mSVGContext;
-  const uint32_t         mWhichFrame;
-  const uint32_t         mFlags;
+  nsRefPtr<ClippedImage>        mImage;
+  const nsIntSize               mSize;
+  const Maybe<SVGImageContext>& mSVGContext;
+  const uint32_t                mWhichFrame;
+  const uint32_t                mFlags;
 };
 
 ClippedImage::ClippedImage(Image* aImage,
@@ -125,13 +132,13 @@ ClippedImage::ShouldClip()
   // once they're available to determine if it's valid and whether we actually
   // need to do any work. We may fail if the image's width and height aren't
   // available yet, in which case we'll try again later.
-  if (mShouldClip.empty()) {
+  if (mShouldClip.isNothing()) {
     int32_t width, height;
-    nsRefPtr<imgStatusTracker> innerImageStatusTracker =
-      InnerImage()->GetStatusTracker();
+    nsRefPtr<ProgressTracker> progressTracker =
+      InnerImage()->GetProgressTracker();
     if (InnerImage()->HasError()) {
       // If there's a problem with the inner image we'll let it handle everything.
-      mShouldClip.construct(false);
+      mShouldClip.emplace(false);
     } else if (NS_SUCCEEDED(InnerImage()->GetWidth(&width)) && width > 0 &&
                NS_SUCCEEDED(InnerImage()->GetHeight(&height)) && height > 0) {
       // Clamp the clipping region to the size of the underlying image.
@@ -139,9 +146,9 @@ ClippedImage::ShouldClip()
 
       // If the clipping region is the same size as the underlying image we
       // don't have to do anything.
-      mShouldClip.construct(!mClip.IsEqualInterior(nsIntRect(0, 0, width, height)));
-    } else if (innerImageStatusTracker &&
-               innerImageStatusTracker->IsLoading()) {
+      mShouldClip.emplace(!mClip.IsEqualInterior(nsIntRect(0, 0, width, height)));
+    } else if (progressTracker &&
+               !(progressTracker->GetProgress() & FLAG_LOAD_COMPLETE)) {
       // The image just hasn't finished loading yet. We don't yet know whether
       // clipping with be needed or not for now. Just return without memoizing
       // anything.
@@ -149,12 +156,12 @@ ClippedImage::ShouldClip()
     } else {
       // We have a fully loaded image without a clearly defined width and
       // height. This can happen with SVG images.
-      mShouldClip.construct(false);
+      mShouldClip.emplace(false);
     }
   }
 
-  MOZ_ASSERT(!mShouldClip.empty(), "Should have computed a result");
-  return mShouldClip.ref();
+  MOZ_ASSERT(mShouldClip.isSome(), "Should have computed a result");
+  return *mShouldClip;
 }
 
 NS_IMPL_ISUPPORTS_INHERITED0(ClippedImage, ImageWrapper)
@@ -217,12 +224,12 @@ NS_IMETHODIMP_(TemporaryRef<SourceSurface>)
 ClippedImage::GetFrame(uint32_t aWhichFrame,
                        uint32_t aFlags)
 {
-  return GetFrameInternal(mClip.Size(), nullptr, aWhichFrame, aFlags);
+  return GetFrameInternal(mClip.Size(), Nothing(), aWhichFrame, aFlags);
 }
 
 TemporaryRef<SourceSurface>
-ClippedImage::GetFrameInternal(const nsIntSize& aViewportSize,
-                               const SVGImageContext* aSVGContext,
+ClippedImage::GetFrameInternal(const nsIntSize& aSize,
+                               const Maybe<SVGImageContext>& aSVGContext,
                                uint32_t aWhichFrame,
                                uint32_t aFlags)
 {
@@ -231,13 +238,13 @@ ClippedImage::GetFrameInternal(const nsIntSize& aViewportSize,
   }
 
   float frameToDraw = InnerImage()->GetFrameIndex(aWhichFrame);
-  if (!mCachedSurface || !mCachedSurface->Matches(aViewportSize,
+  if (!mCachedSurface || !mCachedSurface->Matches(aSize,
                                                   aSVGContext,
                                                   frameToDraw,
                                                   aFlags)) {
     // Create a surface to draw into.
     RefPtr<DrawTarget> target = gfxPlatform::GetPlatform()->
-      CreateOffscreenContentDrawTarget(IntSize(mClip.width, mClip.height),
+      CreateOffscreenContentDrawTarget(IntSize(aSize.width, aSize.height),
                                        SurfaceFormat::B8G8R8A8);
     if (!target) {
       NS_ERROR("Could not create a DrawTarget");
@@ -248,22 +255,20 @@ ClippedImage::GetFrameInternal(const nsIntSize& aViewportSize,
 
     // Create our callback.
     nsRefPtr<gfxDrawingCallback> drawTileCallback =
-      new DrawSingleTileCallback(this, mClip, aViewportSize, aSVGContext, aWhichFrame, aFlags);
+      new DrawSingleTileCallback(this, aSize, aSVGContext, aWhichFrame, aFlags);
     nsRefPtr<gfxDrawable> drawable =
-      new gfxCallbackDrawable(drawTileCallback, mClip.Size());
+      new gfxCallbackDrawable(drawTileCallback, aSize);
 
     // Actually draw. The callback will end up invoking DrawSingleTile.
-    gfxRect imageRect(0, 0, mClip.width, mClip.height);
-    gfxUtils::DrawPixelSnapped(ctx, drawable, gfxMatrix(),
-                               imageRect, imageRect, imageRect, imageRect,
+    gfxUtils::DrawPixelSnapped(ctx, drawable, aSize,
+                               ImageRegion::Create(aSize),
                                SurfaceFormat::B8G8R8A8,
-                               GraphicsFilter::FILTER_FAST);
+                               GraphicsFilter::FILTER_FAST,
+                               imgIContainer::FLAG_CLAMP);
 
     // Cache the resulting surface.
-    mCachedSurface = new ClippedImageCachedSurface(target->Snapshot(),
-                                                   aViewportSize,
-                                                   aSVGContext,
-                                                   frameToDraw,
+    mCachedSurface = new ClippedImageCachedSurface(target->Snapshot(), aSize,
+                                                   aSVGContext, frameToDraw,
                                                    aFlags);
   }
 
@@ -288,123 +293,116 @@ ClippedImage::GetImageContainer(LayerManager* aManager, ImageContainer** _retval
   return NS_OK;
 }
 
-bool
-ClippedImage::MustCreateSurface(gfxContext* aContext,
-                                const gfxMatrix& aTransform,
-                                const gfxRect& aSourceRect,
-                                const nsIntRect& aSubimage,
-                                const uint32_t aFlags) const
+static bool
+MustCreateSurface(gfxContext* aContext,
+                  const nsIntSize& aSize,
+                  const ImageRegion& aRegion,
+                  const uint32_t aFlags)
 {
-  gfxRect gfxImageRect(0, 0, mClip.width, mClip.height);
-  nsIntRect intImageRect(0, 0, mClip.width, mClip.height);
-  bool willTile = !gfxImageRect.Contains(aSourceRect) &&
+  gfxRect imageRect(0, 0, aSize.width, aSize.height);
+  bool willTile = !imageRect.Contains(aRegion.Rect()) &&
                   !(aFlags & imgIContainer::FLAG_CLAMP);
-  bool willResample = (aContext->CurrentMatrix().HasNonIntegerTranslation() ||
-                       aTransform.HasNonIntegerTranslation()) &&
-                      (willTile || !aSubimage.Contains(intImageRect));
+  bool willResample = aContext->CurrentMatrix().HasNonIntegerTranslation() &&
+                      (willTile || !aRegion.RestrictionContains(imageRect));
   return willTile || willResample;
 }
 
 NS_IMETHODIMP
 ClippedImage::Draw(gfxContext* aContext,
-                   GraphicsFilter aFilter,
-                   const gfxMatrix& aUserSpaceToImageSpace,
-                   const gfxRect& aFill,
-                   const nsIntRect& aSubimage,
-                   const nsIntSize& aViewportSize,
-                   const SVGImageContext* aSVGContext,
+                   const nsIntSize& aSize,
+                   const ImageRegion& aRegion,
                    uint32_t aWhichFrame,
+                   GraphicsFilter aFilter,
+                   const Maybe<SVGImageContext>& aSVGContext,
                    uint32_t aFlags)
 {
   if (!ShouldClip()) {
-    return InnerImage()->Draw(aContext, aFilter, aUserSpaceToImageSpace,
-                              aFill, aSubimage, aViewportSize, aSVGContext,
-                              aWhichFrame, aFlags);
+    return InnerImage()->Draw(aContext, aSize, aRegion, aWhichFrame,
+                              aFilter, aSVGContext, aFlags);
   }
 
   // Check for tiling. If we need to tile then we need to create a
   // gfxCallbackDrawable to handle drawing for us.
-  gfxRect sourceRect = aUserSpaceToImageSpace.Transform(aFill);
-  if (MustCreateSurface(aContext, aUserSpaceToImageSpace, sourceRect, aSubimage, aFlags)) {
+  if (MustCreateSurface(aContext, aSize, aRegion, aFlags)) {
     // Create a temporary surface containing a single tile of this image.
     // GetFrame will call DrawSingleTile internally.
     RefPtr<SourceSurface> surface =
-      GetFrameInternal(aViewportSize, aSVGContext, aWhichFrame, aFlags);
+      GetFrameInternal(aSize, aSVGContext, aWhichFrame, aFlags);
     NS_ENSURE_TRUE(surface, NS_ERROR_FAILURE);
 
     // Create a drawable from that surface.
     nsRefPtr<gfxSurfaceDrawable> drawable =
-      new gfxSurfaceDrawable(surface, gfxIntSize(mClip.width, mClip.height));
+      new gfxSurfaceDrawable(surface, aSize);
 
     // Draw.
-    gfxRect imageRect(0, 0, mClip.width, mClip.height);
-    gfxRect subimage(aSubimage.x, aSubimage.y, aSubimage.width, aSubimage.height);
-    gfxUtils::DrawPixelSnapped(aContext, drawable, aUserSpaceToImageSpace,
-                               subimage, sourceRect, imageRect, aFill,
+    gfxUtils::DrawPixelSnapped(aContext, drawable, aSize, aRegion,
                                SurfaceFormat::B8G8R8A8, aFilter);
 
     return NS_OK;
   }
 
-  // Determine the appropriate subimage for the inner image.
-  nsIntRect innerSubimage(aSubimage);
-  innerSubimage.MoveBy(mClip.x, mClip.y);
-  innerSubimage.Intersect(mClip);
-
-  return DrawSingleTile(aContext, aFilter, aUserSpaceToImageSpace, aFill, innerSubimage,
-                        aViewportSize, aSVGContext, aWhichFrame, aFlags);
+  return DrawSingleTile(aContext, aSize, aRegion, aWhichFrame,
+                        aFilter, aSVGContext, aFlags);
 }
 
-gfxFloat
-ClippedImage::ClampFactor(const gfxFloat aToClamp, const int aReference) const
+static SVGImageContext
+UnclipViewport(const SVGImageContext& aOldContext,
+               const pair<nsIntSize, nsIntSize>& aInnerAndClipSize)
 {
-  return aToClamp > aReference ? aReference / aToClamp
-                               : 1.0;
+  nsIntSize innerSize(aInnerAndClipSize.first);
+  nsIntSize clipSize(aInnerAndClipSize.second);
+
+  // Map the viewport to the inner image. (Note that we don't take the aSize
+  // parameter of Draw into account, just the clipping region.)
+  nsIntSize vSize(aOldContext.GetViewportSize());
+  vSize.width = ceil(vSize.width * double(innerSize.width) / clipSize.width);
+  vSize.height = ceil(vSize.height * double(innerSize.height) / clipSize.height);
+
+  return SVGImageContext(vSize,
+                         aOldContext.GetPreserveAspectRatio());
 }
 
 nsresult
 ClippedImage::DrawSingleTile(gfxContext* aContext,
-                             GraphicsFilter aFilter,
-                             const gfxMatrix& aUserSpaceToImageSpace,
-                             const gfxRect& aFill,
-                             const nsIntRect& aSubimage,
-                             const nsIntSize& aViewportSize,
-                             const SVGImageContext* aSVGContext,
+                             const nsIntSize& aSize,
+                             const ImageRegion& aRegion,
                              uint32_t aWhichFrame,
+                             GraphicsFilter aFilter,
+                             const Maybe<SVGImageContext>& aSVGContext,
                              uint32_t aFlags)
 {
-  MOZ_ASSERT(!MustCreateSurface(aContext, aUserSpaceToImageSpace,
-                                aUserSpaceToImageSpace.Transform(aFill),
-                                aSubimage - nsIntPoint(mClip.x, mClip.y), aFlags),
-             "DrawSingleTile shouldn't need to create a surface");
+  MOZ_ASSERT(!MustCreateSurface(aContext, aSize, aRegion, aFlags),
+             "Shouldn't need to create a surface");
 
-  // Make the viewport reflect the original image's size.
-  nsIntSize viewportSize(aViewportSize);
-  int32_t imgWidth, imgHeight;
-  if (NS_SUCCEEDED(InnerImage()->GetWidth(&imgWidth)) &&
-      NS_SUCCEEDED(InnerImage()->GetHeight(&imgHeight))) {
-    viewportSize = nsIntSize(imgWidth, imgHeight);
+  gfxRect clip(mClip.x, mClip.y, mClip.width, mClip.height);
+  nsIntSize size(aSize), innerSize(aSize);
+  if (NS_SUCCEEDED(InnerImage()->GetWidth(&innerSize.width)) &&
+      NS_SUCCEEDED(InnerImage()->GetHeight(&innerSize.height))) {
+    double scaleX = aSize.width / clip.width;
+    double scaleY = aSize.height / clip.height;
+    
+    // Map the clip and size to the scale requested by the caller.
+    clip.Scale(scaleX, scaleY);
+    size = innerSize;
+    size.Scale(scaleX, scaleY);
   } else {
     MOZ_ASSERT(false, "If ShouldClip() led us to draw then we should never get here");
   }
 
-  // Add a translation to the transform to reflect the clipping region.
-  gfxMatrix transform =
-    aUserSpaceToImageSpace * gfxMatrix::Translation(mClip.x, mClip.y);
+  // We restrict our drawing to only the clipping region, and translate so that
+  // the clipping region is placed at the position the caller expects.
+  ImageRegion region(aRegion);
+  region.MoveBy(clip.x, clip.y);
+  region = region.Intersect(clip);
 
-  // "Clamp the source rectangle" to the clipping region's width and height.
-  // Really, this means modifying the transform to get the results we want.
-  gfxRect sourceRect = transform.Transform(aFill);
-  if (sourceRect.width > mClip.width || sourceRect.height > mClip.height) {
-    gfxMatrix clampSource = gfxMatrix::Translation(sourceRect.TopLeft());
-    clampSource.Scale(ClampFactor(sourceRect.width, mClip.width),
-                      ClampFactor(sourceRect.height, mClip.height));
-    clampSource.Translate(-sourceRect.TopLeft());
-    transform *= clampSource;
-  }
+  gfxContextMatrixAutoSaveRestore saveMatrix(aContext);
+  aContext->Multiply(gfxMatrix::Translation(-clip.x, -clip.y));
 
-  return InnerImage()->Draw(aContext, aFilter, transform, aFill, aSubimage,
-                            viewportSize, aSVGContext, aWhichFrame, aFlags);
+  return InnerImage()->Draw(aContext, size, region,
+                            aWhichFrame, aFilter,
+                            aSVGContext.map(UnclipViewport,
+                                            make_pair(innerSize, mClip.Size())),
+                            aFlags);
 }
 
 NS_IMETHODIMP
@@ -424,6 +422,44 @@ ClippedImage::GetOrientation()
   return InnerImage()->GetOrientation();
 }
 
+nsIntSize
+ClippedImage::OptimalImageSizeForDest(const gfxSize& aDest, uint32_t aWhichFrame,
+                                      GraphicsFilter aFilter, uint32_t aFlags)
+{
+  if (!ShouldClip()) {
+    return InnerImage()->OptimalImageSizeForDest(aDest, aWhichFrame, aFilter, aFlags);
+  }
+
+  int32_t imgWidth, imgHeight;
+  if (NS_SUCCEEDED(InnerImage()->GetWidth(&imgWidth)) &&
+      NS_SUCCEEDED(InnerImage()->GetHeight(&imgHeight))) {
+    // To avoid ugly sampling artifacts, ClippedImage needs the image size to
+    // be chosen such that the clipping region lies on pixel boundaries.
+
+    // First, we select a scale that's good for ClippedImage. An integer multiple
+    // of the size of the clipping region is always fine.
+    nsIntSize scale(ceil(aDest.width / mClip.width),
+                    ceil(aDest.height / mClip.height));
+
+    // Determine the size we'd prefer to render the inner image at, and ask the
+    // inner image what size we should actually use.
+    gfxSize desiredSize(imgWidth * scale.width, imgHeight * scale.height);
+    nsIntSize innerDesiredSize =
+      InnerImage()->OptimalImageSizeForDest(desiredSize, aWhichFrame,
+                                            aFilter, aFlags);
+
+    // To get our final result, we take the inner image's desired size and
+    // determine how large the clipped region would be at that scale. (Again, we
+    // ensure an integer multiple of the size of the clipping region.)
+    nsIntSize finalScale(ceil(double(innerDesiredSize.width) / imgWidth),
+                         ceil(double(innerDesiredSize.height) / imgHeight));
+    return mClip.Size() * finalScale;
+  } else {
+    MOZ_ASSERT(false, "If ShouldClip() led us to draw then we should never get here");
+    return InnerImage()->OptimalImageSizeForDest(aDest, aWhichFrame, aFilter, aFlags);
+  }
+}
+
 NS_IMETHODIMP_(nsIntRect)
 ClippedImage::GetImageSpaceInvalidationRect(const nsIntRect& aRect)
 {
