@@ -7,8 +7,10 @@
 #define MOZILLA_IMAGELIB_IMAGE_H_
 
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/TimeStamp.h"
+#include "gfx2DGlue.h"                // for gfxMemoryLocation
 #include "imgIContainer.h"
-#include "imgStatusTracker.h"
+#include "ProgressTracker.h"
 #include "ImageURL.h"
 #include "nsStringFwd.h"
 
@@ -45,14 +47,16 @@ public:
    * INIT_FLAG_DECODE_ON_DRAW: The container should decode on draw rather than
    * decoding on load.
    *
-   * INIT_FLAG_MULTIPART: The container will be used to display a stream of
-   * images in a multipart channel. If this flag is set, INIT_FLAG_DISCARDABLE
-   * and INIT_FLAG_DECODE_ON_DRAW must not be set.
+   * INIT_FLAG_TRANSIENT: The container is likely to exist for only a short time
+   * before being destroyed. (For example, containers for
+   * multipart/x-mixed-replace image parts fall into this category.) If this
+   * flag is set, INIT_FLAG_DISCARDABLE and INIT_FLAG_DECODE_ON_DRAW must not be
+   * set.
    */
   static const uint32_t INIT_FLAG_NONE           = 0x0;
   static const uint32_t INIT_FLAG_DISCARDABLE    = 0x1;
   static const uint32_t INIT_FLAG_DECODE_ON_DRAW = 0x2;
-  static const uint32_t INIT_FLAG_MULTIPART      = 0x4;
+  static const uint32_t INIT_FLAG_TRANSIENT      = 0x4;
 
   /**
    * Creates a new image container.
@@ -63,8 +67,8 @@ public:
   virtual nsresult Init(const char* aMimeType,
                         uint32_t aFlags) = 0;
 
-  virtual already_AddRefed<imgStatusTracker> GetStatusTracker() = 0;
-  virtual void SetStatusTracker(imgStatusTracker* aStatusTracker) {}
+  virtual already_AddRefed<ProgressTracker> GetProgressTracker() = 0;
+  virtual void SetProgressTracker(ProgressTracker* aProgressTracker) {}
 
   /**
    * The rectangle defining the location and size of the given frame.
@@ -72,25 +76,17 @@ public:
   virtual nsIntRect FrameRect(uint32_t aWhichFrame) = 0;
 
   /**
-   * The size, in bytes, occupied by the significant data portions of the image.
-   * This includes both compressed source data and decoded frames.
+   * The size, in bytes, occupied by the compressed source data of the image.
+   * If MallocSizeOf does not work on this platform, uses a fallback approach to
+   * ensure that something reasonable is always returned.
    */
-  virtual uint32_t SizeOfData() = 0;
+  virtual size_t SizeOfSourceWithComputedFallback(MallocSizeOf aMallocSizeOf) const = 0;
 
   /**
-   * The components that make up SizeOfData().
+   * The size, in bytes, occupied by the image's decoded data.
    */
-  virtual size_t HeapSizeOfSourceWithComputedFallback(MallocSizeOf aMallocSizeOf) const = 0;
-  virtual size_t HeapSizeOfDecodedWithComputedFallback(MallocSizeOf aMallocSizeOf) const = 0;
-  virtual size_t NonHeapSizeOfDecoded() const = 0;
-  virtual size_t OutOfProcessSizeOfDecoded() const = 0;
-
-  /**
-   * Gets the size of the memory taken up for the parsed vector image's
-   * document (e.g. SVGDocument), and returns the document's URL via the
-   * aDocURL outparam.
-   */
-  virtual size_t HeapSizeOfVectorImageDocument(nsACString* aDocURL = nullptr) const = 0;
+  virtual size_t SizeOfDecoded(gfxMemoryLocation aLocation,
+                               MallocSizeOf aMallocSizeOf) const = 0;
 
   virtual void IncrementAnimationConsumers() = 0;
   virtual void DecrementAnimationConsumers() = 0;
@@ -129,10 +125,10 @@ public:
                                        bool aLastPart) = 0;
 
   /**
-   * Called for multipart images to allow for any necessary reinitialization
-   * when there's a new part to add.
+   * Called when the SurfaceCache discards a persistent surface belonging to
+   * this image.
    */
-  virtual nsresult OnNewSourceData() = 0;
+  virtual void OnSurfaceDiscarded() = 0;
 
   virtual void SetInnerWindowID(uint64_t aInnerWindowId) = 0;
   virtual uint64_t InnerWindowID() const = 0;
@@ -146,17 +142,16 @@ public:
 class ImageResource : public Image
 {
 public:
-  already_AddRefed<imgStatusTracker> GetStatusTracker() MOZ_OVERRIDE {
-    nsRefPtr<imgStatusTracker> statusTracker = mStatusTracker;
-    MOZ_ASSERT(statusTracker);
-    return statusTracker.forget();
+  already_AddRefed<ProgressTracker> GetProgressTracker() MOZ_OVERRIDE {
+    nsRefPtr<ProgressTracker> progressTracker = mProgressTracker;
+    MOZ_ASSERT(progressTracker);
+    return progressTracker.forget();
   }
-  void SetStatusTracker(imgStatusTracker* aStatusTracker) MOZ_OVERRIDE MOZ_FINAL {
-    MOZ_ASSERT(aStatusTracker);
-    MOZ_ASSERT(!mStatusTracker);
-    mStatusTracker = aStatusTracker;
+  void SetProgressTracker(ProgressTracker* aProgressTracker) MOZ_OVERRIDE MOZ_FINAL {
+    MOZ_ASSERT(aProgressTracker);
+    MOZ_ASSERT(!mProgressTracker);
+    mProgressTracker = aProgressTracker;
   }
-  virtual uint32_t SizeOfData() MOZ_OVERRIDE;
 
   virtual void IncrementAnimationConsumers() MOZ_OVERRIDE;
   virtual void DecrementAnimationConsumers() MOZ_OVERRIDE;
@@ -164,6 +159,8 @@ public:
   virtual uint32_t GetAnimationConsumers() MOZ_OVERRIDE { return mAnimationConsumers; }
 #endif
 
+  virtual void OnSurfaceDiscarded() MOZ_OVERRIDE { }
+
   virtual void SetInnerWindowID(uint64_t aInnerWindowId) MOZ_OVERRIDE {
     mInnerWindowId = aInnerWindowId;
   }
@@ -215,7 +212,7 @@ protected:
   virtual nsresult StopAnimation() = 0;
 
   // Member data shared by all implementations of this abstract class
-  nsRefPtr<imgStatusTracker>    mStatusTracker;
+  nsRefPtr<ProgressTracker>     mProgressTracker;
   nsRefPtr<ImageURL>            mURI;
   TimeStamp                     mLastRefreshTime;
   uint64_t                      mInnerWindowId;
