@@ -31,6 +31,7 @@ public:
 
   void                GetSocketAddr(nsAString& aAddrStr) const;
   SocketConsumerBase* GetConsumer();
+  SocketBase*         GetSocketBase();
 
   // Shutdown state
   //
@@ -157,6 +158,12 @@ UnixSocketConsumerIO::GetConsumer()
   return mConsumer.get();
 }
 
+SocketBase*
+UnixSocketConsumerIO::GetSocketBase()
+{
+  return GetConsumer();
+}
+
 bool
 UnixSocketConsumerIO::IsShutdownOnMainThread() const
 {
@@ -548,7 +555,8 @@ UnixSocketConsumer::SendSocketData(UnixSocketRawData* aData)
 
   MOZ_ASSERT(!mIO->IsShutdownOnMainThread());
   XRE_GetIOMessageLoop()->PostTask(
-    FROM_HERE, new SocketIOSendTask<UnixSocketConsumerIO>(mIO, aData));
+    FROM_HERE,
+    new SocketIOSendTask<UnixSocketConsumerIO, UnixSocketRawData>(mIO, aData));
 
   return true;
 }
