@@ -19,6 +19,7 @@
 #include "asmjs/AsmJSSignalHandlers.h"
 
 #include "mozilla/DebugOnly.h"
+#include "mozilla/PodOperations.h"
 
 #include "asmjs/AsmJSModule.h"
 #include "vm/Runtime.h"
@@ -28,6 +29,51 @@ using namespace js::jit;
 
 using JS::GenericNaN;
 using mozilla::DebugOnly;
+using mozilla::PodArrayZero;
+
+#if defined(ANDROID)
+# include <sys/system_properties.h>
+# if defined(MOZ_LINKER)
+extern "C" MFBT_API bool IsSignalHandlingBroken();
+# endif
+#endif
+
+// For platforms where the signal/exception handler runs on the same
+// thread/stack as the victim (Unix and Windows), we can use TLS to find any
+// currently executing asm.js code.
+static JSRuntime *
+RuntimeForCurrentThread()
+{
+    PerThreadData *threadData = TlsPerThreadData.get();
+    if (!threadData)
+        return nullptr;
+
+    return threadData->runtimeIfOnOwnerThread();
+}
+
+// Crashing inside the signal handler can cause the handler to be recursively
+// invoked, eventually blowing the stack without actually showing a crash
+// report dialog via Breakpad. To guard against this we watch for such
+// recursion and fall through to the next handler immediately rather than
+// trying to handle it.
+class AutoSetHandlingSignal
+{
+    JSRuntime *rt;
+
+  public:
+    explicit AutoSetHandlingSignal(JSRuntime *rt)
+      : rt(rt)
+    {
+        MOZ_ASSERT(!rt->handlingSignal);
+        rt->handlingSignal = true;
+    }
+
+    ~AutoSetHandlingSignal()
+    {
+        MOZ_ASSERT(rt->handlingSignal);
+        rt->handlingSignal = false;
+    }
+};
 
 #if defined(XP_WIN)
 # define XMM_sig(p,i) ((p)->Xmm##i)
@@ -152,71 +198,12 @@ using mozilla::DebugOnly;
 #  define R15_sig(p) ((p)->uc_mcontext.mc_r15)
 # endif
 #elif defined(XP_MACOSX)
-// Mach requires special treatment.
+# define EIP_sig(p) ((p)->uc_mcontext->__ss.__eip)
+# define RIP_sig(p) ((p)->uc_mcontext->__ss.__rip)
 #else
 # error "Don't know how to read/write to the thread state via the mcontext_t."
 #endif
 
-// For platforms where the signal/exception handler runs on the same
-// thread/stack as the victim (Unix and Windows), we can use TLS to find any
-// currently executing asm.js code.
-#if !defined(XP_MACOSX)
-static JSRuntime *
-RuntimeForCurrentThread()
-{
-    PerThreadData *threadData = TlsPerThreadData.get();
-    if (!threadData)
-        return nullptr;
-
-    return threadData->runtimeIfOnOwnerThread();
-}
-#endif // !defined(XP_MACOSX)
-
-// Crashing inside the signal handler can cause the handler to be recursively
-// invoked, eventually blowing the stack without actually showing a crash
-// report dialog via Breakpad. To guard against this we watch for such
-// recursion and fall through to the next handler immediately rather than
-// trying to handle it.
-class AutoSetHandlingSignal
-{
-    JSRuntime *rt;
-
-  public:
-    explicit AutoSetHandlingSignal(JSRuntime *rt)
-      : rt(rt)
-    {
-        MOZ_ASSERT(!rt->handlingSignal);
-        rt->handlingSignal = true;
-    }
-
-    ~AutoSetHandlingSignal()
-    {
-        MOZ_ASSERT(rt->handlingSignal);
-        rt->handlingSignal = false;
-    }
-};
-
-#if defined(JS_CODEGEN_X64)
-template <class T>
-static void
-SetXMMRegToNaN(bool isFloat32, T *xmm_reg)
-{
-    if (isFloat32) {
-        JS_STATIC_ASSERT(sizeof(T) == 4 * sizeof(float));
-        float *floats = reinterpret_cast<float*>(xmm_reg);
-        floats[0] = GenericNaN();
-        floats[1] = 0;
-        floats[2] = 0;
-        floats[3] = 0;
-    } else {
-        JS_STATIC_ASSERT(sizeof(T) == 2 * sizeof(double));
-        double *dbls = reinterpret_cast<double*>(xmm_reg);
-        dbls[0] = GenericNaN();
-        dbls[1] = 0;
-    }
-}
-#endif
-
 #if defined(XP_WIN)
 # include "jswin.h"
 #else
@@ -228,7 +215,7 @@ SetXMMRegToNaN(bool isFloat32, T *xmm_reg)
 # include <sys/ucontext.h> // for ucontext_t, mcontext_t
 #endif
 
-#if defined(JS_CODEGEN_X64)
+#if defined(JS_CPU_X64)
 # if defined(__DragonFly__)
 #  include <machine/npx.h> // for union savefpu
 # elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
@@ -317,18 +304,6 @@ enum { REG_EIP = 14 };
 # endif  // !defined(__BIONIC_HAVE_UCONTEXT_T)
 #endif // defined(ANDROID)
 
-#if defined(ANDROID) && defined(MOZ_LINKER)
-// Apparently, on some Android systems, the signal handler is always passed
-// nullptr as the faulting address. This would cause the asm.js signal handler
-// to think that a safe out-of-bounds access was a nullptr-deref. This
-// brokenness is already detected by ElfLoader (enabled by MOZ_LINKER), so
-// reuse that check to disable asm.js compilation on systems where the signal
-// handler is broken.
-extern "C" MFBT_API bool IsSignalHandlingBroken();
-#else
-static bool IsSignalHandlingBroken() { return false; }
-#endif // defined(MOZ_LINKER)
-
 #if !defined(XP_WIN)
 # define CONTEXT ucontext_t
 #endif
@@ -343,62 +318,85 @@ static bool IsSignalHandlingBroken() { return false; }
 # define PC_sig(p) EPC_sig(p)
 #endif
 
-static bool
-HandleSimulatorInterrupt(JSRuntime *rt, AsmJSActivation *activation, void *faultingAddress)
-{
-    // If the ARM simulator is enabled, the pc is in the simulator C++ code and
-    // not in the generated code, so we check the simulator's pc manually. Also
-    // note that we can't simply use simulator->set_pc() here because the
-    // simulator could be in the middle of an instruction. On ARM, the signal
-    // handlers are currently only used for Odin code, see bug 964258.
-
-#if defined(JS_ARM_SIMULATOR) || defined(JS_MIPS_SIMULATOR)
-    const AsmJSModule &module = activation->module();
-    if (module.containsFunctionPC((void *)rt->mainThread.simulator()->get_pc()) &&
-        module.containsFunctionPC(faultingAddress))
-    {
-        activation->setResumePC(nullptr);
-        int32_t nextpc = int32_t(module.interruptExit());
-        rt->mainThread.simulator()->set_resume_pc(nextpc);
-        return true;
-    }
-#endif
-    return false;
-}
-
-#if !defined(XP_MACOSX)
 static uint8_t **
 ContextToPC(CONTEXT *context)
 {
 #ifdef JS_CODEGEN_NONE
     MOZ_CRASH();
 #else
-    return reinterpret_cast<uint8_t**>(&PC_sig(context));
+     return reinterpret_cast<uint8_t**>(&PC_sig(context));
 #endif
 }
 
-# if defined(JS_CODEGEN_X64)
+#if defined(JS_CODEGEN_X64)
+template <class T>
 static void
-SetRegisterToCoercedUndefined(CONTEXT *context, bool isFloat32, AnyRegister reg)
+SetXMMRegToNaN(AsmJSHeapAccess::ViewType viewType, T *xmm_reg)
+{
+    switch (viewType) {
+      case AsmJSHeapAccess::Float32: {
+        JS_STATIC_ASSERT(sizeof(T) == 4 * sizeof(float));
+        float *floats = reinterpret_cast<float*>(xmm_reg);
+        floats[0] = GenericNaN();
+        floats[1] = 0;
+        floats[2] = 0;
+        floats[3] = 0;
+        break;
+      }
+      case AsmJSHeapAccess::Float64: {
+        JS_STATIC_ASSERT(sizeof(T) == 2 * sizeof(double));
+        double *dbls = reinterpret_cast<double*>(xmm_reg);
+        dbls[0] = GenericNaN();
+        dbls[1] = 0;
+        break;
+      }
+      case AsmJSHeapAccess::Float32x4: {
+        JS_STATIC_ASSERT(sizeof(T) == 4 * sizeof(float));
+        float *floats = reinterpret_cast<float*>(xmm_reg);
+        for (unsigned i = 0; i < 4; i++)
+            floats[i] = GenericNaN();
+        break;
+      }
+      case AsmJSHeapAccess::Int32x4: {
+        JS_STATIC_ASSERT(sizeof(T) == 4 * sizeof(int32_t));
+        int32_t *ints = reinterpret_cast<int32_t*>(xmm_reg);
+        for (unsigned i = 0; i < 4; i++)
+            ints[i] = 0;
+        break;
+      }
+      case AsmJSHeapAccess::Int8:
+      case AsmJSHeapAccess::Uint8:
+      case AsmJSHeapAccess::Int16:
+      case AsmJSHeapAccess::Uint16:
+      case AsmJSHeapAccess::Int32:
+      case AsmJSHeapAccess::Uint32:
+      case AsmJSHeapAccess::Uint8Clamped:
+        MOZ_CRASH("unexpected type in SetXMMRegToNaN");
+    }
+}
+
+# if !defined(XP_MACOSX)
+static void
+SetRegisterToCoercedUndefined(CONTEXT *context, AsmJSHeapAccess::ViewType viewType, AnyRegister reg)
 {
     if (reg.isFloat()) {
         switch (reg.fpu().code()) {
-          case X86Registers::xmm0:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 0)); break;
-          case X86Registers::xmm1:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 1)); break;
-          case X86Registers::xmm2:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 2)); break;
-          case X86Registers::xmm3:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 3)); break;
-          case X86Registers::xmm4:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 4)); break;
-          case X86Registers::xmm5:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 5)); break;
-          case X86Registers::xmm6:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 6)); break;
-          case X86Registers::xmm7:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 7)); break;
-          case X86Registers::xmm8:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 8)); break;
-          case X86Registers::xmm9:  SetXMMRegToNaN(isFloat32, &XMM_sig(context, 9)); break;
-          case X86Registers::xmm10: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 10)); break;
-          case X86Registers::xmm11: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 11)); break;
-          case X86Registers::xmm12: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 12)); break;
-          case X86Registers::xmm13: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 13)); break;
-          case X86Registers::xmm14: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 14)); break;
-          case X86Registers::xmm15: SetXMMRegToNaN(isFloat32, &XMM_sig(context, 15)); break;
+          case X86Registers::xmm0:  SetXMMRegToNaN(viewType, &XMM_sig(context, 0)); break;
+          case X86Registers::xmm1:  SetXMMRegToNaN(viewType, &XMM_sig(context, 1)); break;
+          case X86Registers::xmm2:  SetXMMRegToNaN(viewType, &XMM_sig(context, 2)); break;
+          case X86Registers::xmm3:  SetXMMRegToNaN(viewType, &XMM_sig(context, 3)); break;
+          case X86Registers::xmm4:  SetXMMRegToNaN(viewType, &XMM_sig(context, 4)); break;
+          case X86Registers::xmm5:  SetXMMRegToNaN(viewType, &XMM_sig(context, 5)); break;
+          case X86Registers::xmm6:  SetXMMRegToNaN(viewType, &XMM_sig(context, 6)); break;
+          case X86Registers::xmm7:  SetXMMRegToNaN(viewType, &XMM_sig(context, 7)); break;
+          case X86Registers::xmm8:  SetXMMRegToNaN(viewType, &XMM_sig(context, 8)); break;
+          case X86Registers::xmm9:  SetXMMRegToNaN(viewType, &XMM_sig(context, 9)); break;
+          case X86Registers::xmm10: SetXMMRegToNaN(viewType, &XMM_sig(context, 10)); break;
+          case X86Registers::xmm11: SetXMMRegToNaN(viewType, &XMM_sig(context, 11)); break;
+          case X86Registers::xmm12: SetXMMRegToNaN(viewType, &XMM_sig(context, 12)); break;
+          case X86Registers::xmm13: SetXMMRegToNaN(viewType, &XMM_sig(context, 13)); break;
+          case X86Registers::xmm14: SetXMMRegToNaN(viewType, &XMM_sig(context, 14)); break;
+          case X86Registers::xmm15: SetXMMRegToNaN(viewType, &XMM_sig(context, 15)); break;
           default: MOZ_CRASH();
         }
     } else {
@@ -423,13 +421,13 @@ SetRegisterToCoercedUndefined(CONTEXT *context, bool isFloat32, AnyRegister reg)
         }
     }
 }
-# endif  // JS_CODEGEN_X64
-#endif   // !XP_MACOSX
+# endif  // !XP_MACOSX
+#endif // JS_CODEGEN_X64
 
 #if defined(XP_WIN)
 
 static bool
-HandleException(PEXCEPTION_POINTERS exception)
+HandleFault(PEXCEPTION_POINTERS exception)
 {
     EXCEPTION_RECORD *record = exception->ExceptionRecord;
     CONTEXT *context = exception->ContextRecord;
@@ -444,19 +442,13 @@ HandleException(PEXCEPTION_POINTERS exception)
     if (record->NumberParameters < 2)
         return false;
 
-    void *faultingAddress = (void*)record->ExceptionInformation[1];
-
-    JSRuntime *rt = RuntimeForCurrentThread();
-
     // Don't allow recursive handling of signals, see AutoSetHandlingSignal.
+    JSRuntime *rt = RuntimeForCurrentThread();
     if (!rt || rt->handlingSignal)
         return false;
     AutoSetHandlingSignal handling(rt);
 
-    if (rt->jitRuntime() && rt->jitRuntime()->handleAccessViolation(rt, faultingAddress))
-        return true;
-
-    AsmJSActivation *activation = PerThreadData::innermostAsmJSActivation();
+    AsmJSActivation *activation = rt->mainThread.asmJSActivationStack();
     if (!activation)
         return false;
 
@@ -464,23 +456,10 @@ HandleException(PEXCEPTION_POINTERS exception)
     if (!module.containsFunctionPC(pc))
         return false;
 
-    // If we faulted trying to execute code in 'module', this must be an
-    // interrupt callback (see RequestInterruptForAsmJSCode). Redirect
-    // execution to a trampoline which will call js::HandleExecutionInterrupt.
-    // The trampoline will jump to activation->resumePC if execution isn't
-    // interrupted.
-    if (module.containsFunctionPC(faultingAddress)) {
-        activation->setResumePC(pc);
-        *ppc = module.interruptExit();
-
-        JSRuntime::AutoLockForInterrupt lock(rt);
-        module.unprotectCode(rt);
-        return true;
-    }
-
 # if defined(JS_CODEGEN_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
+    void *faultingAddress = (void*)record->ExceptionInformation[1];
     if (!module.maybeHeap() ||
         faultingAddress < module.maybeHeap() ||
         faultingAddress >= module.maybeHeap() + AsmJSMappedSize)
@@ -503,8 +482,9 @@ HandleException(PEXCEPTION_POINTERS exception)
     // register) and set the PC to the next op. Upon return from the handler,
     // execution will resume at this next PC.
     if (heapAccess->isLoad())
-        SetRegisterToCoercedUndefined(context, heapAccess->isFloat32Load(), heapAccess->loadedReg());
+        SetRegisterToCoercedUndefined(context, heapAccess->viewType(), heapAccess->loadedReg());
     *ppc += heapAccess->opLength();
+
     return true;
 # else
     return false;
@@ -512,9 +492,9 @@ HandleException(PEXCEPTION_POINTERS exception)
 }
 
 static LONG WINAPI
-AsmJSExceptionHandler(LPEXCEPTION_POINTERS exception)
+AsmJSFaultHandler(LPEXCEPTION_POINTERS exception)
 {
-    if (HandleException(exception))
+    if (HandleFault(exception))
         return EXCEPTION_CONTINUE_EXECUTION;
 
     // No need to worry about calling other handlers, the OS does this for us.
@@ -527,13 +507,13 @@ AsmJSExceptionHandler(LPEXCEPTION_POINTERS exception)
 static uint8_t **
 ContextToPC(x86_thread_state_t &state)
 {
-# if defined(JS_CODEGEN_X64)
-    JS_STATIC_ASSERT(sizeof(state.uts.ts64.__rip) == sizeof(void*));
+# if defined(JS_CPU_X64)
+    static_assert(sizeof(state.uts.ts64.__rip) == sizeof(void*),
+                  "stored IP should be compile-time pointer-sized");
     return reinterpret_cast<uint8_t**>(&state.uts.ts64.__rip);
-# elif defined(JS_CODEGEN_NONE)
-    MOZ_CRASH();
 # else
-    JS_STATIC_ASSERT(sizeof(state.uts.ts32.__eip) == sizeof(void*));
+    static_assert(sizeof(state.uts.ts32.__eip) == sizeof(void*),
+                  "stored IP should be compile-time pointer-sized");
     return reinterpret_cast<uint8_t**>(&state.uts.ts32.__eip);
 # endif
 }
@@ -552,24 +532,24 @@ SetRegisterToCoercedUndefined(mach_port_t rtThread, x86_thread_state64_t &state,
         if (kret != KERN_SUCCESS)
             return false;
 
-        bool f32 = heapAccess.isFloat32Load();
+        AsmJSHeapAccess::ViewType viewType = heapAccess.viewType();
         switch (heapAccess.loadedReg().fpu().code()) {
-          case X86Registers::xmm0:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm0); break;
-          case X86Registers::xmm1:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm1); break;
-          case X86Registers::xmm2:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm2); break;
-          case X86Registers::xmm3:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm3); break;
-          case X86Registers::xmm4:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm4); break;
-          case X86Registers::xmm5:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm5); break;
-          case X86Registers::xmm6:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm6); break;
-          case X86Registers::xmm7:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm7); break;
-          case X86Registers::xmm8:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm8); break;
-          case X86Registers::xmm9:  SetXMMRegToNaN(f32, &fstate.__fpu_xmm9); break;
-          case X86Registers::xmm10: SetXMMRegToNaN(f32, &fstate.__fpu_xmm10); break;
-          case X86Registers::xmm11: SetXMMRegToNaN(f32, &fstate.__fpu_xmm11); break;
-          case X86Registers::xmm12: SetXMMRegToNaN(f32, &fstate.__fpu_xmm12); break;
-          case X86Registers::xmm13: SetXMMRegToNaN(f32, &fstate.__fpu_xmm13); break;
-          case X86Registers::xmm14: SetXMMRegToNaN(f32, &fstate.__fpu_xmm14); break;
-          case X86Registers::xmm15: SetXMMRegToNaN(f32, &fstate.__fpu_xmm15); break;
+          case X86Registers::xmm0:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm0); break;
+          case X86Registers::xmm1:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm1); break;
+          case X86Registers::xmm2:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm2); break;
+          case X86Registers::xmm3:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm3); break;
+          case X86Registers::xmm4:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm4); break;
+          case X86Registers::xmm5:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm5); break;
+          case X86Registers::xmm6:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm6); break;
+          case X86Registers::xmm7:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm7); break;
+          case X86Registers::xmm8:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm8); break;
+          case X86Registers::xmm9:  SetXMMRegToNaN(viewType, &fstate.__fpu_xmm9); break;
+          case X86Registers::xmm10: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm10); break;
+          case X86Registers::xmm11: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm11); break;
+          case X86Registers::xmm12: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm12); break;
+          case X86Registers::xmm13: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm13); break;
+          case X86Registers::xmm14: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm14); break;
+          case X86Registers::xmm15: SetXMMRegToNaN(viewType, &fstate.__fpu_xmm15); break;
           default: MOZ_CRASH();
         }
 
@@ -650,45 +630,18 @@ HandleMachException(JSRuntime *rt, const ExceptionRequest &request)
     if (request.body.exception != EXC_BAD_ACCESS || request.body.codeCnt != 2)
         return false;
 
-    void *faultingAddress = (void*)request.body.code[1];
-
-    if (rt->jitRuntime() && rt->jitRuntime()->handleAccessViolation(rt, faultingAddress))
-        return true;
-
     AsmJSActivation *activation = rt->mainThread.asmJSActivationStack();
     if (!activation)
         return false;
 
     const AsmJSModule &module = activation->module();
-    if (HandleSimulatorInterrupt(rt, activation, faultingAddress)) {
-        JSRuntime::AutoLockForInterrupt lock(rt);
-        module.unprotectCode(rt);
-        return true;
-    }
-
     if (!module.containsFunctionPC(pc))
         return false;
 
-    // If we faulted trying to execute code in 'module', this must be an
-    // interrupt callback (see RequestInterruptForAsmJSCode). Redirect
-    // execution to a trampoline which will call js::HandleExecutionInterrupt.
-    // The trampoline will jump to activation->resumePC if execution isn't
-    // interrupted.
-    if (module.containsFunctionPC(faultingAddress)) {
-        activation->setResumePC(pc);
-        *ppc = module.interruptExit();
-
-        JSRuntime::AutoLockForInterrupt lock(rt);
-        module.unprotectCode(rt);
-
-        // Update the thread state with the new pc.
-        kret = thread_set_state(rtThread, x86_THREAD_STATE, (thread_state_t)&state, x86_THREAD_STATE_COUNT);
-        return kret == KERN_SUCCESS;
-    }
-
-# if defined(JS_CODEGEN_X64)
+# if defined(JS_CPU_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
+    void *faultingAddress = (void*)request.body.code[1];
     if (!module.maybeHeap() ||
         faultingAddress < module.maybeHeap() ||
         faultingAddress >= module.maybeHeap() + AsmJSMappedSize)
@@ -879,55 +832,30 @@ AsmJSMachExceptionHandler::install(JSRuntime *rt)
 // Be very cautious and default to not handling; we don't want to accidentally
 // silence real crashes from real bugs.
 static bool
-HandleSignal(int signum, siginfo_t *info, void *ctx)
+HandleFault(int signum, siginfo_t *info, void *ctx)
 {
     CONTEXT *context = (CONTEXT *)ctx;
     uint8_t **ppc = ContextToPC(context);
     uint8_t *pc = *ppc;
 
-    void *faultingAddress = info->si_addr;
-
-    JSRuntime *rt = RuntimeForCurrentThread();
-
     // Don't allow recursive handling of signals, see AutoSetHandlingSignal.
+    JSRuntime *rt = RuntimeForCurrentThread();
     if (!rt || rt->handlingSignal)
         return false;
     AutoSetHandlingSignal handling(rt);
 
-    if (rt->jitRuntime() && rt->jitRuntime()->handleAccessViolation(rt, faultingAddress))
-        return true;
-
-    AsmJSActivation *activation = PerThreadData::innermostAsmJSActivation();
+    AsmJSActivation *activation = rt->mainThread.asmJSActivationStack();
     if (!activation)
         return false;
 
     const AsmJSModule &module = activation->module();
-    if (HandleSimulatorInterrupt(rt, activation, faultingAddress)) {
-        JSRuntime::AutoLockForInterrupt lock(rt);
-        module.unprotectCode(rt);
-        return true;
-    }
-
     if (!module.containsFunctionPC(pc))
         return false;
 
-    // If we faulted trying to execute code in 'module', this must be an
-    // interrupt callback (see RequestInterruptForAsmJSCode). Redirect
-    // execution to a trampoline which will call js::HandleExecutionInterrupt.
-    // The trampoline will jump to activation->resumePC if execution isn't
-    // interrupted.
-    if (module.containsFunctionPC(faultingAddress)) {
-        activation->setResumePC(pc);
-        *ppc = module.interruptExit();
-
-        JSRuntime::AutoLockForInterrupt lock(rt);
-        module.unprotectCode(rt);
-        return true;
-    }
-
 # if defined(JS_CODEGEN_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
+    void *faultingAddress = info->si_addr;
     if (!module.maybeHeap() ||
         faultingAddress < module.maybeHeap() ||
         faultingAddress >= module.maybeHeap() + AsmJSMappedSize)
@@ -946,20 +874,21 @@ HandleSignal(int signum, siginfo_t *info, void *ctx)
     // register) and set the PC to the next op. Upon return from the handler,
     // execution will resume at this next PC.
     if (heapAccess->isLoad())
-        SetRegisterToCoercedUndefined(context, heapAccess->isFloat32Load(), heapAccess->loadedReg());
+        SetRegisterToCoercedUndefined(context, heapAccess->viewType(), heapAccess->loadedReg());
     *ppc += heapAccess->opLength();
+
     return true;
 # else
     return false;
 # endif
 }
 
-static struct sigaction sPrevHandler;
+static struct sigaction sPrevSEGVHandler;
 
 static void
 AsmJSFaultHandler(int signum, siginfo_t *info, void *context)
 {
-    if (HandleSignal(signum, info, context))
+    if (HandleFault(signum, info, context))
         return;
 
     // This signal is not for any asm.js code we expect, so we need to forward
@@ -974,90 +903,197 @@ AsmJSFaultHandler(int signum, siginfo_t *info, void *context)
     // signal to it's original disposition and returning.
     //
     // Note: the order of these tests matter.
-    if (sPrevHandler.sa_flags & SA_SIGINFO)
-        sPrevHandler.sa_sigaction(signum, info, context);
-    else if (sPrevHandler.sa_handler == SIG_DFL || sPrevHandler.sa_handler == SIG_IGN)
-        sigaction(signum, &sPrevHandler, nullptr);
+    if (sPrevSEGVHandler.sa_flags & SA_SIGINFO)
+        sPrevSEGVHandler.sa_sigaction(signum, info, context);
+    else if (sPrevSEGVHandler.sa_handler == SIG_DFL || sPrevSEGVHandler.sa_handler == SIG_IGN)
+        sigaction(signum, &sPrevSEGVHandler, nullptr);
     else
-        sPrevHandler.sa_handler(signum);
+        sPrevSEGVHandler.sa_handler(signum);
 }
 #endif
 
-#if !defined(XP_MACOSX)
-static bool sInstalledHandlers = false;
+static void
+RedirectIonBackedgesToInterruptCheck(JSRuntime *rt)
+{
+    if (jit::JitRuntime *jitRuntime = rt->jitRuntime()) {
+        // If the backedge list is being mutated, the pc must be in C++ code and
+        // thus not in a JIT iloop. We assume that the interrupt flag will be
+        // checked at least once before entering JIT code (if not, no big deal;
+        // the browser will just request another interrupt in a second).
+        if (!jitRuntime->mutatingBackedgeList())
+            jitRuntime->patchIonBackedges(rt, jit::JitRuntime::BackedgeInterruptCheck);
+    }
+}
+
+static void
+RedirectJitCodeToInterruptCheck(JSRuntime *rt, CONTEXT *context)
+{
+    RedirectIonBackedgesToInterruptCheck(rt);
+
+    if (AsmJSActivation *activation = rt->mainThread.asmJSActivationStack()) {
+        const AsmJSModule &module = activation->module();
+
+#if defined(JS_ARM_SIMULATOR) || defined(JS_MIPS_SIMULATOR)
+        if (module.containsFunctionPC((void*)rt->mainThread.simulator()->get_pc()))
+            rt->mainThread.simulator()->set_resume_pc(int32_t(module.interruptExit()));
+#endif
+
+        uint8_t **ppc = ContextToPC(context);
+        uint8_t *pc = *ppc;
+        if (module.containsFunctionPC(pc)) {
+            activation->setResumePC(pc);
+            *ppc = module.interruptExit();
+        }
+    }
+}
+
+#if !defined(XP_WIN)
+// For the interrupt signal, pick a signal number that:
+//  - is not otherwise used by mozilla or standard libraries
+//  - defaults to nostop and noprint on gdb/lldb so that noone is bothered
+// SIGVTALRM a relative of SIGALRM, so intended for user code, but, unlike
+// SIGALRM, not used anywhere else in Mozilla.
+static const int sInterruptSignal = SIGVTALRM;
+
+static void
+JitInterruptHandler(int signum, siginfo_t *info, void *context)
+{
+    if (JSRuntime *rt = RuntimeForCurrentThread())
+        RedirectJitCodeToInterruptCheck(rt, (CONTEXT*)context);
+}
 #endif
 
 bool
-js::EnsureAsmJSSignalHandlersInstalled(JSRuntime *rt)
+js::EnsureSignalHandlersInstalled(JSRuntime *rt)
 {
-#ifdef JS_CODEGEN_NONE
-    // Don't install signal handlers in builds with the JIT disabled.
-    return false;
+#if defined(XP_MACOSX)
+    // On OSX, each JSRuntime gets its own handler thread.
+    if (!rt->asmJSMachExceptionHandler.installed() && !rt->asmJSMachExceptionHandler.install(rt))
+        return false;
 #endif
 
+    // All the rest of the handlers are process-wide and thus must only be
+    // installed once. We assume that there are no races creating the first
+    // JSRuntime of the process.
+    static bool sTried = false;
+    static bool sResult = false;
+    if (sTried)
+        return sResult;
+    sTried = true;
+
+#if defined(ANDROID)
+    // Before Android 4.4 (SDK version 19), there is a bug
+    //   https://android-review.googlesource.com/#/c/52333
+    // in Bionic's pthread_join which causes pthread_join to return early when
+    // pthread_kill is used (on any thread). Nobody expects the pthread_cond_wait
+    // EINTRquisition.
+    char version_string[PROP_VALUE_MAX];
+    PodArrayZero(version_string);
+    if (__system_property_get("ro.build.version.sdk", version_string) > 0) {
+        if (atol(version_string) < 19)
+            return false;
+    }
+# if defined(MOZ_LINKER)
+    // Signal handling is broken on some android systems.
     if (IsSignalHandlingBroken())
         return false;
-
-#if defined(XP_MACOSX)
-    // On OSX, each JSRuntime gets its own handler.
-    return rt->asmJSMachExceptionHandler.installed() || rt->asmJSMachExceptionHandler.install(rt);
-#else
-    // Assume Windows or Unix. For these platforms, there is a single,
-    // process-wide signal handler installed. Take care to only install it once.
-    if (sInstalledHandlers)
-        return true;
-
-# if defined(XP_WIN)
-    if (!AddVectoredExceptionHandler(/* FirstHandler = */true, AsmJSExceptionHandler))
-        return false;
-# else
-    // Assume Unix. SA_NODEFER allows us to reenter the signal handler if we
-    // crash while handling the signal, and fall through to the Breakpad
-    // handler by testing handlingSignal.
-    struct sigaction sigAction;
-    sigAction.sa_flags = SA_SIGINFO | SA_NODEFER;
-    sigAction.sa_sigaction = &AsmJSFaultHandler;
-    sigemptyset(&sigAction.sa_mask);
-    if (sigaction(SIGSEGV, &sigAction, &sPrevHandler))
-        return false;
 # endif
-
-    sInstalledHandlers = true;
 #endif
+
+#if defined(XP_WIN)
+    // Windows uses SuspendThread to stop the main thread from another thread,
+    // so the only handler we need is for asm.js out-of-bound faults.
+    if (!AddVectoredExceptionHandler(/* FirstHandler = */ true, AsmJSFaultHandler))
+        return false;
+#else
+    // The interrupt handler allows the main thread to be paused from another
+    // thread (see InterruptRunningJitCode).
+    struct sigaction interruptHandler;
+    interruptHandler.sa_flags = SA_SIGINFO;
+    interruptHandler.sa_sigaction = &JitInterruptHandler;
+    sigemptyset(&interruptHandler.sa_mask);
+    struct sigaction prev;
+    if (sigaction(sInterruptSignal, &interruptHandler, &prev))
+        MOZ_CRASH("unable to install interrupt handler");
+
+    // There shouldn't be any other handlers installed for sInterruptSignal. If
+    // there are, we could always forward, but we need to understand what we're
+    // doing to avoid problematic interference.
+    if ((prev.sa_flags & SA_SIGINFO && prev.sa_sigaction) ||
+        (prev.sa_handler != SIG_DFL && prev.sa_handler != SIG_IGN))
+    {
+        MOZ_CRASH("contention for interrupt signal");
+    }
+
+    // Lastly, install a SIGSEGV handler to handle safely-out-of-bounds asm.js
+    // heap access. OSX handles seg faults via the Mach exception handler above,
+    // so don't install AsmJSFaultHandler.
+# if !defined(XP_MACOSX)
+    // SA_NODEFER allows us to reenter the signal handler if we crash while
+    // handling the signal, and fall through to the Breakpad handler by testing
+    // handlingSignal.
+    struct sigaction faultHandler;
+    faultHandler.sa_flags = SA_SIGINFO | SA_NODEFER;
+    faultHandler.sa_sigaction = &AsmJSFaultHandler;
+    sigemptyset(&faultHandler.sa_mask);
+    if (sigaction(SIGSEGV, &faultHandler, &sPrevSEGVHandler))
+        MOZ_CRASH("unable to install segv handler");
+# endif // defined(XP_MACOSX)
+#endif // defined(XP_WIN)
+
+    sResult = true;
     return true;
 }
 
-// To interrupt execution of a JSRuntime, any thread may call
-// JS_RequestInterruptCallback (JSRuntime::requestInterruptCallback from inside
-// the engine). In the simplest case, this sets some state that is polled at
-// regular intervals (function prologues, loop headers). For tight loops, this
-// poses non-trivial overhead. For asm.js, we can do better: when another
-// thread requests an interrupt, we simply mprotect all of the innermost asm.js
-// module activation's code. This will trigger a SIGSEGV, taking us into
-// AsmJSFaultHandler. From there, we can manually redirect execution to call
-// js::HandleExecutionInterrupt. The memory is un-protected from the signal
-// handler after control flow is redirected.
+// JSRuntime::requestInterrupt sets interrupt_ (which is checked frequently by
+// C++ code at every Baseline JIT loop backedge) and jitStackLimit_ (which is
+// checked at every Baseline and Ion JIT function prologue). The remaining
+// sources of potential iloops (Ion loop backedges and all asm.js code) are
+// handled by this function:
+//  1. Ion loop backedges are patched to instead point to a stub that handles the
+//     interrupt;
+//  2. if the main thread's pc is inside asm.js code, the pc is updated to point
+//     to a stub that handles the interrupt.
 void
-js::RequestInterruptForAsmJSCode(JSRuntime *rt, int interruptModeRaw)
+js::InterruptRunningJitCode(JSRuntime *rt)
 {
-    switch (JSRuntime::InterruptMode(interruptModeRaw)) {
-      case JSRuntime::RequestInterruptMainThread:
-      case JSRuntime::RequestInterruptAnyThread:
-        break;
-      case JSRuntime::RequestInterruptAnyThreadDontStopIon:
-      case JSRuntime::RequestInterruptAnyThreadForkJoin:
-        // It is ok to wait for asm.js execution to complete; we aren't trying
-        // to break an iloop or anything. Avoid the overhead of protecting all
-        // the code and taking a fault.
+    // If signal handlers weren't installed, then Ion and asm.js emit normal
+    // interrupt checks and don't need asynchronous interruption.
+    if (!rt->canUseSignalHandlers())
+        return;
+
+    // If we are on runtime's main thread, then: pc is not in asm.js code (so
+    // nothing to do for asm.js) and we can patch Ion backedges without any
+    // special synchronization.
+    if (rt == RuntimeForCurrentThread()) {
+        RedirectIonBackedgesToInterruptCheck(rt);
         return;
     }
 
-    AsmJSActivation *activation = rt->mainThread.asmJSActivationStack();
-    if (!activation)
-        return;
-
-    MOZ_ASSERT(rt->currentThreadOwnsInterruptLock());
-    activation->module().protectCode(rt);
+    // We are not on the runtime's main thread, so to do 1 and 2 above, we need
+    // to halt the runtime's main thread first.
+#if defined(XP_WIN)
+    // On Windows, we can simply suspend the main thread and work directly on
+    // its context from this thread. SuspendThread can sporadically fail if the
+    // thread is in the middle of a syscall. Rather than retrying in a loop,
+    // just wait for the next request for interrupt.
+    HANDLE thread = (HANDLE)rt->ownerThreadNative();
+    if (SuspendThread(thread) != -1) {
+        CONTEXT context;
+        context.ContextFlags = CONTEXT_CONTROL;
+        if (GetThreadContext(thread, &context)) {
+            RedirectJitCodeToInterruptCheck(rt, &context);
+            SetThreadContext(thread, &context);
+        }
+        ResumeThread(thread);
+    }
+#else
+    // On Unix, we instead deliver an async signal to the main thread which
+    // halts the thread and callers our JitInterruptHandler (which has already
+    // been installed by EnsureSignalHandlersInstalled).
+    pthread_t thread = (pthread_t)rt->ownerThreadNative();
+    pthread_kill(thread, sInterruptSignal);
+#endif
 }
 
 // This is not supported by clang-cl yet.
