@@ -432,20 +432,18 @@ function ArrayFind(predicate/*, thisArg*/) {
     var T = arguments.length > 1 ? arguments[1] : undefined;
 
     /* Steps 8-9. */
-    /* Steps a (implicit), and e. */
+    /* Steps a (implicit), and g. */
     /* Note: this will hang in some corner-case situations, because of IEEE-754 numbers'
      * imprecision for large values. Example:
      * var obj = { 18014398509481984: true, length: 18014398509481988 };
      * Array.prototype.find.call(obj, () => true);
      */
     for (var k = 0; k < len; k++) {
-        /* Steps b and c (implicit) */
-        if (k in O) {
-            /* Step d. */
-            var kValue = O[k];
-            if (callFunction(predicate, T, kValue, k, O))
-                return kValue;
-        }
+        /* Steps a-c. */
+        var kValue = O[k];
+        /* Steps d-f. */
+        if (callFunction(predicate, T, kValue, k, O))
+            return kValue;
     }
 
     /* Step 10. */
@@ -470,19 +468,16 @@ function ArrayFindIndex(predicate/*, thisArg*/) {
     var T = arguments.length > 1 ? arguments[1] : undefined;
 
     /* Steps 8-9. */
-    /* Steps a (implicit), and e. */
+    /* Steps a (implicit), and g. */
     /* Note: this will hang in some corner-case situations, because of IEEE-754 numbers'
      * imprecision for large values. Example:
      * var obj = { 18014398509481984: true, length: 18014398509481988 };
      * Array.prototype.find.call(obj, () => true);
      */
     for (var k = 0; k < len; k++) {
-        /* Steps b and c (implicit) */
-        if (k in O) {
-            /* Step d. */
-            if (callFunction(predicate, T, O[k], k, O))
-                return k;
-        }
+        /* Steps a-f. */
+        if (callFunction(predicate, T, O[k], k, O))
+            return k;
     }
 
     /* Step 10. */
@@ -586,6 +581,50 @@ function ArrayFill(value, start = 0, end = undefined) {
     return O;
 }
 
+// Proposed for ES7:
+// https://github.com/domenic/Array.prototype.includes/blob/master/spec.md
+function ArrayIncludes(searchElement, fromIndex = 0) {
+    // Steps 1-2.
+    var O = ToObject(this);
+
+    // Steps 3-4.
+    var len = ToLength(O.length);
+
+    // Step 5.
+    if (len === 0)
+        return false;
+
+    // Steps 6-7.
+    var n = ToInteger(fromIndex);
+
+    // Step 8.
+    var k;
+    if (n >= 0) {
+        k = n;
+    }
+    // Step 9.
+    else {
+        // Step a.
+        k = len + n;
+        // Step b.
+        if (k < 0)
+            k = 0;
+    }
+
+    // Step 10.
+    while (k < len) {
+        // Steps a-c.
+        if (SameValueZero(searchElement, O[k]))
+            return true;
+
+        // Step d.
+        k++;
+    }
+
+    // Step 11.
+    return false;
+}
+
 #define ARRAY_ITERATOR_SLOT_ITERATED_OBJECT 0
 #define ARRAY_ITERATOR_SLOT_NEXT_INDEX 1
 #define ARRAY_ITERATOR_SLOT_ITEM_KIND 2
@@ -619,29 +658,35 @@ function ArrayIteratorNext() {
     var a = UnsafeGetReservedSlot(this, ARRAY_ITERATOR_SLOT_ITERATED_OBJECT);
     var index = UnsafeGetReservedSlot(this, ARRAY_ITERATOR_SLOT_NEXT_INDEX);
     var itemKind = UnsafeGetReservedSlot(this, ARRAY_ITERATOR_SLOT_ITEM_KIND);
+    var result = { value: undefined, done: false };
 
     // FIXME: This should be ToLength, which clamps at 2**53.  Bug 924058.
     if (index >= TO_UINT32(a.length)) {
         // When the above is changed to ToLength, use +1/0 here instead
         // of MAX_UINT32.
         UnsafeSetReservedSlot(this, ARRAY_ITERATOR_SLOT_NEXT_INDEX, 0xffffffff);
-        return { value: undefined, done: true };
+        result.done = true;
+        return result;
     }
 
     UnsafeSetReservedSlot(this, ARRAY_ITERATOR_SLOT_NEXT_INDEX, index + 1);
 
-    if (itemKind === ITEM_KIND_VALUE)
-        return { value: a[index], done: false };
+    if (itemKind === ITEM_KIND_VALUE) {
+        result.value = a[index];
+        return result;
+    }
 
     if (itemKind === ITEM_KIND_KEY_AND_VALUE) {
         var pair = NewDenseArray(2);
         pair[0] = index;
         pair[1] = a[index];
-        return { value: pair, done : false };
+        result.value = pair;
+        return result;
     }
 
     assert(itemKind === ITEM_KIND_KEY, itemKind);
-    return { value: index, done: false };
+    result.value = index;
+    return result;
 }
 
 function ArrayValuesAt(n) {
@@ -677,7 +722,7 @@ function ArrayFrom(arrayLike, mapfn=undefined, thisArg=undefined) {
     var attrs = ATTR_CONFIGURABLE | ATTR_ENUMERABLE | ATTR_WRITABLE;
 
     // Steps 6-8.
-    var usingIterator = items["@@iterator"];
+    var usingIterator = items[std_iterator];
     if (usingIterator !== undefined) {
         // Steps 8.a-c.
         var A = IsConstructor(C) ? new C() : [];
@@ -789,8 +834,6 @@ function ArrayMapPar(func, mode) {
     }
     return sliceId;
   }
-
-  return undefined;
 }
 
 /**
@@ -845,8 +888,6 @@ function ArrayReducePar(func, mode) {
     }
     return sliceId;
   }
-
-  return undefined;
 }
 
 /**
@@ -996,8 +1037,6 @@ function ArrayScanPar(func, mode) {
     }
     return sliceId;
   }
-
-  return undefined;
 }
 
 /**
@@ -1079,8 +1118,6 @@ function ArrayScatterPar(targets, defaultValue, conflictFunc, length, mode) {
     // It's not enough to return t, as -0 | 0 === -0.
     return TO_INT32(t);
   }
-
-  return undefined;
 }
 
 /**
@@ -1189,8 +1226,6 @@ function ArrayFilterPar(func, mode) {
 
     return sliceId;
   }
-
-  return undefined;
 }
 
 /**
@@ -1251,8 +1286,6 @@ function ArrayStaticBuildPar(length, func, mode) {
     }
     return sliceId;
   }
-
-  return undefined;
 }
 
 /*
