@@ -7,6 +7,7 @@
 #ifndef ds_LifoAlloc_h
 #define ds_LifoAlloc_h
 
+#include "mozilla/Attributes.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/MemoryChecking.h"
@@ -37,7 +38,7 @@ AlignPtr(void *orig)
                   "LIFO_ALLOC_ALIGN must be a power of two");
 
     char *result = (char *) ((uintptr_t(orig) + (LIFO_ALLOC_ALIGN - 1)) & (~LIFO_ALLOC_ALIGN + 1));
-    JS_ASSERT(uintptr_t(result) % LIFO_ALLOC_ALIGN == 0);
+    MOZ_ASSERT(uintptr_t(result) % LIFO_ALLOC_ALIGN == 0);
     return result;
 }
 
@@ -57,18 +58,18 @@ class BumpChunk
         limit(bump + bumpSpaceSize),
         next_(nullptr), bumpSpaceSize(bumpSpaceSize)
     {
-        JS_ASSERT(bump == AlignPtr(bump));
+        MOZ_ASSERT(bump == AlignPtr(bump));
     }
 
     void setBump(void *ptr) {
-        JS_ASSERT(bumpBase() <= ptr);
-        JS_ASSERT(ptr <= limit);
+        MOZ_ASSERT(bumpBase() <= ptr);
+        MOZ_ASSERT(ptr <= limit);
 #if defined(DEBUG) || defined(MOZ_HAVE_MEM_CHECKS)
         char* prevBump = bump;
 #endif
         bump = static_cast<char *>(ptr);
 #ifdef DEBUG
-        JS_ASSERT(contains(prevBump));
+        MOZ_ASSERT(contains(prevBump));
 
         // Clobber the now-free space.
         if (prevBump > bump)
@@ -108,8 +109,8 @@ class BumpChunk
     void *mark() const { return bump; }
 
     void release(void *mark) {
-        JS_ASSERT(contains(mark));
-        JS_ASSERT(mark <= bump);
+        MOZ_ASSERT(contains(mark));
+        MOZ_ASSERT(mark <= bump);
         setBump(mark);
     }
 
@@ -136,7 +137,7 @@ class BumpChunk
         if (MOZ_UNLIKELY(newBump < bump))
             return nullptr;
 
-        JS_ASSERT(canAlloc(n)); // Ensure consistency between "can" and "try".
+        MOZ_ASSERT(canAlloc(n)); // Ensure consistency between "can" and "try".
         setBump(newBump);
         return aligned;
     }
@@ -147,7 +148,7 @@ class BumpChunk
 
 } // namespace detail
 
-void
+MOZ_NORETURN void
 CrashAtUnhandlableOOM(const char *reason);
 
 // LIFO bump allocator: used for phase-oriented and fast LIFO allocations.
@@ -177,7 +178,7 @@ class LifoAlloc
     BumpChunk *getOrCreateChunk(size_t n);
 
     void reset(size_t defaultChunkSize) {
-        JS_ASSERT(mozilla::RoundUpPow2(defaultChunkSize) == defaultChunkSize);
+        MOZ_ASSERT(mozilla::RoundUpPow2(defaultChunkSize) == defaultChunkSize);
         first = latest = last = nullptr;
         defaultChunkSize_ = defaultChunkSize;
         markCount = 0;
@@ -186,7 +187,7 @@ class LifoAlloc
 
     // Append unused chunks to the end of this LifoAlloc.
     void appendUnused(BumpChunk *start, BumpChunk *end) {
-        JS_ASSERT(start && end);
+        MOZ_ASSERT(start && end);
         if (last)
             last->setNext(start);
         else
@@ -197,7 +198,7 @@ class LifoAlloc
     // Append used chunks to the end of this LifoAlloc. We act as if all the
     // chunks in |this| are used, even if they're not, so memory may be wasted.
     void appendUsed(BumpChunk *start, BumpChunk *latest, BumpChunk *end) {
-        JS_ASSERT(start && latest &&  end);
+        MOZ_ASSERT(start && latest &&  end);
         if (last)
             last->setNext(start);
         else
@@ -212,7 +213,7 @@ class LifoAlloc
             peakSize_ = curSize_;
     }
     void decrementCurSize(size_t size) {
-        JS_ASSERT(curSize_ >= size);
+        MOZ_ASSERT(curSize_ >= size);
         curSize_ -= size;
     }
 
@@ -225,7 +226,8 @@ class LifoAlloc
 
     // Steal allocated chunks from |other|.
     void steal(LifoAlloc *other) {
-        JS_ASSERT(!other->markCount);
+        MOZ_ASSERT(!other->markCount);
+        MOZ_ASSERT(!latest);
 
         // Copy everything from |other| to |this| except for |peakSize_|, which
         // requires some care.
@@ -301,7 +303,9 @@ class LifoAlloc
 
     template <typename T>
     T *newArray(size_t count) {
-        JS_STATIC_ASSERT(mozilla::IsPod<T>::value);
+        static_assert(mozilla::IsPod<T>::value,
+                      "T must be POD so that constructors (and destructors, "
+                      "when the LifoAlloc is freed) need not be called");
         return newArrayUninitialized<T>(count);
     }
 
@@ -341,7 +345,7 @@ class LifoAlloc
     }
 
     void releaseAll() {
-        JS_ASSERT(!markCount);
+        MOZ_ASSERT(!markCount);
         latest = first;
         if (latest)
             latest->resetBump();
@@ -379,6 +383,14 @@ class LifoAlloc
         return n;
     }
 
+    // Get the total size of the arena chunks (including unused space).
+    size_t computedSizeOfExcludingThis() const {
+        size_t n = 0;
+        for (BumpChunk *chunk = first; chunk; chunk = chunk->next())
+            n += chunk->computedSizeOfIncludingThis();
+        return n;
+    }
+
     // Like sizeOfExcludingThis(), but includes the size of the LifoAlloc itself.
     size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
         return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
@@ -391,7 +403,7 @@ class LifoAlloc
     // Doesn't perform construction; useful for lazily-initialized POD types.
     template <typename T>
     MOZ_ALWAYS_INLINE
-    T *newPod() {
+    T *pod_malloc() {
         return static_cast<T *>(alloc(sizeof(T)));
     }
 
@@ -411,7 +423,7 @@ class LifoAlloc
         // If there is not enough room in the remaining block for |size|,
         // advance to the next block and update the position.
         void ensureSpaceAndAlignment(size_t size) {
-            JS_ASSERT(!empty());
+            MOZ_ASSERT(!empty());
             char *aligned = detail::AlignPtr(position_);
             if (aligned + size > chunk_->end()) {
                 chunk_ = chunk_->next();
@@ -419,7 +431,7 @@ class LifoAlloc
             } else {
                 position_ = aligned;
             }
-            JS_ASSERT(uintptr_t(position_) + size <= uintptr_t(chunk_->end()));
+            MOZ_ASSERT(uintptr_t(position_) + size <= uintptr_t(chunk_->end()));
         }
 
       public:
@@ -496,7 +508,7 @@ class LifoAllocScope
     }
 
     void releaseEarly() {
-        JS_ASSERT(shouldRelease);
+        MOZ_ASSERT(shouldRelease);
         lifoAlloc->release(mark);
         shouldRelease = false;
     }
@@ -516,21 +528,29 @@ class LifoAllocPolicy
     MOZ_IMPLICIT LifoAllocPolicy(LifoAlloc &alloc)
       : alloc_(alloc)
     {}
-    void *malloc_(size_t bytes) {
-        return fb == Fallible ? alloc_.alloc(bytes) : alloc_.allocInfallible(bytes);
+    template <typename T>
+    T *pod_malloc(size_t numElems) {
+        if (numElems & mozilla::tl::MulOverflowMask<sizeof(T)>::value)
+            return nullptr;
+        size_t bytes = numElems * sizeof(T);
+        void *p = fb == Fallible ? alloc_.alloc(bytes) : alloc_.allocInfallible(bytes);
+        return static_cast<T *>(p);
     }
-    void *calloc_(size_t bytes) {
-        void *p = malloc_(bytes);
+    template <typename T>
+    T *pod_calloc(size_t numElems) {
+        T *p = pod_malloc<T>(numElems);
         if (fb == Fallible && !p)
             return nullptr;
-        memset(p, 0, bytes);
+        memset(p, 0, numElems * sizeof(T));
         return p;
     }
-    void *realloc_(void *p, size_t oldBytes, size_t bytes) {
-        void *n = malloc_(bytes);
+    template <typename T>
+    T *pod_realloc(T *p, size_t oldSize, size_t newSize) {
+        T *n = pod_malloc<T>(newSize);
         if (fb == Fallible && !n)
             return nullptr;
-        memcpy(n, p, Min(oldBytes, bytes));
+        MOZ_ASSERT(!(oldSize & mozilla::tl::MulOverflowMask<sizeof(T)>::value));
+        memcpy(n, p, Min(oldSize * sizeof(T), newSize * sizeof(T)));
         return n;
     }
     void free_(void *p) {
