@@ -119,7 +119,7 @@ AccumulateEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 {
     VerifyPreTracer *trc = (VerifyPreTracer *)jstrc;
 
-    JS_ASSERT(!IsInsideNursery(*reinterpret_cast<Cell **>(thingp)));
+    MOZ_ASSERT(!IsInsideNursery(*reinterpret_cast<Cell **>(thingp)));
 
     trc->edgeptr += sizeof(EdgeValue);
     if (trc->edgeptr >= trc->term) {
@@ -189,8 +189,8 @@ gc::GCRuntime::startVerifyPreBarriers()
     if (!IsIncrementalGCSafe(rt))
         return;
 
-    for (GCChunkSet::Range r(chunkSet.all()); !r.empty(); r.popFront())
-        r.front()->bitmap.clear();
+    for (auto chunk = allNonEmptyChunks(); !chunk.done(); chunk.next())
+        chunk->bitmap.clear();
 
     number++;
 
@@ -264,7 +264,7 @@ oom:
 }
 
 static bool
-IsMarkedOrAllocated(Cell *cell)
+IsMarkedOrAllocated(TenuredCell *cell)
 {
     return cell->isMarked() || cell->arenaHeader()->allocatedDuringIncremental;
 }
@@ -290,7 +290,7 @@ CheckEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 
     for (uint32_t i = 0; i < node->count; i++) {
         if (node->edges[i].thing == *thingp) {
-            JS_ASSERT(node->edges[i].kind == kind);
+            MOZ_ASSERT(node->edges[i].kind == kind);
             node->edges[i].thing = nullptr;
             return;
         }
@@ -300,7 +300,7 @@ CheckEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 static void
 AssertMarkedOrAllocated(const EdgeValue &edge)
 {
-    if (!edge.thing || IsMarkedOrAllocated(static_cast<Cell *>(edge.thing)))
+    if (!edge.thing || IsMarkedOrAllocated(TenuredCell::fromPointer(edge.thing)))
         return;
 
     // Permanent atoms and well-known symbols aren't marked during graph traversal.
@@ -325,7 +325,7 @@ gc::GCRuntime::endVerifyPreBarriers()
     if (!trc)
         return false;
 
-    JS_ASSERT(!JS::IsGenerationalGCEnabled(rt));
+    MOZ_ASSERT(!JS::IsGenerationalGCEnabled(rt));
 
     AutoPrepareForTracing prep(rt, SkipAtoms);
 
@@ -345,7 +345,7 @@ gc::GCRuntime::endVerifyPreBarriers()
      * We need to bump gcNumber so that the methodjit knows that jitcode has
      * been discarded.
      */
-    JS_ASSERT(trc->number == number);
+    MOZ_ASSERT(trc->number == number);
     number++;
 
     verifyPreData = nullptr;
@@ -470,7 +470,7 @@ PostVerifierVisitEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
         return;
 
     /* Filter out non cross-generational edges. */
-    JS_ASSERT(!trc->runtime()->gc.nursery.isInside(thingp));
+    MOZ_ASSERT(!trc->runtime()->gc.nursery.isInside(thingp));
     JSObject *dst = *reinterpret_cast<JSObject **>(thingp);
     if (!IsInsideNursery(dst))
         return;
@@ -478,7 +478,7 @@ PostVerifierVisitEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
     /*
      * Values will be unpacked to the stack before getting here. However, the
      * only things that enter this callback are marked by the JS_TraceChildren
-     * below. Since ObjectImpl::markChildren handles this, the real trace
+     * below. Since JSObject::markChildren handles this, the real trace
      * location will be set correctly in these cases.
      */
     void **loc = trc->tracingLocation(thingp);
@@ -503,7 +503,10 @@ js::gc::GCRuntime::endVerifyPostBarriers()
     if (!edges.init())
         goto oom;
     trc->edges = &edges;
-    storeBuffer.markAll(trc);
+    {
+        gcstats::AutoPhase ap(stats, gcstats::PHASE_MINOR_GC);
+        storeBuffer.markAll(trc);
+    }
 
     /* Walk the heap to find any edges not the the |edges| set. */
     trc->setTraceCallback(PostVerifierVisitEdge);
