@@ -9,13 +9,13 @@ y = x.mapPar(function() {});
 Object.defineProperty(y, 3, {
     get: (function( ... $8)  {
         try {
-            Int8Array(y);
+            new Int8Array(y);
             x.reducePar(function() function q() 1);
         } catch (e) {}
     })
 });
 var x = [1,2,3];
-function rsh() { 
- return y.toSource() >> 2.0; 
+function rsh() {
+ return y.toSource() >> 2.0;
 }
 rsh();
