@@ -112,6 +112,7 @@ static const JSFunctionSpec boolean_methods[] = {
     JS_FN(js_toSource_str,  bool_toSource,  0, 0),
 #endif
     JS_FN(js_toString_str,  bool_toString,  0, 0),
+    JS_FN(js_valueOf_str,   bool_valueOf,   0, 0),
     JS_FS_END
 };
 
@@ -155,20 +156,6 @@ js_InitBooleanClass(JSContext *cx, HandleObject obj)
     if (!DefinePropertiesAndFunctions(cx, booleanProto, nullptr, boolean_methods))
         return nullptr;
 
-    Handle<PropertyName*> valueOfName = cx->names().valueOf;
-    RootedFunction
-        valueOf(cx, NewFunction(cx, NullPtr(), bool_valueOf, 0, JSFunction::NATIVE_FUN,
-                                global, valueOfName));
-    if (!valueOf)
-        return nullptr;
-
-    RootedValue value(cx, ObjectValue(*valueOf));
-    if (!JSObject::defineProperty(cx, booleanProto, valueOfName, value,
-                                  JS_PropertyStub, JS_StrictPropertyStub, 0))
-    {
-        return nullptr;
-    }
-
     if (!GlobalObject::initBuiltinConstructor(cx, global, JSProto_Boolean, ctor, booleanProto))
         return nullptr;
 
@@ -190,15 +177,3 @@ js::ToBooleanSlow(HandleValue v)
     MOZ_ASSERT(v.isObject());
     return !EmulatesUndefined(&v.toObject());
 }
-
-/*
- * This slow path is only ever taken for proxies wrapping Boolean objects
- * The only caller of the fast path, JSON's PreprocessValue, ensures that.
- */
-bool
-js::BooleanGetPrimitiveValueSlow(HandleObject wrappedBool)
-{
-    JSObject *obj = wrappedBool->as<ProxyObject>().target();
-    MOZ_ASSERT(obj);
-    return obj->as<BooleanObject>().unbox();
-}
