@@ -4,11 +4,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef jscrashreport_h___
-#define jscrashreport_h___
+#ifndef jscrashreport_h
+#define jscrashreport_h
 
 #include "mozilla/GuardObjects.h"
-#include "mozilla/StandardInteger.h"
+
+#include <stdint.h>
 
 namespace js {
 namespace crash {
@@ -26,8 +27,8 @@ template<size_t size, unsigned char marker>
 class StackBuffer
 {
   public:
-    StackBuffer(void *data
-                MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+    explicit StackBuffer(void *data
+                         MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
     {
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
 
@@ -53,4 +54,4 @@ class StackBuffer
 } /* namespace crash */
 } /* namespace js */
 
-#endif /* jscrashreport_h___ */
+#endif /* jscrashreport_h */
