@@ -92,7 +92,7 @@ NativeObject::setDenseElementWithType(ExclusiveContext *cx, uint32_t index,
     // Avoid a slow AddTypePropertyId call if the type is the same as the type
     // of the previous element.
     types::Type thisType = types::GetValueType(val);
-    if (index == 0 || types::GetValueType(elements[index - 1]) != thisType)
+    if (index == 0 || types::GetValueType(elements_[index - 1]) != thisType)
         types::AddTypePropertyId(cx, this, JSID_VOID, thisType);
     setDenseElementMaybeConvertDouble(index, val);
 }
@@ -154,8 +154,8 @@ NativeObject::ensureDenseInitializedLengthNoPackedCheck(ThreadSafeContext *cx, u
 
     if (initlen < index + extra) {
         size_t offset = initlen;
-        for (HeapSlot *sp = elements + initlen;
-             sp != elements + (index + extra);
+        for (HeapSlot *sp = elements_ + initlen;
+             sp != elements_ + (index + extra);
              sp++, offset++)
         {
             sp->init(this, HeapSlot::Element, offset, MagicValue(JS_ELEMENTS_HOLE));
@@ -308,7 +308,7 @@ NativeObject::initDenseElementsUnbarriered(uint32_t dstStart, const Value *src,
             MOZ_ASSERT(!gc::IsInsideGGCNursery(static_cast<gc::Cell *>(value.toGCThing())));
     }
 #endif
-    memcpy(&elements[dstStart], src, count * sizeof(HeapSlot));
+    memcpy(&elements_[dstStart], src, count * sizeof(HeapSlot));
 }
 
 /* static */ inline NativeObject *
@@ -484,9 +484,6 @@ static MOZ_ALWAYS_INLINE bool
 CallResolveOp(JSContext *cx, HandleNativeObject obj, HandleId id, MutableHandleObject objp,
               MutableHandleShape propp, bool *recursedp)
 {
-    const Class *clasp = obj->getClass();
-    JSResolveOp resolve = clasp->resolve;
-
     /*
      * Avoid recursion on (obj, id) already being resolved on cx.
      *
@@ -503,46 +500,21 @@ CallResolveOp(JSContext *cx, HandleNativeObject obj, HandleId id, MutableHandleO
     }
     *recursedp = false;
 
-    propp.set(nullptr);
+    bool resolved = false;
+    if (!obj->getClass()->resolve(cx, obj, id, &resolved))
+        return false;
 
-    if (clasp->flags & JSCLASS_NEW_RESOLVE) {
-        JSNewResolveOp newresolve = reinterpret_cast<JSNewResolveOp>(resolve);
-        RootedObject obj2(cx, nullptr);
-        if (!newresolve(cx, obj, id, &obj2))
-            return false;
+    if (!resolved)
+        return true;
 
-        /*
-         * We trust the new style resolve hook to set obj2 to nullptr when
-         * the id cannot be resolved. But, when obj2 is not null, we do
-         * not assume that id must exist and do full nativeLookup for
-         * compatibility.
-         */
-        if (!obj2)
-            return true;
+    objp.set(obj);
 
-        if (!obj2->isNative()) {
-            /* Whoops, newresolve handed back a foreign obj2. */
-            MOZ_ASSERT(obj2 != obj);
-            return JSObject::lookupGeneric(cx, obj2, id, objp, propp);
-        }
-
-        objp.set(obj2);
-    } else {
-        if (!resolve(cx, obj, id))
-            return false;
-
-        objp.set(obj);
-    }
-
-    NativeObject *nobjp = &objp->as<NativeObject>();
-
-    if (JSID_IS_INT(id) && nobjp->containsDenseElement(JSID_TO_INT(id))) {
+    if (JSID_IS_INT(id) && obj->containsDenseElement(JSID_TO_INT(id))) {
         MarkDenseOrTypedArrayElementFound<CanGC>(propp);
         return true;
     }
 
-    Shape *shape;
-    if (!nobjp->empty() && (shape = nobjp->lookup(cx, id)))
+    if (Shape *shape = obj->lookup(cx, id))
         propp.set(shape);
     else
         objp.set(nullptr);
@@ -680,12 +652,15 @@ DefineNativeProperty(ExclusiveContext *cx, HandleNativeObject obj,
     return DefineNativeProperty(cx, obj, id, value, getter, setter, attrs);
 }
 
-} // namespace js
-
-inline uint8_t *
-JSObject::fakeNativeFixedData(size_t nslots) const
+inline bool
+WarnIfNotConstructing(JSContext *cx, const CallArgs &args, const char *builtinName)
 {
-    return static_cast<const js::NativeObject*>(this)->fixedData(nslots);
+    if (args.isConstructing())
+        return true;
+    return JS_ReportErrorFlagsAndNumber(cx, JSREPORT_WARNING, js_GetErrorMessage, nullptr,
+                                        JSMSG_BUILTIN_CTOR_NO_NEW, builtinName);
 }
 
+} // namespace js
+
 #endif /* vm_NativeObject_inl_h */
