@@ -15,7 +15,7 @@
 
 #include "vm/Interpreter.h"
 #include "vm/ScopeObject.h"
-#include "vm/TypedArrayObject.h"
+#include "vm/TypedArrayCommon.h"
 
 #include "jsatominlines.h"
 #include "jscntxtinlines.h"
@@ -30,8 +30,6 @@ StackBaseShape::StackBaseShape(ThreadSafeContext *cx, const Class *clasp,
     clasp(clasp),
     parent(parent),
     metadata(metadata),
-    rawGetter(nullptr),
-    rawSetter(nullptr),
     compartment(cx->compartment_)
 {}
 
@@ -39,7 +37,7 @@ inline bool
 Shape::get(JSContext* cx, HandleObject receiver, JSObject* obj, JSObject *pobj,
            MutableHandleValue vp)
 {
-    JS_ASSERT(!hasDefaultGetter());
+    MOZ_ASSERT(!hasDefaultGetter());
 
     if (hasGetterValue()) {
         Value fval = getterValue();
@@ -72,7 +70,7 @@ Shape::searchThreadLocal(ThreadSafeContext *cx, Shape *start, jsid id,
      * allocated thread locally. In that case, we may add to the
      * table. Otherwise it is not allowed.
      */
-    JS_ASSERT_IF(adding, cx->isThreadLocal(start) && start->inDictionary());
+    MOZ_ASSERT_IF(adding, cx->isThreadLocal(start) && start->inDictionary());
 
     if (start->inDictionary()) {
         *pspp = start->table().search(id, adding);
@@ -88,7 +86,7 @@ inline bool
 Shape::set(JSContext* cx, HandleObject obj, HandleObject receiver, bool strict,
            MutableHandleValue vp)
 {
-    JS_ASSERT_IF(hasDefaultSetter(), hasGetterValue());
+    MOZ_ASSERT_IF(hasDefaultSetter(), hasGetterValue());
 
     if (attrs & JSPROP_SETTER) {
         Value fval = setterValue();
@@ -140,7 +138,7 @@ Shape::search(ExclusiveContext *cx, Shape *start, jsid id, Shape ***pspp, bool a
          * No table built -- there weren't enough entries, or OOM occurred.
          * Don't increment numLinearSearches, to keep hasTable() false.
          */
-        JS_ASSERT(!start->hasTable());
+        MOZ_ASSERT(!start->hasTable());
     } else {
         start->incrementNumLinearSearches();
     }
@@ -153,6 +151,24 @@ Shape::search(ExclusiveContext *cx, Shape *start, jsid id, Shape ***pspp, bool a
     return nullptr;
 }
 
+inline Shape *
+Shape::new_(ExclusiveContext *cx, StackShape &unrootedOther, uint32_t nfixed)
+{
+    RootedGeneric<StackShape*> other(cx, &unrootedOther);
+    Shape *shape = other->isAccessorShape() ? NewGCAccessorShape(cx) : NewGCShape(cx);
+    if (!shape) {
+        js_ReportOutOfMemory(cx);
+        return nullptr;
+    }
+
+    if (other->isAccessorShape())
+        new (shape) AccessorShape(*other, nfixed);
+    else
+        new (shape) Shape(*other, nfixed);
+
+    return shape;
+}
+
 template<class ObjectSubclass>
 /* static */ inline bool
 EmptyShape::ensureInitialCustomShape(ExclusiveContext *cx, Handle<ObjectSubclass*> obj)
@@ -162,14 +178,14 @@ EmptyShape::ensureInitialCustomShape(ExclusiveContext *cx, Handle<ObjectSubclass
 
     // If the provided object has a non-empty shape, it was given the cached
     // initial shape when created: nothing to do.
-    if (!obj->nativeEmpty())
+    if (!obj->empty())
         return true;
 
     // If no initial shape was assigned, do so.
     RootedShape shape(cx, ObjectSubclass::assignInitialShape(cx, obj));
     if (!shape)
         return false;
-    MOZ_ASSERT(!obj->nativeEmpty());
+    MOZ_ASSERT(!obj->empty());
 
     // If the object is a standard prototype -- |RegExp.prototype|,
     // |String.prototype|, |RangeError.prototype|, &c. -- GlobalObject.cpp's
@@ -192,8 +208,8 @@ AutoRooterGetterSetter::Inner::Inner(ThreadSafeContext *cx, uint8_t attrs,
   : CustomAutoRooter(cx), attrs(attrs),
     pgetter(pgetter_), psetter(psetter_)
 {
-    JS_ASSERT_IF(attrs & JSPROP_GETTER, !IsPoisonedPtr(*pgetter));
-    JS_ASSERT_IF(attrs & JSPROP_SETTER, !IsPoisonedPtr(*psetter));
+    MOZ_ASSERT_IF(attrs & JSPROP_GETTER, !IsPoisonedPtr(*pgetter));
+    MOZ_ASSERT_IF(attrs & JSPROP_SETTER, !IsPoisonedPtr(*psetter));
 }
 
 inline
@@ -206,13 +222,25 @@ AutoRooterGetterSetter::AutoRooterGetterSetter(ThreadSafeContext *cx, uint8_t at
     MOZ_GUARD_OBJECT_NOTIFIER_INIT;
 }
 
+inline
+AutoRooterGetterSetter::AutoRooterGetterSetter(ThreadSafeContext *cx, uint8_t attrs,
+                                               JSNative *pgetter, JSNative *psetter
+                                               MOZ_GUARD_OBJECT_NOTIFIER_PARAM_IN_IMPL)
+{
+    if (attrs & (JSPROP_GETTER | JSPROP_SETTER)) {
+        inner.emplace(cx, attrs, reinterpret_cast<PropertyOp *>(pgetter),
+                      reinterpret_cast<StrictPropertyOp *>(psetter));
+    }
+    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
+}
+
 static inline uint8_t
 GetShapeAttributes(JSObject *obj, Shape *shape)
 {
-    JS_ASSERT(obj->isNative());
+    MOZ_ASSERT(obj->isNative());
 
     if (IsImplicitDenseOrTypedArrayElement(shape)) {
-        if (obj->is<TypedArrayObject>())
+        if (IsAnyTypedArray(obj))
             return JSPROP_ENUMERATE | JSPROP_PERMANENT;
         return JSPROP_ENUMERATE;
     }
