@@ -10,10 +10,9 @@
 #include "prprf.h"
 #include "nsFrame.h"
 #include "nsPresArena.h"
-#ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
-#endif
 #include "nsIFrameInlines.h"
+#include "WritingModes.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Likely.h"
 #include "nsPrintfCString.h"
@@ -28,8 +27,13 @@ int32_t nsLineBox::GetCtorCount() { return ctorCount; }
 const uint32_t nsLineBox::kMinChildCountForHashtable;
 #endif
 
+using namespace mozilla;
+
 nsLineBox::nsLineBox(nsIFrame* aFrame, int32_t aCount, bool aIsBlock)
   : mFirstChild(aFrame)
+  , mContainerWidth(-1)
+  , mBounds(WritingMode()) // mBounds will be initialized with the correct
+                           // writing mode when it is set
 // NOTE: memory is already zeroed since we allocate with AllocateByObjectID.
 {
   MOZ_COUNT_CTOR(nsLineBox);
@@ -75,6 +79,7 @@ NS_NewLineBox(nsIPresShell* aPresShell, nsLineBox* aFromLine,
 {
   nsLineBox* newLine = new (aPresShell) nsLineBox(aFrame, aCount, false);
   newLine->NoteFramesMovedFrom(aFromLine);
+  newLine->mContainerWidth = aFromLine->mContainerWidth;
   return newLine;
 }
 
@@ -238,14 +243,15 @@ nsLineBox::List(FILE* out, const char* aPrefix, uint32_t aFlags) const
   str += nsPrintfCString("line %p: count=%d state=%s ",
           static_cast<const void*>(this), GetChildCount(),
           StateToString(cbuf, sizeof(cbuf)));
-  if (IsBlock() && !GetCarriedOutBottomMargin().IsZero()) {
-    str += nsPrintfCString("bm=%d ", GetCarriedOutBottomMargin().get());
+  if (IsBlock() && !GetCarriedOutBEndMargin().IsZero()) {
+    str += nsPrintfCString("bm=%d ", GetCarriedOutBEndMargin().get());
   }
+  nsRect bounds = GetPhysicalBounds();
   str += nsPrintfCString("{%d,%d,%d,%d} ",
-          mBounds.x, mBounds.y, mBounds.width, mBounds.height);
+          bounds.x, bounds.y, bounds.width, bounds.height);
   if (mData &&
-      (!mData->mOverflowAreas.VisualOverflow().IsEqualEdges(mBounds) ||
-       !mData->mOverflowAreas.ScrollableOverflow().IsEqualEdges(mBounds))) {
+      (!mData->mOverflowAreas.VisualOverflow().IsEqualEdges(bounds) ||
+       !mData->mOverflowAreas.ScrollableOverflow().IsEqualEdges(bounds))) {
     str += nsPrintfCString("vis-overflow=%d,%d,%d,%d scr-overflow=%d,%d,%d,%d ",
             mData->mOverflowAreas.VisualOverflow().x,
             mData->mOverflowAreas.VisualOverflow().y,
@@ -424,30 +430,30 @@ nsLineBox::RFindLineContaining(nsIFrame* aFrame,
 }
 
 nsCollapsingMargin
-nsLineBox::GetCarriedOutBottomMargin() const
+nsLineBox::GetCarriedOutBEndMargin() const
 {
   NS_ASSERTION(IsBlock(),
-               "GetCarriedOutBottomMargin called on non-block line.");
+               "GetCarriedOutBEndMargin called on non-block line.");
   return (IsBlock() && mBlockData)
-    ? mBlockData->mCarriedOutBottomMargin
+    ? mBlockData->mCarriedOutBEndMargin
     : nsCollapsingMargin();
 }
 
 bool
-nsLineBox::SetCarriedOutBottomMargin(nsCollapsingMargin aValue)
+nsLineBox::SetCarriedOutBEndMargin(nsCollapsingMargin aValue)
 {
   bool changed = false;
   if (IsBlock()) {
     if (!aValue.IsZero()) {
       if (!mBlockData) {
-        mBlockData = new ExtraBlockData(mBounds);
+        mBlockData = new ExtraBlockData(GetPhysicalBounds());
       }
-      changed = aValue != mBlockData->mCarriedOutBottomMargin;
-      mBlockData->mCarriedOutBottomMargin = aValue;
+      changed = aValue != mBlockData->mCarriedOutBEndMargin;
+      mBlockData->mCarriedOutBEndMargin = aValue;
     }
     else if (mBlockData) {
-      changed = aValue != mBlockData->mCarriedOutBottomMargin;
-      mBlockData->mCarriedOutBottomMargin = aValue;
+      changed = aValue != mBlockData->mCarriedOutBEndMargin;
+      mBlockData->mCarriedOutBEndMargin = aValue;
       MaybeFreeData();
     }
   }
@@ -457,14 +463,15 @@ nsLineBox::SetCarriedOutBottomMargin(nsCollapsingMargin aValue)
 void
 nsLineBox::MaybeFreeData()
 {
-  if (mData && mData->mOverflowAreas == nsOverflowAreas(mBounds, mBounds)) {
+  nsRect bounds = GetPhysicalBounds();
+  if (mData && mData->mOverflowAreas == nsOverflowAreas(bounds, bounds)) {
     if (IsInline()) {
       if (mInlineData->mFloats.IsEmpty()) {
         delete mInlineData;
         mInlineData = nullptr;
       }
     }
-    else if (mBlockData->mCarriedOutBottomMargin.IsZero()) {
+    else if (mBlockData->mCarriedOutBEndMargin.IsZero()) {
       delete mBlockData;
       mBlockData = nullptr;
     }
@@ -499,7 +506,7 @@ nsLineBox::AppendFloats(nsFloatCacheFreeList& aFreeList)
   if (IsInline()) {
     if (aFreeList.NotEmpty()) {
       if (!mInlineData) {
-        mInlineData = new ExtraInlineData(mBounds);
+        mInlineData = new ExtraInlineData(GetPhysicalBounds());
       }
       mInlineData->mFloats.Append(aFreeList);
     }
@@ -533,14 +540,15 @@ nsLineBox::SetOverflowAreas(const nsOverflowAreas& aOverflowAreas)
     NS_ASSERTION(aOverflowAreas.Overflow(otype).height >= 0,
                  "illegal height for combined area");
   }
-  if (!aOverflowAreas.VisualOverflow().IsEqualInterior(mBounds) ||
-      !aOverflowAreas.ScrollableOverflow().IsEqualEdges(mBounds)) {
+  nsRect bounds = GetPhysicalBounds();
+  if (!aOverflowAreas.VisualOverflow().IsEqualInterior(bounds) ||
+      !aOverflowAreas.ScrollableOverflow().IsEqualEdges(bounds)) {
     if (!mData) {
       if (IsInline()) {
-        mInlineData = new ExtraInlineData(mBounds);
+        mInlineData = new ExtraInlineData(bounds);
       }
       else {
-        mBlockData = new ExtraBlockData(mBounds);
+        mBlockData = new ExtraBlockData(bounds);
       }
     }
     mData->mOverflowAreas = aOverflowAreas;
@@ -644,7 +652,7 @@ nsLineIterator::GetLine(int32_t aLineNumber,
   nsLineBox* line = mLines[aLineNumber];
   *aFirstFrameOnLine = line->mFirstChild;
   *aNumFramesOnLine = line->GetChildCount();
-  aLineBounds = line->mBounds;
+  aLineBounds = line->GetPhysicalBounds();
 
   uint32_t flags = 0;
   if (line->IsBlock()) {
@@ -674,7 +682,6 @@ nsLineIterator::FindLineContaining(nsIFrame* aFrame, int32_t aStartLine)
   return -1;
 }
 
-#ifdef IBMBIDI
 NS_IMETHODIMP
 nsLineIterator::CheckLineOrder(int32_t                  aLine,
                                bool                     *aIsReordered,
@@ -701,18 +708,17 @@ nsLineIterator::CheckLineOrder(int32_t                  aLine,
 
   return NS_OK;
 }
-#endif // IBMBIDI
 
 NS_IMETHODIMP
 nsLineIterator::FindFrameAt(int32_t aLineNumber,
-                            nscoord aX,
+                            nsPoint aPos,
                             nsIFrame** aFrameFound,
-                            bool* aXIsBeforeFirstFrame,
-                            bool* aXIsAfterLastFrame)
+                            bool* aPosIsBeforeFirstFrame,
+                            bool* aPosIsAfterLastFrame)
 {
-  NS_PRECONDITION(aFrameFound && aXIsBeforeFirstFrame && aXIsAfterLastFrame,
+  NS_PRECONDITION(aFrameFound && aPosIsBeforeFirstFrame && aPosIsAfterLastFrame,
                   "null OUT ptr");
-  if (!aFrameFound || !aXIsBeforeFirstFrame || !aXIsAfterLastFrame) {
+  if (!aFrameFound || !aPosIsBeforeFirstFrame || !aPosIsAfterLastFrame) {
     return NS_ERROR_NULL_POINTER;
   }
   if ((aLineNumber < 0) || (aLineNumber >= mNumLines)) {
@@ -722,60 +728,67 @@ nsLineIterator::FindFrameAt(int32_t aLineNumber,
   nsLineBox* line = mLines[aLineNumber];
   if (!line) {
     *aFrameFound = nullptr;
-    *aXIsBeforeFirstFrame = true;
-    *aXIsAfterLastFrame = false;
+    *aPosIsBeforeFirstFrame = true;
+    *aPosIsAfterLastFrame = false;
     return NS_OK;
   }
 
-  if (line->mBounds.width == 0 && line->mBounds.height == 0)
+  if (line->ISize() == 0 && line->BSize() == 0)
     return NS_ERROR_FAILURE;
 
   nsIFrame* frame = line->mFirstChild;
-  nsIFrame* closestFromLeft = nullptr;
-  nsIFrame* closestFromRight = nullptr;
+  nsIFrame* closestFromStart = nullptr;
+  nsIFrame* closestFromEnd = nullptr;
+
+  WritingMode wm = line->mWritingMode;
+  nscoord cw = line->mContainerWidth;
+
+  LogicalPoint pos(wm, aPos, cw);
+
   int32_t n = line->GetChildCount();
   while (n--) {
-    nsRect rect = frame->GetRect();
-    if (rect.width > 0) {
-      // If aX is inside this frame - this is it
-      if (rect.x <= aX && rect.XMost() > aX) {
-        closestFromLeft = closestFromRight = frame;
+    LogicalRect rect = frame->GetLogicalRect(wm, cw);
+    if (rect.ISize(wm) > 0) {
+      // If pos.I() is inside this frame - this is it
+      if (rect.IStart(wm) <= pos.I(wm) && rect.IEnd(wm) > pos.I(wm)) {
+        closestFromStart = closestFromEnd = frame;
         break;
       }
-      if (rect.x < aX) {
-        if (!closestFromLeft || 
-            rect.XMost() > closestFromLeft->GetRect().XMost())
-          closestFromLeft = frame;
+      if (rect.IStart(wm) < pos.I(wm)) {
+        if (!closestFromStart || 
+            rect.IEnd(wm) > closestFromStart->GetLogicalRect(wm, cw).IEnd(wm))
+          closestFromStart = frame;
       }
       else {
-        if (!closestFromRight ||
-            rect.x < closestFromRight->GetRect().x)
-          closestFromRight = frame;
+        if (!closestFromEnd ||
+            rect.IStart(wm) < closestFromEnd->GetLogicalRect(wm, cw).IStart(wm))
+          closestFromEnd = frame;
       }
     }
     frame = frame->GetNextSibling();
   }
-  if (!closestFromLeft && !closestFromRight) {
+  if (!closestFromStart && !closestFromEnd) {
     // All frames were zero-width. Just take the first one.
-    closestFromLeft = closestFromRight = line->mFirstChild;
+    closestFromStart = closestFromEnd = line->mFirstChild;
   }
-  *aXIsBeforeFirstFrame = mRightToLeft ? !closestFromRight : !closestFromLeft;
-  *aXIsAfterLastFrame = mRightToLeft ? !closestFromLeft : !closestFromRight;
-  if (closestFromLeft == closestFromRight) {
-    *aFrameFound = closestFromLeft;
+  *aPosIsBeforeFirstFrame = mRightToLeft ? !closestFromEnd : !closestFromStart;
+  *aPosIsAfterLastFrame = mRightToLeft ? !closestFromStart : !closestFromEnd;
+  if (closestFromStart == closestFromEnd) {
+    *aFrameFound = closestFromStart;
   }
-  else if (!closestFromLeft) {
-    *aFrameFound = closestFromRight;
+  else if (!closestFromStart) {
+    *aFrameFound = closestFromEnd;
   }
-  else if (!closestFromRight) {
-    *aFrameFound = closestFromLeft;
+  else if (!closestFromEnd) {
+    *aFrameFound = closestFromStart;
   }
   else { // we're between two frames
-    nscoord delta = closestFromRight->GetRect().x - closestFromLeft->GetRect().XMost();
-    if (aX < closestFromLeft->GetRect().XMost() + delta/2)
-      *aFrameFound = closestFromLeft;
+    nscoord delta = closestFromEnd->GetLogicalRect(wm, cw).IStart(wm) -
+                    closestFromStart->GetLogicalRect(wm, cw).IEnd(wm);
+    if (pos.I(wm) < closestFromStart->GetLogicalRect(wm, cw).IEnd(wm) + delta/2)
+      *aFrameFound = closestFromStart;
     else
-      *aFrameFound = closestFromRight;
+      *aFrameFound = closestFromEnd;
   }
   return NS_OK;
 }
