@@ -39,9 +39,7 @@ CSS_KEY(-moz-activehyperlinktext, _moz_activehyperlinktext)
 CSS_KEY(-moz-all, _moz_all)
 CSS_KEY(-moz-alt-content, _moz_alt_content)
 CSS_KEY(-moz-anchor-decoration, _moz_anchor_decoration)
-CSS_KEY(-moz-arabic-indic, _moz_arabic_indic)
 CSS_KEY(-moz-available, _moz_available)
-CSS_KEY(-moz-bengali, _moz_bengali)
 CSS_KEY(-moz-box, _moz_box)
 CSS_KEY(-moz-button, _moz_button)
 CSS_KEY(-moz-buttondefault, _moz_buttondefault)
@@ -50,8 +48,6 @@ CSS_KEY(-moz-buttonhovertext, _moz_buttonhovertext)
 CSS_KEY(-moz-cellhighlight, _moz_cellhighlight)
 CSS_KEY(-moz-cellhighlighttext, _moz_cellhighlighttext)
 CSS_KEY(-moz-center, _moz_center)
-CSS_KEY(-moz-cjk-earthly-branch, _moz_cjk_earthly_branch)
-CSS_KEY(-moz-cjk-heavenly-stem, _moz_cjk_heavenly_stem)
 CSS_KEY(-moz-combobox, _moz_combobox)
 CSS_KEY(-moz-comboboxtext, _moz_comboboxtext)
 CSS_KEY(-moz-block-height, _moz_block_height)
@@ -59,32 +55,23 @@ CSS_KEY(-moz-deck, _moz_deck)
 CSS_KEY(-moz-default-background-color, _moz_default_background_color)
 CSS_KEY(-moz-default-color, _moz_default_color)
 CSS_KEY(-moz-desktop, _moz_desktop)
-CSS_KEY(-moz-devanagari, _moz_devanagari)
 CSS_KEY(-moz-dialog, _moz_dialog)
 CSS_KEY(-moz-dialogtext, _moz_dialogtext)
 CSS_KEY(-moz-document, _moz_document)
 CSS_KEY(-moz-dragtargetzone, _moz_dragtargetzone)
 CSS_KEY(-moz-element, _moz_element)
 CSS_KEY(-moz-eventreerow, _moz_eventreerow)
-CSS_KEY(-moz-ethiopic-halehame, _moz_ethiopic_halehame)
-CSS_KEY(-moz-ethiopic-numeric, _moz_ethiopic_numeric)
-CSS_KEY(-moz-ethiopic-halehame-am, _moz_ethiopic_halehame_am)
-CSS_KEY(-moz-ethiopic-halehame-ti-er, _moz_ethiopic_halehame_ti_er)
-CSS_KEY(-moz-ethiopic-halehame-ti-et, _moz_ethiopic_halehame_ti_et)
 CSS_KEY(-moz-field, _moz_field)
 CSS_KEY(-moz-fieldtext, _moz_fieldtext)
 CSS_KEY(-moz-fit-content, _moz_fit_content)
+CSS_KEY(-moz-fixed, _moz_fixed)
 CSS_KEY(-moz-grabbing, _moz_grabbing)
 CSS_KEY(-moz-grab, _moz_grab)
 CSS_KEY(-moz-grid-group, _moz_grid_group)
 CSS_KEY(-moz-grid-line, _moz_grid_line)
 CSS_KEY(-moz-grid, _moz_grid)
 CSS_KEY(-moz-groupbox, _moz_groupbox)
-CSS_KEY(-moz-gujarati, _moz_gujarati)
-CSS_KEY(-moz-gurmukhi, _moz_gurmukhi)
-CSS_KEY(-moz-hangul-consonant, _moz_hangul_consonant)
 CSS_KEY(-moz-hidden-unscrollable, _moz_hidden_unscrollable)
-CSS_KEY(-moz-hangul, _moz_hangul)
 CSS_KEY(-moz-hyperlinktext, _moz_hyperlinktext)
 CSS_KEY(-moz-html-cellhighlight, _moz_html_cellhighlight)
 CSS_KEY(-moz-html-cellhighlighttext, _moz_html_cellhighlighttext)
@@ -95,15 +82,12 @@ CSS_KEY(-moz-inline-grid, _moz_inline_grid)
 CSS_KEY(-moz-inline-stack, _moz_inline_stack)
 CSS_KEY(-moz-isolate, _moz_isolate)
 CSS_KEY(-moz-isolate-override, _moz_isolate_override)
-CSS_KEY(-moz-japanese-formal, _moz_japanese_formal)
-CSS_KEY(-moz-japanese-informal, _moz_japanese_informal)
-CSS_KEY(-moz-kannada, _moz_kannada)
-CSS_KEY(-moz-khmer, _moz_khmer)
-CSS_KEY(-moz-lao, _moz_lao)
 CSS_KEY(-moz-left, _moz_left)
 CSS_KEY(-moz-list, _moz_list)
+CSS_KEY(-moz-mac-buttonactivetext, _moz_mac_buttonactivetext)
 CSS_KEY(-moz-mac-chrome-active, _moz_mac_chrome_active)
 CSS_KEY(-moz-mac-chrome-inactive, _moz_mac_chrome_inactive)
+CSS_KEY(-moz-mac-defaultbuttontext, _moz_mac_defaultbuttontext)
 CSS_KEY(-moz-mac-focusring, _moz_mac_focusring)
 CSS_KEY(-moz-mac-fullscreen-button, _moz_mac_fullscreen_button)
 CSS_KEY(-moz-mac-menuselect, _moz_mac_menuselect)
@@ -112,7 +96,6 @@ CSS_KEY(-moz-mac-menutextdisable, _moz_mac_menutextdisable)
 CSS_KEY(-moz-mac-menutextselect, _moz_mac_menutextselect)
 CSS_KEY(-moz-mac-disabledtoolbartext, _moz_mac_disabledtoolbartext)
 CSS_KEY(-moz-mac-secondaryhighlight, _moz_mac_secondaryhighlight)
-CSS_KEY(-moz-malayalam, _moz_malayalam)
 CSS_KEY(-moz-max-content, _moz_max_content)
 CSS_KEY(-moz-menuhover, _moz_menuhover)
 CSS_KEY(-moz-menuhovertext, _moz_menuhovertext)
@@ -120,30 +103,19 @@ CSS_KEY(-moz-menubartext, _moz_menubartext)
 CSS_KEY(-moz-menubarhovertext, _moz_menubarhovertext)
 CSS_KEY(-moz-middle-with-baseline, _moz_middle_with_baseline)
 CSS_KEY(-moz-min-content, _moz_min_content)
-CSS_KEY(-moz-myanmar, _moz_myanmar)
 CSS_KEY(-moz-nativehyperlinktext, _moz_nativehyperlinktext)
 CSS_KEY(-moz-none, _moz_none)
 CSS_KEY(-moz-oddtreerow, _moz_oddtreerow)
-CSS_KEY(-moz-oriya, _moz_oriya)
-CSS_KEY(-moz-persian, _moz_persian)
 CSS_KEY(-moz-plaintext, _moz_plaintext)
 CSS_KEY(-moz-popup, _moz_popup)
-CSS_KEY(-moz-pre-discard-newlines, _moz_pre_discard_newlines)
+CSS_KEY(-moz-pre-space, _moz_pre_space)
 CSS_KEY(-moz-pull-down-menu, _moz_pull_down_menu)
 CSS_KEY(-moz-right, _moz_right)
 CSS_KEY(-moz-scrollbars-horizontal, _moz_scrollbars_horizontal)
 CSS_KEY(-moz-scrollbars-none, _moz_scrollbars_none)
 CSS_KEY(-moz-scrollbars-vertical, _moz_scrollbars_vertical)
 CSS_KEY(-moz-show-background, _moz_show_background)
-CSS_KEY(-moz-simp-chinese-formal, _moz_simp_chinese_formal)
-CSS_KEY(-moz-simp-chinese-informal, _moz_simp_chinese_informal)
 CSS_KEY(-moz-stack, _moz_stack)
-CSS_KEY(-moz-tamil, _moz_tamil)
-CSS_KEY(-moz-telugu, _moz_telugu)
-CSS_KEY(-moz-thai, _moz_thai)
-CSS_KEY(-moz-trad-chinese-formal, _moz_trad_chinese_formal)
-CSS_KEY(-moz-trad-chinese-informal, _moz_trad_chinese_informal)
-CSS_KEY(-moz-urdu, _moz_urdu)
 CSS_KEY(-moz-use-system-font, _moz_use_system_font)
 CSS_KEY(-moz-use-text-color, _moz_use_text_color)
 CSS_KEY(-moz-visitedhyperlinktext, _moz_visitedhyperlinktext)
@@ -155,6 +127,7 @@ CSS_KEY(absolute, absolute)
 CSS_KEY(active, active)
 CSS_KEY(activeborder, activeborder)
 CSS_KEY(activecaption, activecaption)
+CSS_KEY(additive, additive)
 CSS_KEY(alias, alias)
 CSS_KEY(all, all)
 CSS_KEY(all-petite-caps, all_petite_caps)
@@ -166,7 +139,6 @@ CSS_KEY(alternate-reverse, alternate_reverse)
 CSS_KEY(always, always)
 CSS_KEY(annotation, annotation)
 CSS_KEY(appworkspace, appworkspace)
-CSS_KEY(armenian, armenian)
 CSS_KEY(auto, auto)
 CSS_KEY(avoid, avoid)
 CSS_KEY(background, background)
@@ -188,10 +160,10 @@ CSS_KEY(border-box, border_box)
 CSS_KEY(both, both)
 CSS_KEY(bottom, bottom)
 CSS_KEY(bottom-outside, bottom_outside)
-CSS_KEY(bounding-box, bounding_box)
 CSS_KEY(break-all, break_all)
 CSS_KEY(break-word, break_word)
 CSS_KEY(brightness, brightness)
+CSS_KEY(bullets, bullets)
 CSS_KEY(button, button)
 CSS_KEY(buttonface, buttonface)
 CSS_KEY(buttonhighlight, buttonhighlight)
@@ -206,8 +178,8 @@ CSS_KEY(ch, ch)
 CSS_KEY(character-variant, character_variant)
 CSS_KEY(circle, circle)
 CSS_KEY(cjk-decimal, cjk_decimal)
-CSS_KEY(cjk-ideographic, cjk_ideographic)
 CSS_KEY(clip, clip)
+CSS_KEY(clone, clone)
 CSS_KEY(close-quote, close_quote)
 CSS_KEY(closest-corner, closest_corner)
 CSS_KEY(closest-side, closest_side)
@@ -223,6 +195,7 @@ CSS_KEY(column-reverse, column_reverse)
 CSS_KEY(condensed, condensed)
 CSS_KEY(contain, contain)
 CSS_KEY(content-box, content_box)
+CSS_KEY(contents, contents)
 CSS_KEY(context-fill, context_fill)
 CSS_KEY(context-fill-opacity, context_fill_opacity)
 CSS_KEY(context-menu, context_menu)
@@ -238,11 +211,12 @@ CSS_KEY(crop, crop)
 CSS_KEY(cross, cross)
 CSS_KEY(crosshair, crosshair)
 CSS_KEY(currentcolor, currentcolor)
+CSS_KEY(cursive, cursive)
+CSS_KEY(cyclic, cyclic)
 CSS_KEY(darken, darken)
 CSS_KEY(dashed, dashed)
 CSS_KEY(dense, dense)
 CSS_KEY(decimal, decimal)
-CSS_KEY(decimal-leading-zero, decimal_leading_zero)
 CSS_KEY(default, default)
 CSS_KEY(deg, deg)
 CSS_KEY(diagonal-fractions, diagonal_fractions)
@@ -251,13 +225,15 @@ CSS_KEY(difference, difference)
 CSS_KEY(digits, digits)
 CSS_KEY(disabled, disabled)
 CSS_KEY(disc, disc)
+CSS_KEY(disclosure-closed, disclosure_closed)
+CSS_KEY(disclosure-open, disclosure_open)
 CSS_KEY(discretionary-ligatures, discretionary_ligatures)
 CSS_KEY(dotted, dotted)
 CSS_KEY(double, double)
 CSS_KEY(double-struck, double_struck)
+CSS_KEY(drag, drag)
 CSS_KEY(drop-shadow, drop_shadow)
 CSS_KEY(e-resize, e_resize)
-CSS_KEY(each-box, each_box)
 CSS_KEY(ease, ease)
 CSS_KEY(ease-in, ease_in)
 CSS_KEY(ease-in-out, ease_in_out)
@@ -270,15 +246,19 @@ CSS_KEY(em, em)
 CSS_KEY(embed, embed)
 CSS_KEY(enabled, enabled)
 CSS_KEY(end, end)
+CSS_KEY(ethiopic-numeric, ethiopic_numeric)
 CSS_KEY(ex, ex)
 CSS_KEY(exclusion, exclusion)
 CSS_KEY(expanded, expanded)
+CSS_KEY(extends, extends)
 CSS_KEY(extra-condensed, extra_condensed)
 CSS_KEY(extra-expanded, extra_expanded)
 CSS_KEY(ew-resize, ew_resize)
+CSS_KEY(fantasy, fantasy)
 CSS_KEY(farthest-side, farthest_side)
 CSS_KEY(farthest-corner, farthest_corner)
 CSS_KEY(fill, fill)
+CSS_KEY(fill-box, fill_box)
 CSS_KEY(fixed, fixed)
 CSS_KEY(flat, flat)
 CSS_KEY(flex, flex)
@@ -289,7 +269,6 @@ CSS_KEY(forwards, forwards)
 CSS_KEY(fraktur, fraktur)
 CSS_KEY(from-image, from_image)
 CSS_KEY(full-width, full_width)
-CSS_KEY(georgian, georgian)
 CSS_KEY(grab, grab)
 CSS_KEY(grabbing, grabbing)
 CSS_KEY(grad, grad)
@@ -304,8 +283,6 @@ CSS_KEY(hidden, hidden)
 CSS_KEY(hide, hide)
 CSS_KEY(highlight, highlight)
 CSS_KEY(highlighttext, highlighttext)
-CSS_KEY(hiragana, hiragana)
-CSS_KEY(hiragana-iroha, hiragana_iroha)
 CSS_KEY(historical-forms, historical_forms)
 CSS_KEY(historical-ligatures, historical_ligatures)
 CSS_KEY(horizontal, horizontal)
@@ -335,6 +312,7 @@ CSS_KEY(inline-table, inline_table)
 CSS_KEY(inset, inset)
 CSS_KEY(inside, inside)
 CSS_KEY(interpolatematrix, interpolatematrix)
+CSS_KEY(isolate, isolate)
 CSS_KEY(invert, invert)
 CSS_KEY(italic, italic)
 CSS_KEY(japanese-formal, japanese_formal)
@@ -344,8 +322,6 @@ CSS_KEY(jis83, jis83)
 CSS_KEY(jis90, jis90)
 CSS_KEY(jis04, jis04)
 CSS_KEY(justify, justify)
-CSS_KEY(katakana, katakana)
-CSS_KEY(katakana-iroha, katakana_iroha)
 CSS_KEY(keep-all, keep_all)
 CSS_KEY(khz, khz)
 CSS_KEY(korean-hangul-formal, korean_hangul_formal)
@@ -364,14 +340,11 @@ CSS_KEY(list-item, list_item)
 CSS_KEY(local, local)
 CSS_KEY(logical, logical)
 CSS_KEY(looped, looped)
-CSS_KEY(lower-alpha, lower_alpha)
-CSS_KEY(lower-greek, lower_greek)
-CSS_KEY(lower-latin, lower_latin)
-CSS_KEY(lower-roman, lower_roman)
 CSS_KEY(lowercase, lowercase)
 CSS_KEY(ltr, ltr)
 CSS_KEY(luminance, luminance)
 CSS_KEY(luminosity, luminosity)
+CSS_KEY(manipulation, manipulation)
 CSS_KEY(manual, manual)
 CSS_KEY(margin-box, margin_box)
 CSS_KEY(markers, markers)
@@ -386,6 +359,7 @@ CSS_KEY(middle, middle)
 CSS_KEY(min-content, min_content)
 CSS_KEY(minmax, minmax)
 CSS_KEY(mix, mix)
+CSS_KEY(mixed, mixed)
 CSS_KEY(mm, mm)
 CSS_KEY(monospace, monospace)
 CSS_KEY(move, move)
@@ -399,6 +373,7 @@ CSS_KEY(no-close-quote, no_close_quote)
 CSS_KEY(no-common-ligatures, no_common_ligatures)
 CSS_KEY(no-contextual, no_contextual)
 CSS_KEY(no-discretionary-ligatures, no_discretionary_ligatures)
+CSS_KEY(no-drag, no_drag)
 CSS_KEY(no-drop, no_drop)
 CSS_KEY(no-historical-ligatures, no_historical_ligatures)
 CSS_KEY(no-open-quote, no_open_quote)
@@ -407,6 +382,7 @@ CSS_KEY(none, none)
 CSS_KEY(normal, normal)
 CSS_KEY(not-allowed, not_allowed)
 CSS_KEY(nowrap, nowrap)
+CSS_KEY(numeric, numeric)
 CSS_KEY(ns-resize, ns_resize)
 CSS_KEY(nw-resize, nw_resize)
 CSS_KEY(nwse-resize, nwse_resize)
@@ -430,6 +406,7 @@ CSS_KEY(perspective, perspective)
 CSS_KEY(petite-caps, petite_caps)
 CSS_KEY(physical, physical)
 CSS_KEY(pointer, pointer)
+CSS_KEY(polygon, polygon)
 CSS_KEY(portrait, portrait)
 CSS_KEY(pre, pre)
 CSS_KEY(pre-wrap, pre_wrap)
@@ -462,12 +439,17 @@ CSS_KEY(row-resize, row_resize)
 CSS_KEY(row-reverse, row_reverse)
 CSS_KEY(rtl, rtl)
 CSS_KEY(ruby, ruby)
+CSS_KEY(ruby-base, ruby_base)
+CSS_KEY(ruby-base-container, ruby_base_container)
+CSS_KEY(ruby-text, ruby_text)
+CSS_KEY(ruby-text-container, ruby_text_container)
 CSS_KEY(running, running)
 CSS_KEY(s, s)
 CSS_KEY(s-resize, s_resize)
 CSS_KEY(saturate, saturate)
 CSS_KEY(saturation, saturation)
 CSS_KEY(scale, scale)
+CSS_KEY(scale-down, scale_down)
 CSS_KEY(scale3d, scale3d)
 CSS_KEY(scalex, scalex)
 CSS_KEY(scaley, scaley)
@@ -487,8 +469,11 @@ CSS_KEY(semi-condensed, semi_condensed)
 CSS_KEY(semi-expanded, semi_expanded)
 CSS_KEY(separate, separate)
 CSS_KEY(sepia, sepia)
+CSS_KEY(serif, serif)
 CSS_KEY(show, show)
-CSS_KEY(sideways, sideways)
+/* CSS_KEY(sideways, sideways) */
+/* CSS_KEY(sideways-left, sideways_left) */
+CSS_KEY(sideways-right, sideways_right)
 CSS_KEY(simp-chinese-formal, simp_chinese_formal)
 CSS_KEY(simp-chinese-informal, simp_chinese_informal)
 CSS_KEY(simplified, simplified)
@@ -496,17 +481,21 @@ CSS_KEY(skew, skew)
 CSS_KEY(skewx, skewx)
 CSS_KEY(skewy, skewy)
 CSS_KEY(slashed-zero, slashed_zero)
+CSS_KEY(slice, slice)
 CSS_KEY(small, small)
 CSS_KEY(small-caps, small_caps)
 CSS_KEY(small-caption, small_caption)
 CSS_KEY(smaller, smaller)
+CSS_KEY(smooth, smooth)
 CSS_KEY(soft, soft)
 CSS_KEY(soft-light, soft_light)
 CSS_KEY(solid, solid)
 CSS_KEY(space-around, space_around)
 CSS_KEY(space-between, space_between)
 CSS_KEY(span, span)
+CSS_KEY(spell-out, spell_out)
 CSS_KEY(square, square)
+CSS_KEY(stack, stack)
 CSS_KEY(stacked-fractions, stacked_fractions)
 CSS_KEY(start, start)
 CSS_KEY(static, static)
@@ -518,10 +507,12 @@ CSS_KEY(stretch, stretch)
 CSS_KEY(stretch-to-fit, stretch_to_fit)
 CSS_KEY(stretched, stretched)
 CSS_KEY(stroke, stroke)
+CSS_KEY(stroke-box, stroke_box)
 CSS_KEY(style, style)
 CSS_KEY(styleset, styleset)
 CSS_KEY(stylistic, stylistic)
 CSS_KEY(sub, sub)
+CSS_KEY(subgrid, subgrid)
 CSS_KEY(super, super)
 CSS_KEY(sw-resize, sw_resize)
 CSS_KEY(swash, swash)
@@ -566,15 +557,13 @@ CSS_KEY(ultra-expanded, ultra_expanded)
 CSS_KEY(underline, underline)
 CSS_KEY(unicase, unicase)
 CSS_KEY(unset, unset)
-CSS_KEY(upper-alpha, upper_alpha)
-CSS_KEY(upper-latin, upper_latin)
-CSS_KEY(upper-roman, upper_roman)
 CSS_KEY(uppercase, uppercase)
 CSS_KEY(upright, upright)
 CSS_KEY(vertical, vertical)
 CSS_KEY(vertical-lr, vertical_lr)
 CSS_KEY(vertical-rl, vertical_rl)
 CSS_KEY(vertical-text, vertical_text)
+CSS_KEY(view-box, view_box)
 CSS_KEY(visible, visible)
 CSS_KEY(visiblefill, visiblefill)
 CSS_KEY(visiblepainted, visiblepainted)
@@ -587,6 +576,7 @@ CSS_KEY(wider, wider)
 CSS_KEY(window, window)
 CSS_KEY(windowframe, windowframe)
 CSS_KEY(windowtext, windowtext)
+CSS_KEY(words, words)
 CSS_KEY(wrap, wrap)
 CSS_KEY(wrap-reverse, wrap_reverse)
 CSS_KEY(write-only, write_only)
@@ -619,6 +609,7 @@ CSS_KEY(resizerpanel, resizerpanel)
 CSS_KEY(resizer, resizer)
 CSS_KEY(listbox, listbox)
 CSS_KEY(listitem, listitem)
+CSS_KEY(numbers, numbers)
 CSS_KEY(number-input, number_input)
 CSS_KEY(treeview, treeview)
 CSS_KEY(treeitem, treeitem)
@@ -711,6 +702,10 @@ CSS_KEY(-moz-window-button-box-maximized, _moz_window_button_box_maximized)
 CSS_KEY(-moz-mac-help-button, _moz_mac_help_button)
 CSS_KEY(-moz-mac-unified-toolbar, _moz_mac_unified_toolbar)
 CSS_KEY(-moz-win-exclude-glass, _moz_win_exclude_glass)
+CSS_KEY(-moz-mac-vibrancy-light, _moz_mac_vibrancy_light)
+CSS_KEY(-moz-mac-vibrancy-dark, _moz_mac_vibrancy_dark)
+CSS_KEY(-moz-mac-disclosure-button-closed, _moz_mac_disclosure_button_closed)
+CSS_KEY(-moz-mac-disclosure-button-open, _moz_mac_disclosure_button_open)
 CSS_KEY(alphabetic, alphabetic)
 CSS_KEY(bevel, bevel)
 CSS_KEY(butt, butt)
@@ -735,6 +730,8 @@ CSS_KEY(reset-size, reset_size)
 //CSS_KEY(square, square)
 //CSS_KEY(start, start)
 CSS_KEY(srgb, srgb)
+CSS_KEY(symbolic, symbolic)
+CSS_KEY(symbols, symbols)
 CSS_KEY(text-after-edge, text_after_edge)
 CSS_KEY(text-before-edge, text_before_edge)
 CSS_KEY(use-script, use_script)
