@@ -2,7 +2,10 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 #include "nsTableOuterFrame.h"
+
+#include "nsFrameManager.h"
 #include "nsTableFrame.h"
 #include "nsTableCellFrame.h"
 #include "nsStyleContext.h"
@@ -102,25 +105,32 @@ nsTableCaptionFrame::ComputeAutoSize(nsRenderingContext *aRenderingContext,
   return result;
 }
 
-nsIFrame*
-nsTableCaptionFrame::GetParentStyleContextFrame() const
+nsStyleContext*
+nsTableCaptionFrame::GetParentStyleContext(nsIFrame** aProviderFrame) const
 {
-  NS_PRECONDITION(mContent->GetParent(),
-                  "How could we not have a parent here?");
+  MOZ_ASSERT(GetContent()->GetParent(), "How could we not have a parent here?");
     
+  nsStyleContext* sc =
+    PresContext()->FrameManager()->GetDisplayContentsStyleFor(GetContent()->GetParent());
+  if (sc) {
+    *aProviderFrame = nullptr;
+    return sc;
+  }
+
   // The caption's style context parent is the inner frame, unless
   // it's anonymous.
   nsIFrame* outerFrame = GetParent();
   if (outerFrame && outerFrame->GetType() == nsGkAtoms::tableOuterFrame) {
     nsIFrame* innerFrame = outerFrame->GetFirstPrincipalChild();
     if (innerFrame) {
-      return nsFrame::CorrectStyleParentFrame(innerFrame,
-                                              StyleContext()->GetPseudo());
+      *aProviderFrame = nsFrame::CorrectStyleParentFrame(innerFrame,
+                                   StyleContext()->GetPseudo());
+      return *aProviderFrame ? (*aProviderFrame)->StyleContext() : nullptr;
     }
   }
 
   NS_NOTREACHED("Where is our inner table frame?");
-  return nsBlockFrame::GetParentStyleContextFrame();
+  return nsBlockFrame::GetParentStyleContext(aProviderFrame);
 }
 
 #ifdef ACCESSIBILITY
@@ -324,8 +334,8 @@ nsTableOuterFrame::BuildDisplayListForInnerTable(nsDisplayListBuilder*   aBuilde
   }
 }
 
-nsIFrame*
-nsTableOuterFrame::GetParentStyleContextFrame() const
+nsStyleContext*
+nsTableOuterFrame::GetParentStyleContext(nsIFrame** aProviderFrame) const
 {
   // The table outer frame and the (inner) table frame split the style
   // data by giving the table frame the style context associated with
@@ -337,7 +347,7 @@ nsTableOuterFrame::GetParentStyleContextFrame() const
   // children of the table inherit directly from the inner table, and
   // the outer table's style context is a leaf.
 
-  return InnerTableFrame();
+  return (*aProviderFrame = InnerTableFrame())->StyleContext();
 }
 
 // INCREMENTAL REFLOW HELPER FUNCTIONS 
@@ -493,7 +503,7 @@ ChildShrinkWrapWidth(nsRenderingContext *aRenderingContext,
                   offsets.ComputedLogicalBorderPadding().Size(wm) -
                     offsets.ComputedLogicalPadding().Size(wm),
                   offsets.ComputedLogicalPadding().Size(wm),
-                  true);
+                  nsIFrame::ComputeSizeFlags::eShrinkWrap);
   if (aMarginResult)
     *aMarginResult = offsets.ComputedLogicalMargin().IStartEnd(wm);
   return size.ISize(wm) + offsets.ComputedLogicalMargin().IStartEnd(wm) +
@@ -512,8 +522,7 @@ nsTableOuterFrame::ComputeAutoSize(nsRenderingContext *aRenderingContext,
                                    bool aShrinkWrap)
 {
   nscoord kidAvailableWidth = aAvailableISize - aMargin.ISize(aWM);
-  NS_ASSERTION(aBorder == LogicalSize(aWM) &&
-               aPadding == LogicalSize(aWM),
+  NS_ASSERTION(aBorder.IsAllZero() && aPadding.IsAllZero(),
                "Table outer frames cannot have borders or paddings");
 
   // When we're shrink-wrapping, our auto size needs to wrap around the
