@@ -10,14 +10,19 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <SLES/OpenSLES.h>
+#include <math.h>
 #if defined(__ANDROID__)
+#include <sys/system_properties.h>
 #include "android/sles_definitions.h"
 #include <SLES/OpenSLES_Android.h>
 #include <android/log.h>
 #define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "Cubeb_OpenSL" , ## args)
+#define ANDROID_VERSION_GINGERBREAD_MR1 10
 #endif
 #include "cubeb/cubeb.h"
 #include "cubeb-internal.h"
+#include "cubeb_resampler.h"
+#include "cubeb-sles.h"
 
 static struct cubeb_ops const opensl_ops;
 
@@ -31,6 +36,7 @@ struct cubeb {
 #if defined(__ANDROID__)
   SLInterfaceID SL_IID_ANDROIDCONFIGURATION;
 #endif
+  SLInterfaceID SL_IID_VOLUME;
   SLObjectItf engObj;
   SLEngineItf eng;
   SLObjectItf outmixObj;
@@ -46,7 +52,8 @@ struct cubeb_stream {
   SLObjectItf playerObj;
   SLPlayItf play;
   SLBufferQueueItf bufq;
-  void *queuebuf[NBUFS];
+  SLVolumeItf volume;
+  uint8_t *queuebuf[NBUFS];
   int queuebuf_idx;
   long queuebuf_len;
   long bytespersec;
@@ -58,6 +65,11 @@ struct cubeb_stream {
   cubeb_data_callback data_callback;
   cubeb_state_callback state_callback;
   void * user_ptr;
+
+  cubeb_resampler * resampler;
+  unsigned int inputrate;
+  unsigned int outputrate;
+  unsigned int latency;
 };
 
 static void
@@ -69,9 +81,9 @@ play_callback(SLPlayItf caller, void * user_ptr, SLuint32 event)
     case SL_PLAYEVENT_HEADATMARKER:
       pthread_mutex_lock(&stm->mutex);
       assert(stm->draining);
-      stm->draining = 0;
       pthread_mutex_unlock(&stm->mutex);
       stm->state_callback(stm, stm->user_ptr, CUBEB_STATE_DRAINED);
+      (*stm->play)->SetPlayState(stm->play, SL_PLAYSTATE_PAUSED);
       break;
     default:
       break;
@@ -94,16 +106,17 @@ bufferqueue_callback(SLBufferQueueItf caller, void * user_ptr)
 
   SLuint32 i;
   for (i = state.count; i < NBUFS; i++) {
-    void *buf = stm->queuebuf[stm->queuebuf_idx];
+    uint8_t *buf = stm->queuebuf[stm->queuebuf_idx];
     long written = 0;
     pthread_mutex_lock(&stm->mutex);
     int draining = stm->draining;
     pthread_mutex_unlock(&stm->mutex);
 
     if (!draining) {
-      written = stm->data_callback(stm, stm->user_ptr, buf, stm->queuebuf_len / stm->framesize);
+      written = cubeb_resampler_fill(stm->resampler, buf,
+                                        stm->queuebuf_len / stm->framesize);
       if (written < 0 || written * stm->framesize > stm->queuebuf_len) {
-        (*stm->play)->SetPlayState(stm->play, SL_PLAYSTATE_STOPPED);
+        (*stm->play)->SetPlayState(stm->play, SL_PLAYSTATE_PAUSED);
         return;
       }
     }
@@ -111,7 +124,8 @@ bufferqueue_callback(SLBufferQueueItf caller, void * user_ptr)
     // Keep sending silent data even in draining mode to prevent the audio
     // back-end from being stopped automatically by OpenSL/ES.
     memset(buf + written * stm->framesize, 0, stm->queuebuf_len - written * stm->framesize);
-    (*stm->bufq)->Enqueue(stm->bufq, buf, stm->queuebuf_len);
+    res = (*stm->bufq)->Enqueue(stm->bufq, buf, stm->queuebuf_len);
+    assert(res == SL_RESULT_SUCCESS);
     stm->queuebuf_idx = (stm->queuebuf_idx + 1) % NBUFS;
     if (written > 0) {
       pthread_mutex_lock(&stm->mutex);
@@ -159,11 +173,38 @@ convert_stream_type_to_sl_stream(cubeb_stream_type stream_type)
 
 static void opensl_destroy(cubeb * ctx);
 
+#if defined(__ANDROID__)
+
+static int
+get_android_version(void)
+{
+  char version_string[PROP_VALUE_MAX];
+
+  memset(version_string, 0, PROP_VALUE_MAX);
+
+  int len = __system_property_get("ro.build.version.sdk", version_string);
+  if (len <= 0) {
+    LOG("Failed to get Android version!\n");
+    return len;
+  }
+
+  return (int)strtol(version_string, NULL, 10);
+}
+#endif
+
 /*static*/ int
 opensl_init(cubeb ** context, char const * context_name)
 {
   cubeb * ctx;
 
+#if defined(__ANDROID__)
+  int android_version = get_android_version();
+  if (android_version > 0 && android_version <= ANDROID_VERSION_GINGERBREAD_MR1) {
+    // Don't even attempt to run on Gingerbread and lower
+    return CUBEB_ERROR;
+  }
+#endif
+
   *context = NULL;
 
   ctx = calloc(1, sizeof(*ctx));
@@ -206,6 +247,7 @@ opensl_init(cubeb ** context, char const * context_name)
     (slCreateEngine_t)dlsym(ctx->lib, "slCreateEngine");
   SLInterfaceID SL_IID_ENGINE = *(SLInterfaceID *)dlsym(ctx->lib, "SL_IID_ENGINE");
   SLInterfaceID SL_IID_OUTPUTMIX = *(SLInterfaceID *)dlsym(ctx->lib, "SL_IID_OUTPUTMIX");
+  ctx->SL_IID_VOLUME = *(SLInterfaceID *)dlsym(ctx->lib, "SL_IID_VOLUME");
   ctx->SL_IID_BUFFERQUEUE = *(SLInterfaceID *)dlsym(ctx->lib, "SL_IID_BUFFERQUEUE");
 #if defined(__ANDROID__)
   ctx->SL_IID_ANDROIDCONFIGURATION = *(SLInterfaceID *)dlsym(ctx->lib, "SL_IID_ANDROIDCONFIGURATION");
@@ -226,13 +268,14 @@ opensl_init(cubeb ** context, char const * context_name)
   const SLEngineOption opt[] = {{SL_ENGINEOPTION_THREADSAFE, SL_BOOLEAN_TRUE}};
 
   SLresult res;
-  res = f_slCreateEngine(&ctx->engObj, 1, opt, 0, NULL, NULL);
+  res = cubeb_get_sles_engine(&ctx->engObj, 1, opt, 0, NULL, NULL);
+
   if (res != SL_RESULT_SUCCESS) {
     opensl_destroy(ctx);
     return CUBEB_ERROR;
   }
 
-  res = (*ctx->engObj)->Realize(ctx->engObj, SL_BOOLEAN_FALSE);
+  res = cubeb_realize_sles_engine(ctx->engObj);
   if (res != SL_RESULT_SUCCESS) {
     opensl_destroy(ctx);
     return CUBEB_ERROR;
@@ -405,7 +448,7 @@ opensl_destroy(cubeb * ctx)
   if (ctx->outmixObj)
     (*ctx->outmixObj)->Destroy(ctx->outmixObj);
   if (ctx->engObj)
-    (*ctx->engObj)->Destroy(ctx->engObj);
+    cubeb_destroy_sles_engine(&ctx->engObj);
   dlclose(ctx->lib);
   dlclose(ctx->libmedia);
   free(ctx);
@@ -425,8 +468,7 @@ opensl_stream_init(cubeb * ctx, cubeb_stream ** stream, char const * stream_name
 
   *stream = NULL;
 
-  if (stream_params.rate < 8000 || stream_params.rate > 88200 ||
-      stream_params.channels < 1 || stream_params.channels > 32 ||
+  if (stream_params.channels < 1 || stream_params.channels > 32 ||
       latency < 1 || latency > 2000) {
     return CUBEB_ERROR_INVALID_FORMAT;
   }
@@ -462,19 +504,10 @@ opensl_stream_init(cubeb * ctx, cubeb_stream ** stream, char const * stream_name
   stm->state_callback = state_callback;
   stm->user_ptr = user_ptr;
 
+  stm->inputrate = stream_params.rate;
+  stm->latency = latency;
   stm->stream_type = stream_params.stream_type;
   stm->framesize = stream_params.channels * sizeof(int16_t);
-  stm->bytespersec = stream_params.rate * stm->framesize;
-  stm->queuebuf_len = (stm->bytespersec * latency) / (1000 * NBUFS);
-  // round up to the next multiple of stm->framesize, if needed.
-  if (stm->queuebuf_len % stm->framesize) {
-    stm->queuebuf_len += stm->framesize - (stm->queuebuf_len % stm->framesize);
-  }
-  int i;
-  for (i = 0; i < NBUFS; i++) {
-    stm->queuebuf[i] = malloc(stm->queuebuf_len);
-    assert(stm->queuebuf[i]);
-  }
 
   int r = pthread_mutex_init(&stm->mutex, NULL);
   assert(r == 0);
@@ -494,20 +527,62 @@ opensl_stream_init(cubeb * ctx, cubeb_stream ** stream, char const * stream_name
   sink.pFormat = NULL;
 
 #if defined(__ANDROID__)
-  const SLInterfaceID ids[] = {ctx->SL_IID_BUFFERQUEUE, ctx->SL_IID_ANDROIDCONFIGURATION};
-  const SLboolean req[] = {SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};
+  const SLInterfaceID ids[] = {ctx->SL_IID_BUFFERQUEUE,
+                               ctx->SL_IID_VOLUME,
+                               ctx->SL_IID_ANDROIDCONFIGURATION};
+  const SLboolean req[] = {SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};
 #else
-  const SLInterfaceID ids[] = {ctx->SL_IID_BUFFERQUEUE};
-  const SLboolean req[] = {SL_BOOLEAN_TRUE};
+  const SLInterfaceID ids[] = {ctx->SL_IID_BUFFERQUEUE, ctx->SL_IID_VOLUME};
+  const SLboolean req[] = {SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};
 #endif
   assert(NELEMS(ids) == NELEMS(req));
   SLresult res = (*ctx->eng)->CreateAudioPlayer(ctx->eng, &stm->playerObj,
                                                 &source, &sink, NELEMS(ids), ids, req);
+
+  uint32_t preferred_sampling_rate = stm->inputrate;
+  // Sample rate not supported? Try again with primary sample rate!
+  if (res == SL_RESULT_CONTENT_UNSUPPORTED) {
+    if (opensl_get_preferred_sample_rate(ctx, &preferred_sampling_rate)) {
+      opensl_stream_destroy(stm);
+      return CUBEB_ERROR;
+    }
+
+    format.samplesPerSec = preferred_sampling_rate * 1000;
+    res = (*ctx->eng)->CreateAudioPlayer(ctx->eng, &stm->playerObj,
+                                         &source, &sink, NELEMS(ids), ids, req);
+  }
+
   if (res != SL_RESULT_SUCCESS) {
     opensl_stream_destroy(stm);
     return CUBEB_ERROR;
   }
 
+  stm->outputrate = preferred_sampling_rate;
+  stm->bytespersec = preferred_sampling_rate * stm->framesize;
+  stm->queuebuf_len = (stm->bytespersec * latency) / (1000 * NBUFS);
+  // round up to the next multiple of stm->framesize, if needed.
+  if (stm->queuebuf_len % stm->framesize) {
+    stm->queuebuf_len += stm->framesize - (stm->queuebuf_len % stm->framesize);
+  }
+
+  stm->resampler = cubeb_resampler_create(stm, stream_params,
+                                          preferred_sampling_rate,
+                                          data_callback,
+                                          stm->queuebuf_len / stm->framesize,
+                                          user_ptr,
+                                          CUBEB_RESAMPLER_QUALITY_DEFAULT);
+
+  if (!stm->resampler) {
+    opensl_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
+
+  int i;
+  for (i = 0; i < NBUFS; i++) {
+    stm->queuebuf[i] = malloc(stm->queuebuf_len);
+    assert(stm->queuebuf[i]);
+  }
+
 #if defined(__ANDROID__)
   SLuint32 stream_type = convert_stream_type_to_sl_stream(stream_params.stream_type);
   if (stream_type != 0xFFFFFFFF) {
@@ -542,6 +617,14 @@ opensl_stream_init(cubeb * ctx, cubeb_stream ** stream, char const * stream_name
     return CUBEB_ERROR;
   }
 
+  res = (*stm->playerObj)->GetInterface(stm->playerObj, ctx->SL_IID_VOLUME,
+                                        &stm->volume);
+
+  if (res != SL_RESULT_SUCCESS) {
+    opensl_stream_destroy(stm);
+    return CUBEB_ERROR;
+  }
+
   res = (*stm->play)->RegisterCallback(stm->play, play_callback, stm);
   if (res != SL_RESULT_SUCCESS) {
     opensl_stream_destroy(stm);
@@ -575,6 +658,8 @@ opensl_stream_destroy(cubeb_stream * stm)
   }
   pthread_mutex_destroy(&stm->mutex);
 
+  cubeb_resampler_destroy(stm->resampler);
+
   free(stm);
 }
 
@@ -614,25 +699,24 @@ opensl_stream_get_position(cubeb_stream * stm, uint64_t * position)
   if (res != SL_RESULT_SUCCESS)
     return CUBEB_ERROR;
 
-  samplerate = stm->bytespersec / stm->framesize;
+  samplerate = stm->inputrate;
 
   rv = stm->context->get_output_latency(&mixer_latency, stm->stream_type);
   if (rv) {
     return CUBEB_ERROR;
   }
 
-  int64_t compensated_position = samplerate * (msec - mixer_latency) / 1000;
   pthread_mutex_lock(&stm->mutex);
-  int64_t maximum_position = stm->written;
+  int64_t maximum_position = stm->written * (int64_t)stm->inputrate / stm->outputrate;
   pthread_mutex_unlock(&stm->mutex);
   assert(maximum_position >= 0);
 
-  if (compensated_position < 0) {
-    *position = 0;
-  } else if(compensated_position > maximum_position) {
-    *position = maximum_position;
+  if (msec > mixer_latency) {
+    int64_t unadjusted_position = samplerate * (msec - mixer_latency) / 1000;
+    *position = unadjusted_position < maximum_position ?
+                unadjusted_position : maximum_position;
   } else {
-    *position = compensated_position;
+    *position = 0;
   }
   return CUBEB_OK;
 }
@@ -641,15 +725,7 @@ int
 opensl_stream_get_latency(cubeb_stream * stm, uint32_t * latency)
 {
   int rv;
-  uint32_t mixer_latency;
-  uint32_t samplerate;
-
-  /* The latency returned by AudioFlinger is in ms, so we have to get
-   * AudioFlinger's samplerate to convert it to frames. */
-  rv = opensl_get_preferred_sample_rate(stm->context, &samplerate);
-  if (rv) {
-    return CUBEB_ERROR;
-  }
+  uint32_t mixer_latency; // The latency returned by AudioFlinger is in ms.
 
   /* audio_stream_type_t is an int, so this is okay. */
   rv = stm->context->get_output_latency(&mixer_latency, stm->stream_type);
@@ -657,8 +733,47 @@ opensl_stream_get_latency(cubeb_stream * stm, uint32_t * latency)
     return CUBEB_ERROR;
   }
 
-  *latency = NBUFS * stm->queuebuf_len / stm->framesize + // OpenSL latency
-             mixer_latency * samplerate / 1000; // AudioFlinger latency
+  *latency = stm->latency * stm->inputrate / 1000 + // OpenSL latency
+             mixer_latency * stm->inputrate / 1000; // AudioFlinger latency
+
+  return CUBEB_OK;
+}
+
+int
+opensl_stream_set_volume(cubeb_stream * stm, float volume)
+{
+  SLresult res;
+  SLmillibel max_level, millibels;
+  float unclamped_millibels;
+
+  res = (*stm->volume)->GetMaxVolumeLevel(stm->volume, &max_level);
+
+  if (res != SL_RESULT_SUCCESS) {
+    return CUBEB_ERROR;
+  }
+
+  /* millibels are 100*dB, so the conversion from the volume's linear amplitude
+   * is 100 * 20 * log(volume). However we clamp the resulting value before
+   * passing it to lroundf() in order to prevent it from silently returning an
+   * erroneous value when the unclamped value exceeds the size of a long. */
+  unclamped_millibels = 100.0f * 20.0f * log10f(fmaxf(volume, 0.0f));
+  unclamped_millibels = fmaxf(unclamped_millibels, SL_MILLIBEL_MIN);
+  unclamped_millibels = fminf(unclamped_millibels, max_level);
+
+  millibels = lroundf(unclamped_millibels);
+
+  res = (*stm->volume)->SetVolumeLevel(stm->volume, millibels);
+
+  if (res != SL_RESULT_SUCCESS) {
+    return CUBEB_ERROR;
+  }
+  return CUBEB_OK;
+}
+
+int
+opensl_stream_set_panning(cubeb_stream * stream, float panning)
+{
+  assert(0 && "not implemented.");
 
   return CUBEB_OK;
 }
@@ -675,5 +790,10 @@ static struct cubeb_ops const opensl_ops = {
   .stream_start = opensl_stream_start,
   .stream_stop = opensl_stream_stop,
   .stream_get_position = opensl_stream_get_position,
-  .stream_get_latency = opensl_stream_get_latency
+  .stream_get_latency = opensl_stream_get_latency,
+  .stream_set_volume = opensl_stream_set_volume,
+  .stream_set_panning = opensl_stream_set_panning,
+  .stream_get_current_device = NULL,
+  .stream_device_destroy = NULL,
+  .stream_register_device_changed_callback = NULL
 };
