@@ -21,6 +21,11 @@
 #include "cubeb_resampler.h"
 #include <stdio.h>
 
+/**Taken from winbase.h, Not in MinGW.*/
+#ifndef STACK_SIZE_PARAM_IS_A_RESERVATION
+#define STACK_SIZE_PARAM_IS_A_RESERVATION   0x00010000    // Threads only
+#endif
+
 #if 1
 #  define LOG(...) do {         \
   fprintf(stderr, __VA_ARGS__); \
@@ -313,8 +318,10 @@ wasapi_stream_render_loop(LPVOID stream)
       assert(padding <= stm->buffer_frame_count);
 
       if (stm->draining) {
-        stm->state_callback(stm, stm->user_ptr, CUBEB_STATE_DRAINED);
-        is_playing = false;
+        if (padding == 0) {
+          stm->state_callback(stm, stm->user_ptr, CUBEB_STATE_DRAINED);
+          is_playing = false;
+        }
         continue;
       }
 
