@@ -6,8 +6,10 @@
 #include "mp4_demuxer/AnnexB.h"
 #include "mp4_demuxer/ByteReader.h"
 #include "mp4_demuxer/DecoderData.h"
+#include <media/stagefright/foundation/ABitReader.h>
 #include "media/stagefright/MetaData.h"
 #include "media/stagefright/MediaBuffer.h"
+#include "media/stagefright/MediaDefs.h"
 #include "media/stagefright/Utils.h"
 #include "mozilla/ArrayUtils.h"
 #include "include/ESDS.h"
@@ -73,6 +75,12 @@ FindData(sp<MetaData>& aMetaData, uint32_t aKey, nsTArray<T>* aDest)
   return true;
 }
 
+static bool
+FindData(sp<MetaData>& aMetaData, uint32_t aKey, ByteBuffer* aDest)
+{
+  return FindData(aMetaData, aKey, static_cast<nsTArray<uint8_t>*>(aDest));
+}
+
 bool
 CryptoFile::DoUpdate(sp<MetaData>& aMetaData)
 {
@@ -124,55 +132,63 @@ CryptoSample::Update(sp<MetaData>& aMetaData)
 }
 
 void
-AudioDecoderConfig::Update(sp<MetaData>& aMetaData, const char* aMimeType)
+TrackConfig::Update(sp<MetaData>& aMetaData, const char* aMimeType)
 {
   // aMimeType points to a string from MediaDefs.cpp so we don't need to copy it
   mime_type = aMimeType;
   duration = FindInt64(aMetaData, kKeyDuration);
+  media_time = FindInt64(aMetaData, kKeyMediaTime);
+  mTrackId = FindInt32(aMetaData, kKeyTrackID);
+  crypto.Update(aMetaData);
+}
+
+void
+AudioDecoderConfig::Update(sp<MetaData>& aMetaData, const char* aMimeType)
+{
+  TrackConfig::Update(aMetaData, aMimeType);
   channel_count = FindInt32(aMetaData, kKeyChannelCount);
   bits_per_sample = FindInt32(aMetaData, kKeySampleSize);
   samples_per_second = FindInt32(aMetaData, kKeySampleRate);
   frequency_index = Adts::GetFrequencyIndex(samples_per_second);
   aac_profile = FindInt32(aMetaData, kKeyAACProfile);
 
-  if (FindData(aMetaData, kKeyESDS, &extra_data)) {
-    ESDS esds(&extra_data[0], extra_data.length());
+  if (FindData(aMetaData, kKeyESDS, extra_data)) {
+    ESDS esds(extra_data->Elements(), extra_data->Length());
 
     const void* data;
     size_t size;
     if (esds.getCodecSpecificInfo(&data, &size) == OK) {
-      audio_specific_config.append(reinterpret_cast<const uint8_t*>(data),
-                                   size);
+      const uint8_t* cdata = reinterpret_cast<const uint8_t*>(data);
+      audio_specific_config->AppendElements(cdata, size);
+      if (size > 1) {
+        ABitReader br(cdata, size);
+        extended_profile = br.getBits(5);
+
+        if (extended_profile == 31) {  // AAC-ELD => additional 6 bits
+          extended_profile = 32 + br.getBits(6);
+        }
+      }
     }
   }
-
-  crypto.Update(aMetaData);
 }
 
 bool
 AudioDecoderConfig::IsValid()
 {
   return channel_count > 0 && samples_per_second > 0 && frequency_index > 0 &&
-         aac_profile > 0;
+         (mime_type != MEDIA_MIMETYPE_AUDIO_AAC || aac_profile > 0);
 }
 
 void
 VideoDecoderConfig::Update(sp<MetaData>& aMetaData, const char* aMimeType)
 {
-  // aMimeType points to a string from MediaDefs.cpp so we don't need to copy it
-  mime_type = aMimeType;
-  duration = FindInt64(aMetaData, kKeyDuration);
+  TrackConfig::Update(aMetaData, aMimeType);
   display_width = FindInt32(aMetaData, kKeyDisplayWidth);
   display_height = FindInt32(aMetaData, kKeyDisplayHeight);
+  image_width = FindInt32(aMetaData, kKeyWidth);
+  image_height = FindInt32(aMetaData, kKeyHeight);
 
-  if (FindData(aMetaData, kKeyAVCC, &extra_data) && extra_data.length() >= 7) {
-    // Set size of the NAL length to 4. The demuxer formats its output with
-    // this NAL length size.
-    extra_data[4] |= 3;
-    annex_b = AnnexB::ConvertExtraDataToAnnexB(extra_data);
-  }
-
-  crypto.Update(aMetaData);
+  FindData(aMetaData, kKeyAVCC, extra_data);
 }
 
 bool
@@ -183,15 +199,32 @@ VideoDecoderConfig::IsValid()
 
 MP4Sample::MP4Sample()
   : mMediaBuffer(nullptr)
+  , decode_timestamp(0)
   , composition_timestamp(0)
   , duration(0)
   , byte_offset(0)
   , is_sync_point(0)
   , data(nullptr)
   , size(0)
+  , extra_data(nullptr)
 {
 }
 
+MP4Sample::MP4Sample(const MP4Sample& copy)
+  : mMediaBuffer(nullptr)
+  , decode_timestamp(copy.decode_timestamp)
+  , composition_timestamp(copy.composition_timestamp)
+  , duration(copy.duration)
+  , byte_offset(copy.byte_offset)
+  , is_sync_point(copy.is_sync_point)
+  , size(copy.size)
+  , crypto(copy.crypto)
+  , extra_data(copy.extra_data)
+{
+  extra_buffer = data = new uint8_t[size];
+  memcpy(data, copy.data, size);
+}
+
 MP4Sample::~MP4Sample()
 {
   if (mMediaBuffer) {
@@ -200,10 +233,13 @@ MP4Sample::~MP4Sample()
 }
 
 void
-MP4Sample::Update()
+MP4Sample::Update(int64_t& aMediaTime, int64_t& aTimestampOffset)
 {
   sp<MetaData> m = mMediaBuffer->meta_data();
-  composition_timestamp = FindInt64(m, kKeyTime);
+  // XXXbholley - Why don't we adjust decode_timestamp for aMediaTime?
+  // According to k17e, this code path is no longer used - we should probably remove it.
+  decode_timestamp = FindInt64(m, kKeyDecodingTime) + aTimestampOffset;
+  composition_timestamp = FindInt64(m, kKeyTime) - aMediaTime + aTimestampOffset;
   duration = FindInt64(m, kKeyDuration);
   byte_offset = FindInt64(m, kKey64BitFileOffset);
   is_sync_point = FindInt32(m, kKeyIsSyncFrame);
@@ -216,8 +252,6 @@ MP4Sample::Update()
 void
 MP4Sample::Pad(size_t aPaddingBytes)
 {
-  MOZ_ASSERT(data == mMediaBuffer->data());
-
   size_t newSize = size + aPaddingBytes;
 
   // If the existing MediaBuffer has enough space then we just recycle it. If
@@ -261,4 +295,25 @@ MP4Sample::Prepend(const uint8_t* aData, size_t aSize)
     }
   }
 }
+
+void
+MP4Sample::Replace(const uint8_t* aData, size_t aSize)
+{
+  // If the existing MediaBuffer has enough space then we just recycle it. If
+  // not then we copy to a new buffer.
+  uint8_t* newData = mMediaBuffer && aSize <= mMediaBuffer->size()
+                       ? data
+                       : new uint8_t[aSize];
+
+  memcpy(newData, aData, aSize);
+  size = aSize;
+
+  if (newData != data) {
+    extra_buffer = data = newData;
+    if (mMediaBuffer) {
+      mMediaBuffer->release();
+      mMediaBuffer = nullptr;
+    }
+  }
+}
 }
