@@ -78,6 +78,7 @@
 #include "mozilla/dom/RTCPeerConnectionBinding.h"
 #include "mozilla/dom/PeerConnectionImplBinding.h"
 #include "mozilla/dom/DataChannelBinding.h"
+#include "mozilla/dom/PluginCrashedEvent.h"
 #include "MediaStreamList.h"
 #include "MediaStreamTrack.h"
 #include "AudioStreamTrack.h"
@@ -412,6 +413,9 @@ PeerConnectionImpl::ConvertRTCConfiguration(const RTCConfiguration& aSrc,
     if (!(isStun || isStuns || isTurn || isTurns)) {
       return NS_ERROR_FAILURE;
     }
+    if (isTurns || isStuns) {
+      continue; // TODO: Support TURNS and STUNS (Bug 1056934)
+    }
     nsAutoCString spec;
     rv = url->GetSpec(spec);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1789,53 +1793,20 @@ PeerConnectionImpl::PluginCrash(uint64_t aPluginID,
     return true;
   }
 
-  ErrorResult rv;
-  nsRefPtr<Event> event =
-    doc->CreateEvent(NS_LITERAL_STRING("customevent"), rv);
-  nsCOMPtr<nsIDOMCustomEvent> customEvent(do_QueryObject(event));
-  if (!customEvent) {
-    NS_WARNING("Couldn't QI event for PluginCrashed event!");
-    return true;
-  }
+  PluginCrashedEventInit init;
+  init.mPluginDumpID = aPluginDumpID;
+  init.mPluginName = aPluginName;
+  init.mSubmittedCrashReport = false;
+  init.mGmpPlugin = true;
+  init.mBubbles = true;
+  init.mCancelable = true;
 
-  nsCOMPtr<nsIWritableVariant> variant;
-  variant = do_CreateInstance("@mozilla.org/variant;1");
-  if (!variant) {
-    NS_WARNING("Couldn't create detail variant for PluginCrashed event!");
-    return true;
-  }
+  nsRefPtr<PluginCrashedEvent> event =
+    PluginCrashedEvent::Constructor(doc, NS_LITERAL_STRING("PluginCrashed"), init);
 
-  customEvent->InitCustomEvent(NS_LITERAL_STRING("PluginCrashed"),
-                               true, true, variant);
   event->SetTrusted(true);
   event->GetInternalNSEvent()->mFlags.mOnlyChromeDispatch = true;
 
-  nsCOMPtr<nsIWritablePropertyBag2> propBag;
-  propBag = do_CreateInstance("@mozilla.org/hash-property-bag;1");
-  if (!propBag) {
-    NS_WARNING("Couldn't create a property bag for PluginCrashed event!");
-    return true;
-  }
-
-  // add a "pluginDumpID" property to this event
-  propBag->SetPropertyAsAString(NS_LITERAL_STRING("pluginDumpID"),
-                                aPluginDumpID);
-
-
-  // add a "pluginName" property to this event
-  propBag->SetPropertyAsAString(NS_LITERAL_STRING("pluginName"),
-                                aPluginName);
-
-  // add a "pluginName" property to this event
-  propBag->SetPropertyAsBool(NS_LITERAL_STRING("gmpPlugin"),
-                             true);
-
-  // add a "submittedCrashReport" property to this event
-  propBag->SetPropertyAsBool(NS_LITERAL_STRING("submittedCrashReport"),
-                             false);
-
-  variant->SetAsISupports(propBag);
-
   EventDispatcher::DispatchDOMEvent(mWindow, nullptr, event, nullptr, nullptr);
 #endif
 
