@@ -574,30 +574,35 @@ public:
                                   const CheckedInt<U>& aRhs);
   template<typename U>
   CheckedInt& operator +=(U aRhs);
+  CheckedInt& operator +=(const CheckedInt<T>& aRhs);
 
   template<typename U>
   friend CheckedInt<U> operator -(const CheckedInt<U>& aLhs,
                                   const CheckedInt<U>& aRhs);
   template<typename U>
   CheckedInt& operator -=(U aRhs);
+  CheckedInt& operator -=(const CheckedInt<T>& aRhs);
 
   template<typename U>
   friend CheckedInt<U> operator *(const CheckedInt<U>& aLhs,
                                   const CheckedInt<U>& aRhs);
   template<typename U>
   CheckedInt& operator *=(U aRhs);
+  CheckedInt& operator *=(const CheckedInt<T>& aRhs);
 
   template<typename U>
   friend CheckedInt<U> operator /(const CheckedInt<U>& aLhs,
                                   const CheckedInt<U>& aRhs);
   template<typename U>
   CheckedInt& operator /=(U aRhs);
+  CheckedInt& operator /=(const CheckedInt<T>& aRhs);
 
   template<typename U>
   friend CheckedInt<U> operator %(const CheckedInt<U>& aLhs,
                                   const CheckedInt<U>& aRhs);
   template<typename U>
   CheckedInt& operator %=(U aRhs);
+  CheckedInt& operator %=(const CheckedInt<T>& aRhs);
 
   CheckedInt operator -() const
   {
@@ -672,7 +677,7 @@ private:
 #define MOZ_CHECKEDINT_BASIC_BINARY_OPERATOR(NAME, OP)                        \
   template<typename T>                                                        \
   inline CheckedInt<T>                                                        \
-  operator OP(const CheckedInt<T> &aLhs, const CheckedInt<T> &aRhs)           \
+  operator OP(const CheckedInt<T>& aLhs, const CheckedInt<T>& aRhs)           \
   {                                                                           \
     if (!detail::Is##NAME##Valid(aLhs.mValue, aRhs.mValue)) {                 \
       return CheckedInt<T>(0, false);                                         \
@@ -723,23 +728,29 @@ castToCheckedInt(U aU)
   return detail::CastToCheckedIntImpl<T, U>::run(aU);
 }
 
-#define MOZ_CHECKEDINT_CONVENIENCE_BINARY_OPERATORS(OP, COMPOUND_OP)          \
-  template<typename T>                                                        \
-  template<typename U>                                                        \
-  CheckedInt<T>& CheckedInt<T>::operator COMPOUND_OP(U aRhs)                  \
-  {                                                                           \
-    *this = *this OP castToCheckedInt<T>(aRhs);                               \
-    return *this;                                                             \
-  }                                                                           \
-  template<typename T, typename U>                                            \
-  inline CheckedInt<T> operator OP(const CheckedInt<T> &aLhs, U aRhs)         \
-  {                                                                           \
-    return aLhs OP castToCheckedInt<T>(aRhs);                                 \
-  }                                                                           \
-  template<typename T, typename U>                                            \
-  inline CheckedInt<T> operator OP(U aLhs, const CheckedInt<T> &aRhs)         \
-  {                                                                           \
-    return castToCheckedInt<T>(aLhs) OP aRhs;                                 \
+#define MOZ_CHECKEDINT_CONVENIENCE_BINARY_OPERATORS(OP, COMPOUND_OP)            \
+  template<typename T>                                                          \
+  template<typename U>                                                          \
+  CheckedInt<T>& CheckedInt<T>::operator COMPOUND_OP(U aRhs)                    \
+  {                                                                             \
+    *this = *this OP castToCheckedInt<T>(aRhs);                                 \
+    return *this;                                                               \
+  }                                                                             \
+  template<typename T>                                                          \
+  CheckedInt<T>& CheckedInt<T>::operator COMPOUND_OP(const CheckedInt<T>& aRhs) \
+  {                                                                             \
+    *this = *this OP aRhs;                                                      \
+    return *this;                                                               \
+  }                                                                             \
+  template<typename T, typename U>                                              \
+  inline CheckedInt<T> operator OP(const CheckedInt<T>& aLhs, U aRhs)           \
+  {                                                                             \
+    return aLhs OP castToCheckedInt<T>(aRhs);                                   \
+  }                                                                             \
+  template<typename T, typename U>                                              \
+  inline CheckedInt<T> operator OP(U aLhs, const CheckedInt<T>& aRhs)           \
+  {                                                                             \
+    return castToCheckedInt<T>(aLhs) OP aRhs;                                   \
   }
 
 MOZ_CHECKEDINT_CONVENIENCE_BINARY_OPERATORS(+, +=)
@@ -752,14 +763,14 @@ MOZ_CHECKEDINT_CONVENIENCE_BINARY_OPERATORS(%, %=)
 
 template<typename T, typename U>
 inline bool
-operator ==(const CheckedInt<T> &aLhs, U aRhs)
+operator ==(const CheckedInt<T>& aLhs, U aRhs)
 {
   return aLhs == castToCheckedInt<T>(aRhs);
 }
 
 template<typename T, typename U>
 inline bool
-operator ==(U aLhs, const CheckedInt<T> &aRhs)
+operator ==(U aLhs, const CheckedInt<T>& aRhs)
 {
   return castToCheckedInt<T>(aLhs) == aRhs;
 }
