@@ -43,12 +43,9 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
     private static final int IMAGE_LEVEL_LTR = 0;
     private static final int IMAGE_LEVEL_RTL = 1;
 
-    private GeckoApp mActivity;
-
     public TextSelectionHandle(Context context, AttributeSet attrs) {
         super(context, attrs);
         setOnTouchListener(this);
-        mActivity = (GeckoApp) context;
 
         TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TextSelectionHandle);
         int handleType = a.getInt(R.styleable.TextSelectionHandle_handleType, 0x01);
@@ -60,7 +57,6 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
         else
             mHandleType = HandleType.END;
 
-        mIsRTL = false;
         mGeckoPoint = new PointF(0.0f, 0.0f);
 
         mWidth = getResources().getDimensionPixelSize(R.dimen.text_selection_handle_width);
@@ -76,7 +72,7 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
                 mTouchStartY = event.getY();
 
                 int[] rect = new int[2];
-                mActivity.getLayerView().getLocationOnScreen(rect);
+                GeckoAppShell.getLayerView().getLocationOnScreen(rect);
                 mLayerViewX = rect[0];
                 mLayerViewY = rect[1];
                 break;
@@ -111,7 +107,7 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
         mLeft = newX - mLayerViewX - mTouchStartX;
         mTop = newY - mLayerViewY - mTouchStartY;
 
-        LayerView layerView = mActivity.getLayerView();
+        LayerView layerView = GeckoAppShell.getLayerView();
         if (layerView == null) {
             Log.e(LOGTAG, "Can't move selection because layerView is null");
             return;
@@ -136,13 +132,13 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
         // will tell us the position of the caret, so we set the handle
         // position then. This allows us to lock the handle to wherever the
         // caret appears.
-        if (!mHandleType.equals(HandleType.MIDDLE)) {
+        if (mHandleType != HandleType.MIDDLE) {
             setLayoutPosition();
         }
     }
 
     void positionFromGecko(int left, int top, boolean rtl) {
-        LayerView layerView = mActivity.getLayerView();
+        LayerView layerView = GeckoAppShell.getLayerView();
         if (layerView == null) {
             Log.e(LOGTAG, "Can't position handle because layerView is null");
             return;
@@ -170,12 +166,13 @@ class TextSelectionHandle extends ImageView implements View.OnTouchListener {
     }
 
     private float adjustLeftForHandle() {
-        if (mHandleType.equals(HandleType.START))
+        if (mHandleType == HandleType.START) {
             return mIsRTL ? mShadow : mWidth - mShadow;
-        else if (mHandleType.equals(HandleType.MIDDLE))
+        } else if (mHandleType == HandleType.MIDDLE) {
             return mWidth / 2;
-        else
+        } else {
             return mIsRTL ? mWidth - mShadow : mShadow;
+        }
     }
 
     private void setLayoutPosition() {
