@@ -10,6 +10,7 @@ import java.util.Set;
 
 import org.mozilla.gecko.R;
 import org.mozilla.gecko.background.common.log.Logger;
+import org.mozilla.gecko.background.fxa.FxAccountUtils;
 import org.mozilla.gecko.background.preferences.PreferenceFragment;
 import org.mozilla.gecko.fxa.FirefoxAccounts;
 import org.mozilla.gecko.fxa.FxAccountConstants;
@@ -21,6 +22,7 @@ import org.mozilla.gecko.fxa.tasks.FxAccountCodeResender;
 import org.mozilla.gecko.sync.ExtendedJSONObject;
 import org.mozilla.gecko.sync.SharedPreferencesClientsDataDelegate;
 import org.mozilla.gecko.sync.SyncConfiguration;
+import org.mozilla.gecko.util.HardwareUtils;
 
 import android.accounts.Account;
 import android.content.ContentResolver;
@@ -37,6 +39,7 @@ import android.preference.Preference.OnPreferenceClickListener;
 import android.preference.PreferenceCategory;
 import android.preference.PreferenceScreen;
 import android.text.TextUtils;
+import android.text.format.DateUtils;
 
 /**
  * A fragment that displays the status of an AndroidFxAccount.
@@ -84,6 +87,8 @@ public class FxAccountStatusFragment
 
   protected EditTextPreference deviceNamePreference;
   protected Preference syncServerPreference;
+  protected Preference morePreference;
+  protected Preference syncNowPreference;
 
   protected volatile AndroidFxAccount fxAccount;
   // The contract is: when fxAccount is non-null, then clientsDataDelegate is
@@ -111,6 +116,13 @@ public class FxAccountStatusFragment
   @Override
   public void onCreate(Bundle savedInstanceState) {
     super.onCreate(savedInstanceState);
+
+    // We need to do this before we can query the hardware menu button state.
+    // We're guaranteed to have an activity at this point (onAttach is called
+    // before onCreate). It's okay to call this multiple times (with different
+    // contexts).
+    HardwareUtils.init(getActivity());
+
     addPreferences();
   }
 
@@ -134,7 +146,7 @@ public class FxAccountStatusFragment
     tabsPreference = (CheckBoxPreference) ensureFindPreference("tabs");
     passwordsPreference = (CheckBoxPreference) ensureFindPreference("passwords");
 
-    if (!FxAccountConstants.LOG_PERSONAL_INFORMATION) {
+    if (!FxAccountUtils.LOG_PERSONAL_INFORMATION) {
       removeDebugButtons();
     } else {
       connectDebugButtons();
@@ -155,6 +167,16 @@ public class FxAccountStatusFragment
     deviceNamePreference.setOnPreferenceChangeListener(this);
 
     syncServerPreference = ensureFindPreference("sync_server");
+    morePreference = ensureFindPreference("more");
+    morePreference.setOnPreferenceClickListener(this);
+
+    syncNowPreference = ensureFindPreference("sync_now");
+    syncNowPreference.setEnabled(true);
+    syncNowPreference.setOnPreferenceClickListener(this);
+
+    if (HardwareUtils.hasMenuButton()) {
+      syncCategory.removePreference(morePreference);
+    }
   }
 
   /**
@@ -209,6 +231,18 @@ public class FxAccountStatusFragment
       return true;
     }
 
+    if (preference == morePreference) {
+      getActivity().openOptionsMenu();
+      return true;
+    }
+
+    if (preference == syncNowPreference) {
+      if (fxAccount != null) {
+        FirefoxAccounts.requestSync(fxAccount.getAndroidAccount(), FirefoxAccounts.FORCE, null, null);
+      }
+      return true;
+    }
+
     return false;
   }
 
@@ -230,6 +264,7 @@ public class FxAccountStatusFragment
     passwordsPreference.setEnabled(enabled);
     // Since we can't sync, we can't update our remote client record.
     deviceNamePreference.setEnabled(enabled);
+    syncNowPreference.setEnabled(enabled);
   }
 
   /**
@@ -348,7 +383,7 @@ public class FxAccountStatusFragment
     }
     this.fxAccount = fxAccount;
     try {
-      this.clientsDataDelegate = new SharedPreferencesClientsDataDelegate(fxAccount.getSyncPrefs());
+      this.clientsDataDelegate = new SharedPreferencesClientsDataDelegate(fxAccount.getSyncPrefs(), getActivity().getApplicationContext());
     } catch (Exception e) {
       Logger.error(LOG_TAG, "Got exception fetching Sync prefs associated to Firefox Account; aborting.", e);
       // Something is terribly wrong; best to get a stack trace rather than
@@ -450,6 +485,26 @@ public class FxAccountStatusFragment
     final String clientName = clientsDataDelegate.getClientName();
     deviceNamePreference.setSummary(clientName);
     deviceNamePreference.setText(clientName);
+
+    updateSyncNowPreference();
+  }
+
+  // This is a helper function similar to TabsAccessor.getLastSyncedString() to calculate relative "Last synced" time span.
+  private String getLastSyncedString(final long startTime) {
+    final CharSequence relativeTimeSpanString = DateUtils.getRelativeTimeSpanString(startTime);
+    return getActivity().getResources().getString(R.string.fxaccount_status_last_synced, relativeTimeSpanString);
+  }
+
+  protected void updateSyncNowPreference() {
+    final boolean currentlySyncing = fxAccount.isCurrentlySyncing();
+    syncNowPreference.setEnabled(!currentlySyncing);
+    if (currentlySyncing) {
+      syncNowPreference.setTitle(R.string.fxaccount_status_syncing);
+    } else {
+      syncNowPreference.setTitle(R.string.fxaccount_status_sync_now);
+    }
+    final String lastSynced = getLastSyncedString(fxAccount.getLastSyncedTimestamp());
+    syncNowPreference.setSummary(lastSynced);
   }
 
   protected void updateAuthServerPreference() {
@@ -656,7 +711,7 @@ public class FxAccountStatusFragment
   }
 
   /**
-   * Iterate through debug buttons, adding a special deubg preference click
+   * Iterate through debug buttons, adding a special debug preference click
    * listener to each of them.
    */
   protected void connectDebugButtons() {
