@@ -142,23 +142,10 @@ public class BookmarksPanel extends HomeFragment {
     public void onConfigurationChanged(Configuration newConfig) {
         super.onConfigurationChanged(newConfig);
 
-        // Reattach the fragment, forcing a reinflation of its view.
-        // We use commitAllowingStateLoss() instead of commit() here to avoid
-        // an IllegalStateException. If the phone is rotated while Fennec
-        // is in the background, onConfigurationChanged() is fired.
-        // onConfigurationChanged() is called before onResume(), so
-        // using commit() would throw an IllegalStateException since it can't
-        // be used between the Activity's onSaveInstanceState() and
-        // onResume().
         if (isVisible()) {
             // The parent stack is saved just so that the folder state can be
             // restored on rotation.
             mSavedParentStack = mListAdapter.getParentStack();
-
-            getFragmentManager().beginTransaction()
-                                .detach(this)
-                                .attach(this)
-                                .commitAllowingStateLoss();
         }
     }
 
@@ -229,7 +216,7 @@ public class BookmarksPanel extends HomeFragment {
     /**
      * Loader callbacks for the LoaderManager of this fragment.
      */
-    private class CursorLoaderCallbacks implements LoaderCallbacks<Cursor> {
+    private class CursorLoaderCallbacks extends TransitionAwareCursorLoaderCallbacks {
         @Override
         public Loader<Cursor> onCreateLoader(int id, Bundle args) {
             if (args == null) {
@@ -242,7 +229,7 @@ public class BookmarksPanel extends HomeFragment {
         }
 
         @Override
-        public void onLoadFinished(Loader<Cursor> loader, Cursor c) {
+        public void onLoadFinishedAfterTransitions(Loader<Cursor> loader, Cursor c) {
             BookmarksLoader bl = (BookmarksLoader) loader;
             mListAdapter.swapCursor(c, bl.getFolderInfo(), bl.getRefreshType());
             updateUiFromCursor(c);
@@ -250,6 +237,8 @@ public class BookmarksPanel extends HomeFragment {
 
         @Override
         public void onLoaderReset(Loader<Cursor> loader) {
+            super.onLoaderReset(loader);
+
             if (mList != null) {
                 mListAdapter.swapCursor(null);
             }
