@@ -5,13 +5,18 @@
 
 package org.mozilla.gecko.home;
 
+import org.mozilla.gecko.NewTabletUI;
 import org.mozilla.gecko.R;
 import org.mozilla.gecko.ThumbnailHelper;
 
 import android.content.Context;
+import android.content.res.Resources;
+import android.graphics.Bitmap;
 import android.graphics.Canvas;
+import android.graphics.Matrix;
 import android.graphics.Paint;
 import android.graphics.PorterDuff.Mode;
+import android.graphics.RectF;
 import android.graphics.drawable.Drawable;
 import android.util.AttributeSet;
 import android.widget.ImageView;
@@ -26,26 +31,24 @@ public class TopSitesThumbnailView extends ImageView {
     private static final int COLOR_FILTER = 0x46FFFFFF;
 
     // Default filter color for "Add a bookmark" views.
-    private static final int DEFAULT_COLOR = 0xFFECF0F3;
+    private final int mDefaultColor = getResources().getColor(R.color.top_site_default);
 
     // Stroke width for the border.
     private final float mStrokeWidth = getResources().getDisplayMetrics().density * 2;
 
     // Paint for drawing the border.
-    private static final Paint sBorderPaint;
+    private final Paint mBorderPaint;
 
-    // Initializing the static border paint.
-    static {
-        sBorderPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
-        sBorderPaint.setColor(0xFFCFD9E1);
-        sBorderPaint.setStyle(Paint.Style.STROKE);
-    }
+    private boolean mResize = false;
+    private int mWidth;
+    private int mHeight;
 
     public TopSitesThumbnailView(Context context) {
         this(context, null);
 
         // A border will be drawn if needed.
         setWillNotDraw(false);
+
     }
 
     public TopSitesThumbnailView(Context context, AttributeSet attrs) {
@@ -54,6 +57,41 @@ public class TopSitesThumbnailView extends ImageView {
 
     public TopSitesThumbnailView(Context context, AttributeSet attrs, int defStyle) {
         super(context, attrs, defStyle);
+
+        // Initialize the border paint.
+        final Resources res = getResources();
+        mBorderPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
+        mBorderPaint.setColor(res.getColor(R.color.top_site_border));
+        mBorderPaint.setStyle(Paint.Style.STROKE);
+    }
+
+    public void setImageBitmap(Bitmap bm, boolean resize) {
+        super.setImageBitmap(bm);
+        mResize = resize;
+    }
+
+    @Override
+    public void setImageResource(int resId) {
+        super.setImageResource(resId);
+        mResize = false;
+    }
+
+    @Override
+    public void setImageDrawable(Drawable drawable) {
+        super.setImageDrawable(drawable);
+        mResize = false;
+    }
+
+    @Override
+    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
+        super.onLayout(changed, left, top, right, bottom);
+        if(NewTabletUI.isEnabled(getContext()) && mResize) {
+            setScaleType(ScaleType.MATRIX);
+            RectF rect = new RectF(0, 0, mWidth, mHeight);
+            Matrix matrix = new Matrix();
+            matrix.setRectToRect(rect, rect, Matrix.ScaleToFit.CENTER);
+            setImageMatrix(matrix);
+        }
     }
 
     /**
@@ -69,9 +107,9 @@ public class TopSitesThumbnailView extends ImageView {
         super.onMeasure(widthMeasureSpec, heightMeasureSpec);
 
         // Force the height based on the aspect ratio.
-        final int width = getMeasuredWidth();
-        final int height = (int) (width * ThumbnailHelper.THUMBNAIL_ASPECT_RATIO);
-        setMeasuredDimension(width, height);
+        mWidth = getMeasuredWidth();
+        mHeight = (int) (mWidth * ThumbnailHelper.THUMBNAIL_ASPECT_RATIO);
+        setMeasuredDimension(mWidth, mHeight);
     }
 
     /**
@@ -82,8 +120,8 @@ public class TopSitesThumbnailView extends ImageView {
         super.onDraw(canvas);
 
         if (getBackground() == null) {
-            sBorderPaint.setStrokeWidth(mStrokeWidth);
-            canvas.drawRect(0, 0, getWidth(), getHeight(), sBorderPaint);
+            mBorderPaint.setStrokeWidth(mStrokeWidth);
+            canvas.drawRect(0, 0, getWidth(), getHeight(), mBorderPaint);
         }
     }
 
@@ -104,7 +142,7 @@ public class TopSitesThumbnailView extends ImageView {
     @Override
     public void setBackgroundColor(int color) {
         if (color == 0) {
-            color = DEFAULT_COLOR;
+            color = mDefaultColor;
         }
 
         Drawable drawable = getResources().getDrawable(R.drawable.top_sites_thumbnail_bg);
