@@ -16,9 +16,12 @@ import org.mozilla.gecko.NewTabletUI;
 import org.mozilla.gecko.R;
 import org.mozilla.gecko.SiteIdentity;
 import org.mozilla.gecko.SiteIdentity.SecurityMode;
+import org.mozilla.gecko.SiteIdentity.MixedMode;
+import org.mozilla.gecko.SiteIdentity.TrackingMode;
 import org.mozilla.gecko.Tab;
 import org.mozilla.gecko.animation.PropertyAnimator;
 import org.mozilla.gecko.animation.ViewHelper;
+import org.mozilla.gecko.favicons.Favicons;
 import org.mozilla.gecko.toolbar.BrowserToolbarTabletBase.ForwardButtonAnimation;
 import org.mozilla.gecko.util.StringUtils;
 import org.mozilla.gecko.widget.ThemedLinearLayout;
@@ -120,7 +123,10 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
     private TranslateAnimation mTitleSlideRight;
 
     private final SiteIdentityPopup mSiteIdentityPopup;
-    private SecurityMode mSecurityMode;
+    private int mSecurityImageLevel;
+
+    private final int LEVEL_SHIELD_ENABLED = 3;
+    private final int LEVEL_SHIELD_DISABLED = 4;
 
     private PropertyAnimator mForwardAnim;
 
@@ -178,7 +184,7 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
             if (Versions.feature16Plus) {
                 mFavicon.setImportantForAccessibility(View.IMPORTANT_FOR_ACCESSIBILITY_NO);
             }
-            mFaviconSize = Math.round(res.getDimension(R.dimen.browser_toolbar_favicon_size));
+            mFaviconSize = Math.round(Favicons.browserToolbarFaviconSize);
         }
 
         mSiteSecurityVisible = (mSiteSecurity.getVisibility() == View.VISIBLE);
@@ -421,15 +427,34 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
         mSiteIdentityPopup.setSiteIdentity(siteIdentity);
 
         final SecurityMode securityMode;
+        final MixedMode mixedMode;
+        final TrackingMode trackingMode;
         if (siteIdentity == null) {
             securityMode = SecurityMode.UNKNOWN;
+            mixedMode = MixedMode.UNKNOWN;
+            trackingMode = TrackingMode.UNKNOWN;
         } else {
             securityMode = siteIdentity.getSecurityMode();
+            mixedMode = siteIdentity.getMixedMode();
+            trackingMode = siteIdentity.getTrackingMode();
         }
 
-        if (mSecurityMode != securityMode) {
-            mSecurityMode = securityMode;
-            mSiteSecurity.setImageLevel(mSecurityMode.ordinal());
+        // This is a bit tricky, but we have one icon and three potential indicators.
+        // Default to the identity level
+        int imageLevel = securityMode.ordinal();
+
+        // Check to see if any protection was overridden first
+        if (trackingMode == TrackingMode.TRACKING_CONTENT_LOADED ||
+            mixedMode == MixedMode.MIXED_CONTENT_LOADED) {
+          imageLevel = LEVEL_SHIELD_DISABLED;
+        } else if (trackingMode == TrackingMode.TRACKING_CONTENT_BLOCKED ||
+                   mixedMode == MixedMode.MIXED_CONTENT_BLOCKED) {
+          imageLevel = LEVEL_SHIELD_ENABLED;
+        }
+
+        if (mSecurityImageLevel != imageLevel) {
+            mSecurityImageLevel = imageLevel;
+            mSiteSecurity.setImageLevel(mSecurityImageLevel);
             updatePageActions(flags);
         }
     }
@@ -466,8 +491,7 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
         mStop.setVisibility(isShowingProgress ? View.VISIBLE : View.GONE);
         mPageActionLayout.setVisibility(!isShowingProgress ? View.VISIBLE : View.GONE);
 
-        boolean shouldShowSiteSecurity = (!isShowingProgress &&
-                                          mSecurityMode != SecurityMode.UNKNOWN);
+        boolean shouldShowSiteSecurity = (!isShowingProgress && mSecurityImageLevel > 0);
 
         setSiteSecurityVisibility(shouldShowSiteSecurity, flags);
 
@@ -535,6 +559,8 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
         mForwardAnim = anim;
 
         if (animation == ForwardButtonAnimation.HIDE) {
+            // We animate these items individually, rather than this entire view,
+            // so that we don't animate certain views, e.g. the stop button.
             anim.attach(mTitle,
                         PropertyAnimator.Property.TRANSLATION_X,
                         0);
@@ -580,7 +606,7 @@ public class ToolbarDisplayLayout extends ThemedLinearLayout
 
     void prepareStopEditingAnimation(PropertyAnimator anim) {
         // Fade toolbar buttons (page actions, stop) after the entry
-        // is schrunk back to its original size.
+        // is shrunk back to its original size.
         anim.attach(mPageActionLayout,
                     PropertyAnimator.Property.ALPHA,
                     1);
