@@ -10,6 +10,7 @@ import org.mozilla.gecko.mozglue.RobocopTarget;
 import java.util.Map;
 
 import android.os.Handler;
+import android.os.Looper;
 import android.os.MessageQueue;
 import android.util.Log;
 
@@ -25,10 +26,10 @@ public final class ThreadUtils {
         THROW,
     }
 
-    private static volatile Thread sUiThread;
-    private static volatile Thread sBackgroundThread;
+    private static final Thread sUiThread = Looper.getMainLooper().getThread();
+    private static final Handler sUiHandler = new Handler(Looper.getMainLooper());
 
-    private static Handler sUiHandler;
+    private static volatile Thread sBackgroundThread;
 
     // Referenced directly from GeckoAppShell in highly performance-sensitive code (The extra
     // function call of the getter was harming performance. (Bug 897123))
@@ -36,7 +37,7 @@ public final class ThreadUtils {
     // this out at compile time.
     public static Handler sGeckoHandler;
     public static MessageQueue sGeckoQueue;
-    public static Thread sGeckoThread;
+    public static volatile Thread sGeckoThread;
 
     // Delayed Runnable that resets the Gecko thread priority.
     private static final Runnable sPriorityResetRunnable = new Runnable() {
@@ -79,11 +80,6 @@ public final class ThreadUtils {
         }
     }
 
-    public static void setUiThread(Thread thread, Handler handler) {
-        sUiThread = thread;
-        sUiHandler = handler;
-    }
-
     public static void setBackgroundThread(Thread thread) {
         sBackgroundThread = thread;
     }
@@ -130,6 +126,10 @@ public final class ThreadUtils {
     }
 
     public static void assertNotOnGeckoThread() {
+        if (sGeckoThread == null) {
+            // Cannot be on Gecko thread if Gecko thread is not live yet.
+            return;
+        }
         assertNotOnThread(sGeckoThread, AssertBehavior.THROW);
     }
 
@@ -178,10 +178,18 @@ public final class ThreadUtils {
         }
     }
 
+    public static boolean isOnGeckoThread() {
+        if (sGeckoThread != null) {
+            return isOnThread(sGeckoThread);
+        }
+        return false;
+    }
+
     public static boolean isOnUiThread() {
         return isOnThread(getUiThread());
     }
 
+    @RobocopTarget
     public static boolean isOnBackgroundThread() {
         if (sBackgroundThread == null) {
             return false;
@@ -190,6 +198,7 @@ public final class ThreadUtils {
         return isOnThread(sBackgroundThread);
     }
 
+    @RobocopTarget
     public static boolean isOnThread(Thread thread) {
         return (Thread.currentThread().getId() == thread.getId());
     }
@@ -204,7 +213,13 @@ public final class ThreadUtils {
      * @param timeout Timeout in ms after which the priority will be reset
      */
     public static void reduceGeckoPriority(long timeout) {
-        if (!sIsGeckoPriorityReduced) {
+        if (Runtime.getRuntime().availableProcessors() > 1) {
+            // Don't reduce priority for multicore devices. We use availableProcessors()
+            // for its fast performance. It may give false negatives (i.e. multicore
+            // detected as single-core), but we can tolerate this behavior.
+            return;
+        }
+        if (!sIsGeckoPriorityReduced && sGeckoThread != null) {
             sIsGeckoPriorityReduced = true;
             sGeckoThread.setPriority(Thread.MIN_PRIORITY);
             getUiHandler().postDelayed(sPriorityResetRunnable, timeout);
