@@ -502,8 +502,11 @@ class JSRuntimeWrapper
  public:
   static JSRuntimeWrapper *Create()
   {
-    JSRuntimeWrapper *entry = new JSRuntimeWrapper();
+    JSRuntime *runtime = JS_NewRuntime(sRuntimeHeapSize);
+    if (NS_WARN_IF(!runtime))
+      return nullptr;
 
+    JSRuntimeWrapper *entry = new JSRuntimeWrapper(runtime);
     if (NS_FAILED(entry->Init())) {
       delete entry;
       return nullptr;
@@ -524,6 +527,8 @@ class JSRuntimeWrapper
 
   ~JSRuntimeWrapper()
   {
+    mGlobal = nullptr;
+
     MOZ_COUNT_DTOR(JSRuntimeWrapper);
     if (mContext) {
       JS_DestroyContext(mContext);
@@ -549,22 +554,19 @@ private:
 
   JSRuntime *mRuntime;
   JSContext *mContext;
-  JSObject  *mGlobal;
+  JS::PersistentRooted<JSObject*> mGlobal;
   bool      mOK;
 
   static const JSClass sGlobalClass;
 
-  JSRuntimeWrapper()
-    : mRuntime(nullptr), mContext(nullptr), mGlobal(nullptr), mOK(false)
+  JSRuntimeWrapper(JSRuntime* rt)
+     : mRuntime(rt), mContext(nullptr), mGlobal(rt, nullptr), mOK(false)
   {
       MOZ_COUNT_CTOR(JSRuntimeWrapper);
   }
 
   nsresult Init()
   {
-    mRuntime = JS_NewRuntime(sRuntimeHeapSize);
-    NS_ENSURE_TRUE(mRuntime, NS_ERROR_OUT_OF_MEMORY);
-
     /*
      * Not setting this will cause JS_CHECK_RECURSION to report false
      * positives
