@@ -8,12 +8,21 @@
 #include "CacheFileChunk.h"
 #include "CacheFileInputStream.h"
 #include "CacheFileOutputStream.h"
-#include "CacheIndex.h"
 #include "nsThreadUtils.h"
 #include "mozilla/DebugOnly.h"
 #include <algorithm>
 #include "nsComponentManagerUtils.h"
 #include "nsProxyRelease.h"
+#include "mozilla/Telemetry.h"
+
+// When CACHE_CHUNKS is defined we always cache unused chunks in mCacheChunks.
+// When it is not defined, we always release the chunks ASAP, i.e. we cache
+// unused chunks only when:
+//  - CacheFile is memory-only
+//  - CacheFile is still waiting for the handle
+//  - the chunk is preloaded
+
+//#define CACHE_CHUNKS
 
 namespace mozilla {
 namespace net {
@@ -32,6 +41,7 @@ public:
     MOZ_COUNT_CTOR(NotifyCacheFileListenerEvent);
   }
 
+protected:
   ~NotifyCacheFileListenerEvent()
   {
     LOG(("NotifyCacheFileListenerEvent::~NotifyCacheFileListenerEvent() "
@@ -39,6 +49,7 @@ public:
     MOZ_COUNT_DTOR(NotifyCacheFileListenerEvent);
   }
 
+public:
   NS_IMETHOD Run()
   {
     LOG(("NotifyCacheFileListenerEvent::Run() [this=%p]", this));
@@ -69,6 +80,7 @@ public:
     MOZ_COUNT_CTOR(NotifyChunkListenerEvent);
   }
 
+protected:
   ~NotifyChunkListenerEvent()
   {
     LOG(("NotifyChunkListenerEvent::~NotifyChunkListenerEvent() [this=%p]",
@@ -76,6 +88,7 @@ public:
     MOZ_COUNT_DTOR(NotifyChunkListenerEvent);
   }
 
+public:
   NS_IMETHOD Run()
   {
     LOG(("NotifyChunkListenerEvent::Run() [this=%p]", this));
@@ -97,7 +110,7 @@ class DoomFileHelper : public CacheFileIOListener
 public:
   NS_DECL_THREADSAFE_ISUPPORTS
 
-  DoomFileHelper(CacheFileListener *aListener)
+  explicit DoomFileHelper(CacheFileListener *aListener)
     : mListener(aListener)
   {
     MOZ_COUNT_CTOR(DoomFileHelper);
@@ -151,7 +164,7 @@ private:
   nsCOMPtr<CacheFileListener>  mListener;
 };
 
-NS_IMPL_ISUPPORTS1(DoomFileHelper, CacheFileIOListener)
+NS_IMPL_ISUPPORTS(DoomFileHelper, CacheFileIOListener)
 
 
 NS_IMPL_ADDREF(CacheFile)
@@ -169,9 +182,13 @@ CacheFile::CacheFile()
   , mOpeningFile(false)
   , mReady(false)
   , mMemoryOnly(false)
+  , mOpenAsMemoryOnly(false)
+  , mPriority(false)
   , mDataAccessed(false)
   , mDataIsDirty(false)
   , mWritingMetadata(false)
+  , mPreloadWithoutInputStreams(true)
+  , mPreloadChunkCount(0)
   , mStatus(NS_OK)
   , mDataSize(-1)
   , mOutput(nullptr)
@@ -203,15 +220,27 @@ CacheFile::Init(const nsACString &aKey,
   nsresult rv;
 
   mKey = aKey;
-  mMemoryOnly = aMemoryOnly;
+  mOpenAsMemoryOnly = mMemoryOnly = aMemoryOnly;
+  mPriority = aPriority;
+
+  // Some consumers (at least nsHTTPCompressConv) assume that Read() can read
+  // such amount of data that was announced by Available().
+  // CacheFileInputStream::Available() uses also preloaded chunks to compute
+  // number of available bytes in the input stream, so we have to make sure the
+  // preloadChunkCount won't change during CacheFile's lifetime since otherwise
+  // we could potentially release some cached chunks that was used to calculate
+  // available bytes but would not be available later during call to
+  // CacheFileInputStream::Read().
+  mPreloadChunkCount = CacheObserver::PreloadChunkCount();
 
   LOG(("CacheFile::Init() [this=%p, key=%s, createNew=%d, memoryOnly=%d, "
-       "listener=%p]", this, mKey.get(), aCreateNew, aMemoryOnly, aCallback));
+       "priority=%d, listener=%p]", this, mKey.get(), aCreateNew, aMemoryOnly,
+       aPriority, aCallback));
 
   if (mMemoryOnly) {
     MOZ_ASSERT(!aCallback);
 
-    mMetadata = new CacheFileMetadata(mKey);
+    mMetadata = new CacheFileMetadata(mOpenAsMemoryOnly, mKey);
     mReady = true;
     mDataSize = mMetadata->Offset();
     return NS_OK;
@@ -223,40 +252,16 @@ CacheFile::Init(const nsACString &aKey,
       flags = CacheFileIOManager::CREATE_NEW;
 
       // make sure we can use this entry immediately
-      mMetadata = new CacheFileMetadata(mKey);
+      mMetadata = new CacheFileMetadata(mOpenAsMemoryOnly, mKey);
       mReady = true;
       mDataSize = mMetadata->Offset();
-    }
-    else {
+    } else {
       flags = CacheFileIOManager::CREATE;
-
-      // Have a look into index and change to CREATE_NEW when we are sure
-      // that the entry does not exist.
-      CacheIndex::EntryStatus status;
-      rv = CacheIndex::HasEntry(mKey, &status);
-      if (status == CacheIndex::DOES_NOT_EXIST) {
-        LOG(("CacheFile::Init() - Forcing CREATE_NEW flag since we don't have"
-             " this entry according to index"));
-        flags = CacheFileIOManager::CREATE_NEW;
-
-        // make sure we can use this entry immediately
-        mMetadata = new CacheFileMetadata(mKey);
-        mReady = true;
-        mDataSize = mMetadata->Offset();
-
-        // Notify callback now and don't store it in mListener, no further
-        // operation can change the result.
-        nsRefPtr<NotifyCacheFileListenerEvent> ev;
-        ev = new NotifyCacheFileListenerEvent(aCallback, NS_OK, true);
-        rv = NS_DispatchToCurrentThread(ev);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        aCallback = nullptr;
-      }
     }
 
-    if (aPriority)
+    if (mPriority) {
       flags |= CacheFileIOManager::PRIORITY;
+    }
 
     mOpeningFile = true;
     mListener = aCallback;
@@ -280,7 +285,7 @@ CacheFile::Init(const nsACString &aKey,
              "initializing entry as memory-only. [this=%p]", this));
 
         mMemoryOnly = true;
-        mMetadata = new CacheFileMetadata(mKey);
+        mMetadata = new CacheFileMetadata(mOpenAsMemoryOnly, mKey);
         mReady = true;
         mDataSize = mMetadata->Offset();
 
@@ -307,10 +312,12 @@ CacheFile::OnChunkRead(nsresult aResult, CacheFileChunk *aChunk)
 
   uint32_t index = aChunk->Index();
 
-  LOG(("CacheFile::OnChunkRead() [this=%p, rv=0x%08x, chunk=%p, idx=%d]",
+  LOG(("CacheFile::OnChunkRead() [this=%p, rv=0x%08x, chunk=%p, idx=%u]",
        this, aResult, aChunk, index));
 
-  // TODO handle ERROR state
+  if (NS_FAILED(aResult)) {
+    SetError(aResult);
+  }
 
   if (HaveChunkListeners(index)) {
     rv = NotifyChunkListeners(index, aResult, aChunk);
@@ -323,22 +330,26 @@ CacheFile::OnChunkRead(nsresult aResult, CacheFileChunk *aChunk)
 nsresult
 CacheFile::OnChunkWritten(nsresult aResult, CacheFileChunk *aChunk)
 {
+  // In case the chunk was reused, made dirty and released between calls to
+  // CacheFileChunk::Write() and CacheFile::OnChunkWritten(), we must write
+  // the chunk to the disk again. When the chunk is unused and is dirty simply
+  // addref and release (outside the lock) the chunk which ensures that
+  // CacheFile::DeactivateChunk() will be called again.
+  nsRefPtr<CacheFileChunk> deactivateChunkAgain;
+
   CacheFileAutoLock lock(this);
 
   nsresult rv;
 
-  LOG(("CacheFile::OnChunkWritten() [this=%p, rv=0x%08x, chunk=%p, idx=%d]",
+  LOG(("CacheFile::OnChunkWritten() [this=%p, rv=0x%08x, chunk=%p, idx=%u]",
        this, aResult, aChunk, aChunk->Index()));
 
   MOZ_ASSERT(!mMemoryOnly);
-
-  // TODO handle ERROR state
+  MOZ_ASSERT(!mOpeningFile);
+  MOZ_ASSERT(mHandle);
 
   if (NS_FAILED(aResult)) {
-    // TODO ??? doom entry
-    // TODO mark this chunk as memory only, since it wasn't written to disk and
-    // therefore cannot be released from memory
-    // LOG
+    SetError(aResult);
   }
 
   if (NS_SUCCEEDED(aResult) && !aChunk->IsDirty()) {
@@ -349,7 +360,7 @@ CacheFile::OnChunkWritten(nsresult aResult, CacheFileChunk *aChunk)
   // notify listeners if there is any
   if (HaveChunkListeners(aChunk->Index())) {
     // don't release the chunk since there are some listeners queued
-    rv = NotifyChunkListeners(aChunk->Index(), NS_OK, aChunk);
+    rv = NotifyChunkListeners(aChunk->Index(), aResult, aChunk);
     if (NS_SUCCEEDED(rv)) {
       MOZ_ASSERT(aChunk->mRefCnt != 2);
       return NS_OK;
@@ -363,14 +374,26 @@ CacheFile::OnChunkWritten(nsresult aResult, CacheFileChunk *aChunk)
     return NS_OK;
   }
 
-  LOG(("CacheFile::OnChunkWritten() - Caching unused chunk [this=%p, chunk=%p]",
-       this, aChunk));
+  if (aChunk->IsDirty()) {
+    LOG(("CacheFile::OnChunkWritten() - Unused chunk is dirty. We must go "
+         "through deactivation again. [this=%p, chunk=%p]", this, aChunk));
+
+    deactivateChunkAgain = aChunk;
+    return NS_OK;
+  }
+
+  bool keepChunk = false;
+  if (NS_SUCCEEDED(aResult)) {
+    keepChunk = ShouldCacheChunk(aChunk->Index());
+    LOG(("CacheFile::OnChunkWritten() - %s unused chunk [this=%p, chunk=%p]",
+         keepChunk ? "Caching" : "Releasing", this, aChunk));
+  } else {
+    LOG(("CacheFile::OnChunkWritten() - Releasing failed chunk [this=%p, "
+         "chunk=%p]", this, aChunk));
+  }
+
+  RemoveChunkInternal(aChunk, keepChunk);
 
-  aChunk->mRemovingChunk = true;
-  ReleaseOutsideLock(static_cast<CacheFileChunkListener *>(
-                       aChunk->mFile.forget().take()));
-  mCachedChunks.Put(aChunk->Index(), aChunk);
-  mChunks.Remove(aChunk->Index());
   WriteMetadataIfNeededLocked();
 
   return NS_OK;
@@ -401,7 +424,7 @@ CacheFile::OnFileOpened(CacheFileHandle *aHandle, nsresult aResult)
   class AutoFailDoomListener
   {
   public:
-    AutoFailDoomListener(CacheFileHandle *aHandle)
+    explicit AutoFailDoomListener(CacheFileHandle *aHandle)
       : mHandle(aHandle)
       , mAlreadyDoomed(false)
     {}
@@ -477,7 +500,7 @@ CacheFile::OnFileOpened(CacheFileHandle *aHandle, nsresult aResult)
              this));
 
         mMemoryOnly = true;
-        mMetadata = new CacheFileMetadata(mKey);
+        mMetadata = new CacheFileMetadata(mOpenAsMemoryOnly, mKey);
         mReady = true;
         mDataSize = mMetadata->Offset();
 
@@ -494,6 +517,9 @@ CacheFile::OnFileOpened(CacheFileHandle *aHandle, nsresult aResult)
     }
     else {
       mHandle = aHandle;
+      if (NS_FAILED(mStatus)) {
+        CacheFileIOManager::DoomFile(mHandle, nullptr);
+      }
 
       if (mMetadata) {
         InitIndexEntry();
@@ -558,6 +584,9 @@ CacheFile::OnMetadataRead(nsresult aResult)
     if (mDataSize == 0 && mMetadata->ElementsSize() == 0) {
       isNew = true;
       mMetadata->MarkDirty();
+    } else {
+      CacheFileAutoLock lock(this);
+      PreloadChunks(0);
     }
 
     InitIndexEntry();
@@ -649,6 +678,24 @@ CacheFile::OpenInputStream(nsIInputStream **_retval)
     return NS_ERROR_NOT_AVAILABLE;
   }
 
+  if (NS_FAILED(mStatus)) {
+    LOG(("CacheFile::OpenInputStream() - CacheFile is in a failure state "
+         "[this=%p, status=0x%08x]", this, mStatus));
+
+    // Don't allow opening the input stream when this CacheFile is in
+    // a failed state.  This is the only way to protect consumers correctly
+    // from reading a broken entry.  When the file is in the failed state,
+    // it's also doomed, so reopening the entry won't make any difference -
+    // data will still be inaccessible anymore.  Note that for just doomed 
+    // files, we must allow reading the data.
+    return mStatus;
+  }
+
+  // Once we open input stream we no longer allow preloading of chunks without
+  // input stream, i.e. we will no longer keep first few chunks preloaded when
+  // the last input stream is closed.
+  mPreloadWithoutInputStreams = false;
+
   CacheFileInputStream *input = new CacheFileInputStream(this);
 
   LOG(("CacheFile::OpenInputStream() - Creating new input stream %p [this=%p]",
@@ -683,6 +730,12 @@ CacheFile::OpenOutputStream(CacheOutputCloseListener *aCloseListener, nsIOutputS
     return NS_ERROR_NOT_AVAILABLE;
   }
 
+  // Once we open output stream we no longer allow preloading of chunks without
+  // input stream. There is no reason to believe that some input stream will be
+  // opened soon. Otherwise we would cache unused chunks of all newly created
+  // entries until the CacheFile is destroyed.
+  mPreloadWithoutInputStreams = false;
+
   mOutput = new CacheFileOutputStream(this, aCloseListener);
 
   LOG(("CacheFile::OpenOutputStream() - Creating new output stream %p "
@@ -724,11 +777,19 @@ CacheFile::SetMemoryOnly()
 nsresult
 CacheFile::Doom(CacheFileListener *aCallback)
 {
+  LOG(("CacheFile::Doom() [this=%p, listener=%p]", this, aCallback));
+
   CacheFileAutoLock lock(this);
 
+  return DoomLocked(aCallback);
+}
+
+nsresult
+CacheFile::DoomLocked(CacheFileListener *aCallback)
+{
   MOZ_ASSERT(mHandle || mMemoryOnly || mOpeningFile);
 
-  LOG(("CacheFile::Doom() [this=%p, listener=%p]", this, aCallback));
+  LOG(("CacheFile::DoomLocked() [this=%p, listener=%p]", this, aCallback));
 
   nsresult rv = NS_OK;
 
@@ -780,7 +841,10 @@ CacheFile::ThrowMemoryCachedData()
     return NS_ERROR_ABORT;
   }
 
-  mCachedChunks.Clear();
+  // We cannot release all cached chunks since we need to keep preloaded chunks
+  // in memory. See initialization of mPreloadChunkCount for explanation.
+  mCachedChunks.Enumerate(&CacheFile::CleanUpCachedChunks, this);
+
   return NS_OK;
 }
 
@@ -811,6 +875,17 @@ CacheFile::SetElement(const char *aKey, const char *aValue)
   return mMetadata->SetElement(aKey, aValue);
 }
 
+nsresult
+CacheFile::VisitMetaData(nsICacheEntryMetaDataVisitor *aVisitor)
+{
+  CacheFileAutoLock lock(this);
+  MOZ_ASSERT(mMetadata);
+  MOZ_ASSERT(mReady);
+  NS_ENSURE_TRUE(mMetadata, NS_ERROR_UNEXPECTED);
+
+  return mMetadata->Visit(aVisitor);
+}
+
 nsresult
 CacheFile::ElementsSize(uint32_t *_retval)
 {
@@ -848,27 +923,6 @@ CacheFile::GetExpirationTime(uint32_t *_retval)
   return mMetadata->GetExpirationTime(_retval);
 }
 
-nsresult
-CacheFile::SetLastModified(uint32_t aLastModified)
-{
-  CacheFileAutoLock lock(this);
-  MOZ_ASSERT(mMetadata);
-  NS_ENSURE_TRUE(mMetadata, NS_ERROR_UNEXPECTED);
-
-  PostWriteTimer();
-  return mMetadata->SetLastModified(aLastModified);
-}
-
-nsresult
-CacheFile::GetLastModified(uint32_t *_retval)
-{
-  CacheFileAutoLock lock(this);
-  MOZ_ASSERT(mMetadata);
-  NS_ENSURE_TRUE(mMetadata, NS_ERROR_UNEXPECTED);
-
-  return mMetadata->GetLastModified(_retval);
-}
-
 nsresult
 CacheFile::SetFrecency(uint32_t aFrecency)
 {
@@ -894,6 +948,16 @@ CacheFile::GetFrecency(uint32_t *_retval)
   return mMetadata->GetFrecency(_retval);
 }
 
+nsresult
+CacheFile::GetLastModified(uint32_t *_retval)
+{
+  CacheFileAutoLock lock(this);
+  MOZ_ASSERT(mMetadata);
+  NS_ENSURE_TRUE(mMetadata, NS_ERROR_UNEXPECTED);
+
+  return mMetadata->GetLastModified(_retval);
+}
+
 nsresult
 CacheFile::GetLastFetched(uint32_t *_retval)
 {
@@ -914,6 +978,18 @@ CacheFile::GetFetchCount(uint32_t *_retval)
   return mMetadata->GetFetchCount(_retval);
 }
 
+nsresult
+CacheFile::OnFetched()
+{
+  CacheFileAutoLock lock(this);
+  MOZ_ASSERT(mMetadata);
+  NS_ENSURE_TRUE(mMetadata, NS_ERROR_UNEXPECTED);
+
+  PostWriteTimer();
+
+  return mMetadata->OnFetched();
+}
+
 void
 CacheFile::Lock()
 {
@@ -947,26 +1023,32 @@ CacheFile::ReleaseOutsideLock(nsISupports *aObject)
 }
 
 nsresult
-CacheFile::GetChunk(uint32_t aIndex, bool aWriter,
+CacheFile::GetChunk(uint32_t aIndex, ECallerType aCaller,
                     CacheFileChunkListener *aCallback, CacheFileChunk **_retval)
 {
   CacheFileAutoLock lock(this);
-  return GetChunkLocked(aIndex, aWriter, aCallback, _retval);
+  return GetChunkLocked(aIndex, aCaller, aCallback, _retval);
 }
 
 nsresult
-CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
+CacheFile::GetChunkLocked(uint32_t aIndex, ECallerType aCaller,
                           CacheFileChunkListener *aCallback,
                           CacheFileChunk **_retval)
 {
   AssertOwnsLock();
 
-  LOG(("CacheFile::GetChunkLocked() [this=%p, idx=%d, writer=%d, listener=%p]",
-       this, aIndex, aWriter, aCallback));
+  LOG(("CacheFile::GetChunkLocked() [this=%p, idx=%u, caller=%d, listener=%p]",
+       this, aIndex, aCaller, aCallback));
 
   MOZ_ASSERT(mReady);
   MOZ_ASSERT(mHandle || mMemoryOnly || mOpeningFile);
-  MOZ_ASSERT((aWriter && !aCallback) || (!aWriter && aCallback));
+  MOZ_ASSERT((aCaller == READER && aCallback) ||
+             (aCaller == WRITER && !aCallback) ||
+             (aCaller == PRELOADER && !aCallback));
+
+  // Preload chunks from disk when this is disk backed entry and the listener
+  // is reader.
+  bool preload = !mMemoryOnly && (aCaller == READER);
 
   nsresult rv;
 
@@ -975,14 +1057,30 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
     LOG(("CacheFile::GetChunkLocked() - Found chunk %p in mChunks [this=%p]",
          chunk.get(), this));
 
-    if (chunk->IsReady() || aWriter) {
-      chunk.swap(*_retval);
+    // Preloader calls this method to preload only non-loaded chunks.
+    MOZ_ASSERT(aCaller != PRELOADER, "Unexpected!");
+
+    // We might get failed chunk between releasing the lock in
+    // CacheFileChunk::OnDataWritten/Read and CacheFile::OnChunkWritten/Read
+    rv = chunk->GetStatus();
+    if (NS_FAILED(rv)) {
+      SetError(rv);
+      LOG(("CacheFile::GetChunkLocked() - Found failed chunk in mChunks "
+           "[this=%p]", this));
+      return rv;
     }
-    else {
+
+    if (chunk->IsReady() || aCaller == WRITER) {
+      chunk.swap(*_retval);
+    } else {
       rv = QueueChunkListener(aIndex, aCallback);
       NS_ENSURE_SUCCESS(rv, rv);
     }
 
+    if (preload) {
+      PreloadChunks(aIndex + 1);
+    }
+
     return NS_OK;
   }
 
@@ -990,14 +1088,22 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
     LOG(("CacheFile::GetChunkLocked() - Reusing cached chunk %p [this=%p]",
          chunk.get(), this));
 
+    // Preloader calls this method to preload only non-loaded chunks.
+    MOZ_ASSERT(aCaller != PRELOADER, "Unexpected!");
+
     mChunks.Put(aIndex, chunk);
     mCachedChunks.Remove(aIndex);
     chunk->mFile = this;
-    chunk->mRemovingChunk = false;
+    chunk->mActiveChunk = true;
 
     MOZ_ASSERT(chunk->IsReady());
 
     chunk.swap(*_retval);
+
+    if (preload) {
+      PreloadChunks(aIndex + 1);
+    }
+
     return NS_OK;
   }
 
@@ -1016,8 +1122,9 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
       return NS_ERROR_UNEXPECTED;
     }
 
-    chunk = new CacheFileChunk(this, aIndex);
+    chunk = new CacheFileChunk(this, aIndex, aCaller == WRITER);
     mChunks.Put(aIndex, chunk);
+    chunk->mActiveChunk = true;
 
     LOG(("CacheFile::GetChunkLocked() - Reading newly created chunk %p from "
          "the disk [this=%p]", chunk.get(), this));
@@ -1026,34 +1133,34 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
     rv = chunk->Read(mHandle, std::min(static_cast<uint32_t>(mDataSize - off),
                      static_cast<uint32_t>(kChunkSize)),
                      mMetadata->GetHash(aIndex), this);
-    if (NS_FAILED(rv)) {
-      chunk->mRemovingChunk = true;
-      ReleaseOutsideLock(static_cast<CacheFileChunkListener *>(
-                           chunk->mFile.forget().take()));
-      mChunks.Remove(aIndex);
-      NS_ENSURE_SUCCESS(rv, rv);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      RemoveChunkInternal(chunk, false);
+      return rv;
     }
 
-    if (aWriter) {
+    if (aCaller == WRITER) {
       chunk.swap(*_retval);
-    }
-    else {
+    } else if (aCaller != PRELOADER) {
       rv = QueueChunkListener(aIndex, aCallback);
       NS_ENSURE_SUCCESS(rv, rv);
     }
 
+    if (preload) {
+      PreloadChunks(aIndex + 1);
+    }
+
     return NS_OK;
-  }
-  else if (off == mDataSize) {
-    if (aWriter) {
+  } else if (off == mDataSize) {
+    if (aCaller == WRITER) {
       // this listener is going to write to the chunk
-      chunk = new CacheFileChunk(this, aIndex);
+      chunk = new CacheFileChunk(this, aIndex, true);
       mChunks.Put(aIndex, chunk);
+      chunk->mActiveChunk = true;
 
       LOG(("CacheFile::GetChunkLocked() - Created new empty chunk %p [this=%p]",
            chunk.get(), this));
 
-      chunk->InitNew(this);
+      chunk->InitNew();
       mMetadata->SetHash(aIndex, chunk->Hash());
 
       if (HaveChunkListeners(aIndex)) {
@@ -1064,9 +1171,8 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
       chunk.swap(*_retval);
       return NS_OK;
     }
-  }
-  else {
-    if (aWriter) {
+  } else {
+    if (aCaller == WRITER) {
       // this chunk was requested by writer, but we need to fill the gap first
 
       // Fill with zero the last chunk if it is incomplete
@@ -1086,8 +1192,7 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
           rv = PadChunkWithZeroes(i);
           NS_ENSURE_SUCCESS(rv, rv);
         }
-      }
-      else {
+      } else {
         // We don't need to create CacheFileChunk for other empty chunks unless
         // there is some input stream waiting for this chunk.
 
@@ -1104,8 +1209,7 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
           if (HaveChunkListeners(i)) {
             rv = PadChunkWithZeroes(i);
             NS_ENSURE_SUCCESS(rv, rv);
-          }
-          else {
+          } else {
             mMetadata->SetHash(i, kEmptyChunkHash);
             mDataSize = (i + 1) * kChunkSize;
           }
@@ -1113,7 +1217,7 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
       }
 
       MOZ_ASSERT(mDataSize == off);
-      rv = GetChunkLocked(aIndex, true, nullptr, getter_AddRefs(chunk));
+      rv = GetChunkLocked(aIndex, WRITER, nullptr, getter_AddRefs(chunk));
       NS_ENSURE_SUCCESS(rv, rv);
 
       chunk.swap(*_retval);
@@ -1121,20 +1225,122 @@ CacheFile::GetChunkLocked(uint32_t aIndex, bool aWriter,
     }
   }
 
+  // We can be here only if the caller is reader since writer always create a
+  // new chunk above and preloader calls this method to preload only chunks that
+  // are not loaded but that do exist.
+  MOZ_ASSERT(aCaller == READER, "Unexpected!");
+
   if (mOutput) {
     // the chunk doesn't exist but mOutput may create it
     rv = QueueChunkListener(aIndex, aCallback);
     NS_ENSURE_SUCCESS(rv, rv);
-  }
-  else {
+  } else {
     return NS_ERROR_NOT_AVAILABLE;
   }
 
   return NS_OK;
 }
 
+void
+CacheFile::PreloadChunks(uint32_t aIndex)
+{
+  AssertOwnsLock();
+
+  uint32_t limit = aIndex + mPreloadChunkCount;
+
+  for (uint32_t i = aIndex; i < limit; ++i) {
+    int64_t off = i * kChunkSize;
+
+    if (off >= mDataSize) {
+      // This chunk is beyond EOF.
+      return;
+    }
+
+    if (mChunks.GetWeak(i) || mCachedChunks.GetWeak(i)) {
+      // This chunk is already in memory or is being read right now.
+      continue;
+    }
+
+    LOG(("CacheFile::PreloadChunks() - Preloading chunk [this=%p, idx=%u]",
+         this, i));
+
+    nsRefPtr<CacheFileChunk> chunk;
+    GetChunkLocked(i, PRELOADER, nullptr, getter_AddRefs(chunk));
+    // We've checked that we don't have this chunk, so no chunk must be
+    // returned.
+    MOZ_ASSERT(!chunk);
+  }
+}
+
+bool
+CacheFile::ShouldCacheChunk(uint32_t aIndex)
+{
+  AssertOwnsLock();
+
+#ifdef CACHE_CHUNKS
+  // We cache all chunks.
+  return true;
+#else
+
+  if (mPreloadChunkCount != 0 && mInputs.Length() == 0 &&
+      mPreloadWithoutInputStreams && aIndex < mPreloadChunkCount) {
+    // We don't have any input stream yet, but it is likely that some will be
+    // opened soon. Keep first mPreloadChunkCount chunks in memory. The
+    // condition is here instead of in MustKeepCachedChunk() since these
+    // chunks should be preloaded and can be kept in memory as an optimization,
+    // but they can be released at any time until they are considered as
+    // preloaded chunks for any input stream.
+    return true;
+  }
+
+  // Cache only chunks that we really need to keep.
+  return MustKeepCachedChunk(aIndex);
+#endif
+}
+
+bool
+CacheFile::MustKeepCachedChunk(uint32_t aIndex)
+{
+  AssertOwnsLock();
+
+  // We must keep the chunk when this is memory only entry or we don't have
+  // a handle yet.
+  if (mMemoryOnly || mOpeningFile) {
+    return true;
+  }
+
+  if (mPreloadChunkCount == 0) {
+    // Preloading of chunks is disabled
+    return false;
+  }
+
+  // Check whether this chunk should be considered as preloaded chunk for any
+  // existing input stream.
+
+  // maxPos is the position of the last byte in the given chunk
+  int64_t maxPos = static_cast<int64_t>(aIndex + 1) * kChunkSize - 1;
+
+  // minPos is the position of the first byte in a chunk that precedes the given
+  // chunk by mPreloadChunkCount chunks
+  int64_t minPos;
+  if (mPreloadChunkCount >= aIndex) {
+    minPos = 0;
+  } else {
+    minPos = static_cast<int64_t>(aIndex - mPreloadChunkCount) * kChunkSize;
+  }
+
+  for (uint32_t i = 0; i < mInputs.Length(); ++i) {
+    int64_t inputPos = mInputs[i]->GetPosition();
+    if (inputPos >= minPos && inputPos <= maxPos) {
+      return true;
+    }
+  }
+
+  return false;
+}
+
 nsresult
-CacheFile::RemoveChunk(CacheFileChunk *aChunk)
+CacheFile::DeactivateChunk(CacheFileChunk *aChunk)
 {
   nsresult rv;
 
@@ -1144,15 +1350,17 @@ CacheFile::RemoveChunk(CacheFileChunk *aChunk)
   {
     CacheFileAutoLock lock(this);
 
-    LOG(("CacheFile::RemoveChunk() [this=%p, chunk=%p, idx=%d]",
+    LOG(("CacheFile::DeactivateChunk() [this=%p, chunk=%p, idx=%u]",
          this, aChunk, aChunk->Index()));
 
     MOZ_ASSERT(mReady);
-    MOZ_ASSERT(mHandle || mMemoryOnly || mOpeningFile);
+    MOZ_ASSERT((mHandle && !mMemoryOnly && !mOpeningFile) ||
+               (!mHandle && mMemoryOnly && !mOpeningFile) ||
+               (!mHandle && !mMemoryOnly && mOpeningFile));
 
     if (aChunk->mRefCnt != 2) {
-      LOG(("CacheFile::RemoveChunk() - Chunk is still used [this=%p, chunk=%p, "
-           "refcnt=%d]", this, aChunk, aChunk->mRefCnt.get()));
+      LOG(("CacheFile::DeactivateChunk() - Chunk is still used [this=%p, "
+           "chunk=%p, refcnt=%d]", this, aChunk, aChunk->mRefCnt.get()));
 
       // somebody got the reference before the lock was acquired
       return NS_OK;
@@ -1172,37 +1380,51 @@ CacheFile::RemoveChunk(CacheFileChunk *aChunk)
     }
 #endif
 
+    if (NS_FAILED(chunk->GetStatus())) {
+      SetError(chunk->GetStatus());
+    }
+
+    if (NS_FAILED(mStatus)) {
+      // Don't write any chunk to disk since this entry will be doomed
+      LOG(("CacheFile::DeactivateChunk() - Releasing chunk because of status "
+           "[this=%p, chunk=%p, mStatus=0x%08x]", this, chunk.get(), mStatus));
+
+      RemoveChunkInternal(chunk, false);
+      return mStatus;
+    }
+
     if (chunk->IsDirty() && !mMemoryOnly && !mOpeningFile) {
-      LOG(("CacheFile::RemoveChunk() - Writing dirty chunk to the disk "
+      LOG(("CacheFile::DeactivateChunk() - Writing dirty chunk to the disk "
            "[this=%p]", this));
 
       mDataIsDirty = true;
 
       rv = chunk->Write(mHandle, this);
       if (NS_FAILED(rv)) {
-        // TODO ??? doom entry
-        // TODO mark this chunk as memory only, since it wasn't written to disk
-        // and therefore cannot be released from memory
-        // LOG
-      }
-      else {
-        // Chunk will be removed in OnChunkWritten if it is still unused
+        LOG(("CacheFile::DeactivateChunk() - CacheFileChunk::Write() failed "
+             "synchronously. Removing it. [this=%p, chunk=%p, rv=0x%08x]",
+             this, chunk.get(), rv));
 
-        // chunk needs to be released under the lock to be able to rely on
-        // CacheFileChunk::mRefCnt in CacheFile::OnChunkWritten()
-        chunk = nullptr;
-        return NS_OK;
+        RemoveChunkInternal(chunk, false);
+
+        SetError(rv);
+        return rv;
       }
+
+      // Chunk will be removed in OnChunkWritten if it is still unused
+
+      // chunk needs to be released under the lock to be able to rely on
+      // CacheFileChunk::mRefCnt in CacheFile::OnChunkWritten()
+      chunk = nullptr;
+      return NS_OK;
     }
 
-    LOG(("CacheFile::RemoveChunk() - Caching unused chunk [this=%p, chunk=%p]",
-         this, chunk.get()));
+    bool keepChunk = ShouldCacheChunk(aChunk->Index());
+    LOG(("CacheFile::DeactivateChunk() - %s unused chunk [this=%p, chunk=%p]",
+         keepChunk ? "Caching" : "Releasing", this, chunk.get()));
+
+    RemoveChunkInternal(chunk, keepChunk);
 
-    chunk->mRemovingChunk = true;
-    ReleaseOutsideLock(static_cast<CacheFileChunkListener *>(
-                         chunk->mFile.forget().take()));
-    mCachedChunks.Put(chunk->Index(), chunk);
-    mChunks.Remove(chunk->Index());
     if (!mMemoryOnly)
       WriteMetadataIfNeededLocked();
   }
@@ -1210,12 +1432,111 @@ CacheFile::RemoveChunk(CacheFileChunk *aChunk)
   return NS_OK;
 }
 
+void
+CacheFile::RemoveChunkInternal(CacheFileChunk *aChunk, bool aCacheChunk)
+{
+  AssertOwnsLock();
+
+  aChunk->mActiveChunk = false;
+  ReleaseOutsideLock(static_cast<CacheFileChunkListener *>(
+                       aChunk->mFile.forget().take()));
+
+  if (aCacheChunk) {
+    mCachedChunks.Put(aChunk->Index(), aChunk);
+  }
+
+  mChunks.Remove(aChunk->Index());
+}
+
+int64_t
+CacheFile::BytesFromChunk(uint32_t aIndex)
+{
+  AssertOwnsLock();
+
+  if (!mDataSize)
+    return 0;
+
+  // Index of the last existing chunk.
+  uint32_t lastChunk = (mDataSize - 1) / kChunkSize;
+  if (aIndex > lastChunk)
+    return 0;
+
+  // We can use only preloaded chunks for the given stream to calculate
+  // available bytes if this is an entry stored on disk, since only those
+  // chunks are guaranteed not to be released.
+  uint32_t maxPreloadedChunk;
+  if (mMemoryOnly) {
+    maxPreloadedChunk = lastChunk;
+  } else {
+    maxPreloadedChunk = std::min(aIndex + mPreloadChunkCount, lastChunk);
+  }
+
+  uint32_t i;
+  for (i = aIndex; i <= maxPreloadedChunk; ++i) {
+    CacheFileChunk * chunk;
+
+    chunk = mChunks.GetWeak(i);
+    if (chunk) {
+      MOZ_ASSERT(i == lastChunk || chunk->mDataSize == kChunkSize);
+      if (chunk->IsReady()) {
+        continue;
+      }
+
+      // don't search this chunk in cached
+      break;
+    }
+
+    chunk = mCachedChunks.GetWeak(i);
+    if (chunk) {
+      MOZ_ASSERT(i == lastChunk || chunk->mDataSize == kChunkSize);
+      continue;
+    }
+
+    break;
+  }
+
+  // theoretic bytes in advance
+  int64_t advance = int64_t(i - aIndex) * kChunkSize;
+  // real bytes till the end of the file
+  int64_t tail = mDataSize - (aIndex * kChunkSize);
+
+  return std::min(advance, tail);
+}
+
+static uint32_t
+StatusToTelemetryEnum(nsresult aStatus)
+{
+  if (NS_SUCCEEDED(aStatus)) {
+    return 0;
+  }
+
+  switch (aStatus) {
+    case NS_BASE_STREAM_CLOSED:
+      return 0; // Log this as a success
+    case NS_ERROR_OUT_OF_MEMORY:
+      return 2;
+    case NS_ERROR_FILE_DISK_FULL:
+      return 3;
+    case NS_ERROR_FILE_CORRUPTED:
+      return 4;
+    case NS_ERROR_FILE_NOT_FOUND:
+      return 5;
+    case NS_BINDING_ABORTED:
+      return 6;
+    default:
+      return 1; // other error
+  }
+
+  NS_NOTREACHED("We should never get here");
+}
+
 nsresult
-CacheFile::RemoveInput(CacheFileInputStream *aInput)
+CacheFile::RemoveInput(CacheFileInputStream *aInput, nsresult aStatus)
 {
   CacheFileAutoLock lock(this);
 
-  LOG(("CacheFile::RemoveInput() [this=%p, input=%p]", this, aInput));
+  LOG(("CacheFile::RemoveInput() [this=%p, input=%p, status=0x%08x]", this,
+       aInput, aStatus));
 
   DebugOnly<bool> found;
   found = mInputs.RemoveElement(aInput);
@@ -1226,15 +1547,23 @@ CacheFile::RemoveInput(CacheFileInputStream *aInput)
   if (!mMemoryOnly)
     WriteMetadataIfNeededLocked();
 
+  // If the input didn't read all data, there might be left some preloaded
+  // chunks that won't be used anymore.
+  mCachedChunks.Enumerate(&CacheFile::CleanUpCachedChunks, this);
+
+  Telemetry::Accumulate(Telemetry::NETWORK_CACHE_V2_INPUT_STREAM_STATUS,
+                        StatusToTelemetryEnum(aStatus));
+
   return NS_OK;
 }
 
 nsresult
-CacheFile::RemoveOutput(CacheFileOutputStream *aOutput)
+CacheFile::RemoveOutput(CacheFileOutputStream *aOutput, nsresult aStatus)
 {
   AssertOwnsLock();
 
-  LOG(("CacheFile::RemoveOutput() [this=%p, output=%p]", this, aOutput));
+  LOG(("CacheFile::RemoveOutput() [this=%p, output=%p, status=0x%08x]", this,
+       aOutput, aStatus));
 
   if (mOutput != aOutput) {
     LOG(("CacheFile::RemoveOutput() - This output was already removed, ignoring"
@@ -1250,9 +1579,19 @@ CacheFile::RemoveOutput(CacheFileOutputStream *aOutput)
   if (!mMemoryOnly)
     WriteMetadataIfNeededLocked();
 
+  // Make sure the CacheFile status is set to a failure when the output stream
+  // is closed with a fatal error.  This way we propagate correctly and w/o any
+  // windows the failure state of this entry to end consumers.
+  if (NS_SUCCEEDED(mStatus) && NS_FAILED(aStatus) && aStatus != NS_BASE_STREAM_CLOSED) {
+    mStatus = aStatus;
+  }
+
   // Notify close listener as the last action
   aOutput->NotifyCloseListener();
 
+  Telemetry::Accumulate(Telemetry::NETWORK_CACHE_V2_OUTPUT_STREAM_STATUS,
+                        StatusToTelemetryEnum(aStatus));
+
   return NS_OK;
 }
 
@@ -1264,7 +1603,7 @@ CacheFile::NotifyChunkListener(CacheFileChunkListener *aCallback,
                                CacheFileChunk *aChunk)
 {
   LOG(("CacheFile::NotifyChunkListener() [this=%p, listener=%p, target=%p, "
-       "rv=0x%08x, idx=%d, chunk=%p]", this, aCallback, aTarget, aResult,
+       "rv=0x%08x, idx=%u, chunk=%p]", this, aCallback, aTarget, aResult,
        aChunkIdx, aChunk));
 
   nsresult rv;
@@ -1283,7 +1622,7 @@ nsresult
 CacheFile::QueueChunkListener(uint32_t aIndex,
                               CacheFileChunkListener *aCallback)
 {
-  LOG(("CacheFile::QueueChunkListener() [this=%p, idx=%d, listener=%p]",
+  LOG(("CacheFile::QueueChunkListener() [this=%p, idx=%u, listener=%p]",
        this, aIndex, aCallback));
 
   AssertOwnsLock();
@@ -1291,7 +1630,12 @@ CacheFile::QueueChunkListener(uint32_t aIndex,
   MOZ_ASSERT(aCallback);
 
   ChunkListenerItem *item = new ChunkListenerItem();
-  item->mTarget = NS_GetCurrentThread();
+  item->mTarget = CacheFileIOManager::IOTarget();
+  if (!item->mTarget) {
+    LOG(("CacheFile::QueueChunkListener() - Cannot get Cache I/O thread! Using "
+         "main thread for callback."));
+    item->mTarget = do_GetMainThread();
+  }
   item->mCallback = aCallback;
 
   ChunkListeners *listeners;
@@ -1308,7 +1652,7 @@ nsresult
 CacheFile::NotifyChunkListeners(uint32_t aIndex, nsresult aResult,
                                 CacheFileChunk *aChunk)
 {
-  LOG(("CacheFile::NotifyChunkListeners() [this=%p, idx=%d, rv=0x%08x, "
+  LOG(("CacheFile::NotifyChunkListeners() [this=%p, idx=%u, rv=0x%08x, "
        "chunk=%p]", this, aIndex, aResult, aChunk));
 
   AssertOwnsLock();
@@ -1372,14 +1716,35 @@ CacheFile::DataSize(int64_t* aSize)
 bool
 CacheFile::IsDoomed()
 {
-  CacheFileAutoLock lock(this);
-
   if (!mHandle)
     return false;
 
   return mHandle->IsDoomed();
 }
 
+bool
+CacheFile::IsWriteInProgress()
+{
+  // Returns true when there is a potentially unfinished write operation.
+  // Not using lock for performance reasons.  mMetadata is never released
+  // during life time of CacheFile.
+
+  bool result = false;
+
+  if (!mMemoryOnly) {
+    result = mDataIsDirty ||
+             (mMetadata && mMetadata->IsDirty()) ||
+             mWritingMetadata;
+  }
+
+  result = result ||
+           mOpeningFile ||
+           mOutput ||
+           mChunks.Count();
+
+  return result;
+}
+
 bool
 CacheFile::IsDirty()
 {
@@ -1415,12 +1780,6 @@ CacheFile::WriteMetadataIfNeededLocked(bool aFireAndForget)
     return;
   }
 
-  if (!aFireAndForget) {
-    // if aFireAndForget is set, we are called from dtor. Write
-    // scheduler hard-refers CacheFile otherwise, so we cannot be here.
-    CacheFileIOManager::UnscheduleMetadataWrite(this);
-  }
-
   if (NS_FAILED(mStatus))
     return;
 
@@ -1428,6 +1787,12 @@ CacheFile::WriteMetadataIfNeededLocked(bool aFireAndForget)
       mWritingMetadata || mOpeningFile)
     return;
 
+  if (!aFireAndForget) {
+    // if aFireAndForget is set, we are called from dtor. Write
+    // scheduler hard-refers CacheFile otherwise, so we cannot be here.
+    CacheFileIOManager::UnscheduleMetadataWrite(this);
+  }
+
   LOG(("CacheFile::WriteMetadataIfNeededLocked() - Writing metadata [this=%p]",
        this));
 
@@ -1436,17 +1801,19 @@ CacheFile::WriteMetadataIfNeededLocked(bool aFireAndForget)
     mWritingMetadata = true;
     mDataIsDirty = false;
   } else {
-    LOG(("CacheFile::WriteMetadataIfNeededLocked() - Writing synchronously failed "
-         "[this=%p]", this));
+    LOG(("CacheFile::WriteMetadataIfNeededLocked() - Writing synchronously "
+         "failed [this=%p]", this));
     // TODO: close streams with error
-    if (NS_SUCCEEDED(mStatus))
-      mStatus = rv;
+    SetError(rv);
   }
 }
 
 void
 CacheFile::PostWriteTimer()
 {
+  if (mMemoryOnly)
+    return;
+
   LOG(("CacheFile::PostWriteTimer() [this=%p]", this));
 
   CacheFileIOManager::ScheduleMetadataWrite(this);
@@ -1459,12 +1826,12 @@ CacheFile::WriteAllCachedChunks(const uint32_t& aIdx,
 {
   CacheFile *file = static_cast<CacheFile*>(aClosure);
 
-  LOG(("CacheFile::WriteAllCachedChunks() [this=%p, idx=%d, chunk=%p]",
+  LOG(("CacheFile::WriteAllCachedChunks() [this=%p, idx=%u, chunk=%p]",
        file, aIdx, aChunk.get()));
 
   file->mChunks.Put(aIdx, aChunk);
   aChunk->mFile = file;
-  aChunk->mRemovingChunk = false;
+  aChunk->mActiveChunk = true;
 
   MOZ_ASSERT(aChunk->IsReady());
 
@@ -1482,7 +1849,7 @@ CacheFile::FailListenersIfNonExistentChunk(
 {
   CacheFile *file = static_cast<CacheFile*>(aClosure);
 
-  LOG(("CacheFile::FailListenersIfNonExistentChunk() [this=%p, idx=%d]",
+  LOG(("CacheFile::FailListenersIfNonExistentChunk() [this=%p, idx=%u]",
        file, aIdx));
 
   nsRefPtr<CacheFileChunk> chunk;
@@ -1512,7 +1879,7 @@ CacheFile::FailUpdateListeners(
   CacheFile *file = static_cast<CacheFile*>(aClosure);
 #endif
 
-  LOG(("CacheFile::FailUpdateListeners() [this=%p, idx=%d]",
+  LOG(("CacheFile::FailUpdateListeners() [this=%p, idx=%u]",
        file, aIdx));
 
   if (aChunk->IsReady()) {
@@ -1522,6 +1889,25 @@ CacheFile::FailUpdateListeners(
   return PL_DHASH_NEXT;
 }
 
+PLDHashOperator
+CacheFile::CleanUpCachedChunks(const uint32_t& aIdx,
+                               nsRefPtr<CacheFileChunk>& aChunk,
+                               void* aClosure)
+{
+  CacheFile *file = static_cast<CacheFile*>(aClosure);
+
+  LOG(("CacheFile::CleanUpCachedChunks() [this=%p, idx=%u, chunk=%p]", file,
+       aIdx, aChunk.get()));
+
+  if (file->MustKeepCachedChunk(aIdx)) {
+    LOG(("CacheFile::CleanUpCachedChunks() - Keeping chunk"));
+    return PL_DHASH_NEXT;
+  }
+
+  LOG(("CacheFile::CleanUpCachedChunks() - Removing chunk"));
+  return PL_DHASH_REMOVE;
+}
+
 nsresult
 CacheFile::PadChunkWithZeroes(uint32_t aChunkIdx)
 {
@@ -1533,7 +1919,7 @@ CacheFile::PadChunkWithZeroes(uint32_t aChunkIdx)
 
   nsresult rv;
   nsRefPtr<CacheFileChunk> chunk;
-  rv = GetChunkLocked(aChunkIdx, true, nullptr, getter_AddRefs(chunk));
+  rv = GetChunkLocked(aChunkIdx, WRITER, nullptr, getter_AddRefs(chunk));
   NS_ENSURE_SUCCESS(rv, rv);
 
   LOG(("CacheFile::PadChunkWithZeroes() - Zeroing hole in chunk %d, range %d-%d"
@@ -1550,6 +1936,19 @@ CacheFile::PadChunkWithZeroes(uint32_t aChunkIdx)
   return NS_OK;
 }
 
+void
+CacheFile::SetError(nsresult aStatus)
+{
+  AssertOwnsLock();
+
+  if (NS_SUCCEEDED(mStatus)) {
+    mStatus = aStatus;
+    if (mHandle) {
+      CacheFileIOManager::DoomFile(mHandle, nullptr);
+    }
+  }
+}
+
 nsresult
 CacheFile::InitIndexEntry()
 {
@@ -1598,6 +1997,7 @@ CacheFile::SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const
   CacheFileAutoLock lock(const_cast<CacheFile*>(this));
 
   size_t n = 0;
+  n += mKey.SizeOfExcludingThisIfUnshared(mallocSizeOf);
   n += mChunks.SizeOfExcludingThis(CollectChunkSize, mallocSizeOf);
   n += mCachedChunks.SizeOfExcludingThis(CollectChunkSize, mallocSizeOf);
   if (mMetadata) {
