@@ -27,6 +27,7 @@
 #include "nsHttpHandler.h"
 #include "nsHttpRequestHead.h"
 #include "nsISocketTransport.h"
+#include "nsStandardURL.h"
 #include "prnetdb.h"
 
 #ifdef DEBUG
@@ -241,6 +242,28 @@ Http2Stream::WriteSegments(nsAHttpSegmentWriter *writer,
   return rv;
 }
 
+nsresult
+Http2Stream::MakeOriginURL(const nsACString &origin, nsRefPtr<nsStandardURL> &url)
+{
+  nsAutoCString scheme;
+  nsresult rv = net_ExtractURLScheme(origin, nullptr, nullptr, &scheme);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return MakeOriginURL(scheme, origin, url);
+}
+
+nsresult
+Http2Stream::MakeOriginURL(const nsACString &scheme, const nsACString &origin,
+                           nsRefPtr<nsStandardURL> &url)
+{
+  url = new nsStandardURL();
+  nsresult rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY,
+                          scheme.EqualsLiteral("http") ?
+                              NS_HTTP_DEFAULT_PORT :
+                              NS_HTTPS_DEFAULT_PORT,
+                          origin, nullptr, nullptr);
+  return rv;
+}
+
 void
 Http2Stream::CreatePushHashKey(const nsCString &scheme,
                                const nsCString &hostHeader,
@@ -249,9 +272,22 @@ Http2Stream::CreatePushHashKey(const nsCString &scheme,
                                nsCString &outOrigin,
                                nsCString &outKey)
 {
-  outOrigin = scheme;
-  outOrigin.AppendLiteral("://");
-  outOrigin.Append(hostHeader);
+  nsCString fullOrigin = scheme;
+  fullOrigin.AppendLiteral("://");
+  fullOrigin.Append(hostHeader);
+
+  nsRefPtr<nsStandardURL> origin;
+  nsresult rv = Http2Stream::MakeOriginURL(scheme, fullOrigin, origin);
+
+  if (NS_SUCCEEDED(rv)) {
+    rv = origin->GetAsciiSpec(outOrigin);
+    outOrigin.Trim("/", false, true, false);
+  }
+
+  if (NS_FAILED(rv)) {
+    // Fallback to plain text copy - this may end up behaving poorly
+    outOrigin = fullOrigin;
+  }
 
   outKey = outOrigin;
   outKey.AppendLiteral("/[http2.");
@@ -312,15 +348,32 @@ Http2Stream::ParseHttpRequestHeaders(const char *buf,
     // from :scheme, :authority, :path
     nsILoadGroupConnectionInfo *loadGroupCI = mTransaction->LoadGroupConnectionInfo();
     SpdyPushCache *cache = nullptr;
-    if (loadGroupCI)
+    if (loadGroupCI) {
       loadGroupCI->GetSpdyPushCache(&cache);
+    }
 
     Http2PushedStream *pushedStream = nullptr;
+
+    // If a push stream is attached to the transaction via onPush, match only with that
+    // one. This occurs when a push was made with in conjunction with a nsIHttpPushListener
+    nsHttpTransaction *trans = mTransaction->QueryHttpTransaction();
+    if (trans && (pushedStream = trans->TakePushedStream())) {
+      if (pushedStream->mSession == mSession) {
+        LOG3(("Pushed Stream match based on OnPush correlation %p", pushedStream));
+      } else {
+        LOG3(("Pushed Stream match failed due to stream mismatch %p %d %d\n", pushedStream,
+              pushedStream->mSession->Serial(), mSession->Serial()));
+        pushedStream->OnPushFailed();
+        pushedStream = nullptr;
+      }
+    }
+
     // we remove the pushedstream from the push cache so that
     // it will not be used for another GET. This does not destroy the
     // stream itself - that is done when the transactionhash is done with it.
-    if (cache)
-      pushedStream = cache->RemovePushedStreamHttp2(hashkey);
+    if (cache && !pushedStream){
+        pushedStream = cache->RemovePushedStreamHttp2(hashkey);
+    }
 
     LOG3(("Pushed Stream Lookup "
           "session=%p key=%s loadgroupci=%p cache=%p hit=%p\n",
@@ -399,6 +452,11 @@ Http2Stream::ParseHttpRequestHeaders(const char *buf,
                                             head->IsConnect(),
                                             compressedData);
 
+  int64_t clVal = mSession->Compressor()->GetParsedContentLength();
+  if (clVal != -1) {
+    mRequestBodyLenRemaining = clVal;
+  }
+
   // Determine whether to put the fin bit on the header frame or whether
   // to wait for a data packet to put it on.
   uint8_t firstFrameFlags =  Http2Session::kFlag_PRIORITY;
@@ -502,42 +560,6 @@ Http2Stream::ParseHttpRequestHeaders(const char *buf,
     (11 + head->RequestURI().Length() +
      mFlatHttpRequestHeaders.Length());
 
-  const char *beginBuffer = mFlatHttpRequestHeaders.BeginReading();
-  int32_t crlfIndex = mFlatHttpRequestHeaders.Find("\r\n");
-  while (true) {
-    int32_t startIndex = crlfIndex + 2;
-
-    crlfIndex = mFlatHttpRequestHeaders.Find("\r\n", false, startIndex);
-    if (crlfIndex == -1)
-      break;
-
-    int32_t colonIndex = mFlatHttpRequestHeaders.Find(":", false, startIndex,
-                                                      crlfIndex - startIndex);
-    if (colonIndex == -1)
-      break;
-
-    nsDependentCSubstring name = Substring(beginBuffer + startIndex,
-                                           beginBuffer + colonIndex);
-    // all header names are lower case in spdy
-    ToLowerCase(name);
-
-    if (name.EqualsLiteral("content-length")) {
-      nsCString *val = new nsCString();
-      int32_t valueIndex = colonIndex + 1;
-      while (valueIndex < crlfIndex && beginBuffer[valueIndex] == ' ')
-        ++valueIndex;
-
-      nsDependentCSubstring v = Substring(beginBuffer + valueIndex,
-                                          beginBuffer + crlfIndex);
-      val->Append(v);
-
-      int64_t len;
-      if (nsHttp::ParseInt64(val->get(), nullptr, &len))
-        mRequestBodyLenRemaining = len;
-      break;
-    }
-  }
-
   mFlatHttpRequestHeaders.Truncate();
   Telemetry::Accumulate(Telemetry::SPDY_SYN_RATIO, ratio);
   return NS_OK;
@@ -569,6 +591,17 @@ Http2Stream::AdjustInitialWindow()
       return;
   }
 
+  if (stream->mState == RESERVED_BY_REMOTE) {
+    // h2-14 prevents sending a window update in this state
+    return;
+  }
+
+  MOZ_ASSERT(mClientReceiveWindow <= ASpdySession::kInitialRwin);
+  uint32_t bump = ASpdySession::kInitialRwin - mClientReceiveWindow;
+  if (!bump) { // nothing to do
+    return;
+  }
+
   uint8_t *packet = mTxInlineFrame.get() + mTxInlineFrameUsed;
   EnsureBuffer(mTxInlineFrame, mTxInlineFrameUsed + Http2Session::kFrameHeaderBytes + 4,
                mTxInlineFrameUsed, mTxInlineFrameSize);
@@ -578,8 +611,6 @@ Http2Stream::AdjustInitialWindow()
                               Http2Session::FRAME_TYPE_WINDOW_UPDATE,
                               0, stream->mStreamID);
 
-  MOZ_ASSERT(mClientReceiveWindow <= ASpdySession::kInitialRwin);
-  uint32_t bump = ASpdySession::kInitialRwin - mClientReceiveWindow;
   mClientReceiveWindow += bump;
   bump = PR_htonl(bump);
   memcpy(packet + Http2Session::kFrameHeaderBytes, &bump, 4);
@@ -835,7 +866,7 @@ Http2Stream::GenerateDataFrameHeader(uint32_t dataLength, bool lastFrame)
   mTxStreamFrameSize = dataLength;
 }
 
-// ConvertHeaders is used to convert the response headers
+// ConvertResponseHeaders is used to convert the response headers
 // into HTTP/1 format and report some telemetry
 nsresult
 Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
@@ -851,14 +882,14 @@ Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
                                     aHeadersIn.Length(),
                                     aHeadersOut, false);
   if (NS_FAILED(rv)) {
-    LOG3(("Http2Stream::ConvertHeaders %p decode Error\n", this));
+    LOG3(("Http2Stream::ConvertResponseHeaders %p decode Error\n", this));
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
   nsAutoCString statusString;
   decompressor->GetStatus(statusString);
   if (statusString.IsEmpty()) {
-    LOG3(("Http2Stream::ConvertHeaders %p Error - no status\n", this));
+    LOG3(("Http2Stream::ConvertResponseHeaders %p Error - no status\n", this));
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
@@ -873,7 +904,7 @@ Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
 
   if (httpResponseCode == 101) {
     // 8.1.1 of h2 disallows 101.. throw PROTOCOL_ERROR on stream
-    LOG3(("Http2Stream::ConvertHeaders %p Error - status == 101\n", this));
+    LOG3(("Http2Stream::ConvertResponseHeaders %p Error - status == 101\n", this));
     return NS_ERROR_ILLEGAL_VALUE;
   }
 
@@ -901,7 +932,7 @@ Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
   return NS_OK;
 }
 
-// ConvertHeaders is used to convert the response headers
+// ConvertPushHeaders is used to convert the pushed request headers
 // into HTTP/1 format and report some telemetry
 nsresult
 Http2Stream::ConvertPushHeaders(Http2Decompressor *decompressor,
@@ -909,6 +940,7 @@ Http2Stream::ConvertPushHeaders(Http2Decompressor *decompressor,
                                 nsACString &aHeadersOut)
 {
   aHeadersOut.Truncate();
+  aHeadersOut.SetCapacity(aHeadersIn.Length() + 512);
   nsresult rv =
     decompressor->DecodeHeaderBlock(reinterpret_cast<const uint8_t *>(aHeadersIn.BeginReading()),
                                     aHeadersIn.Length(),
@@ -955,6 +987,14 @@ Http2Stream::SetAllHeadersReceived()
     return;
   }
 
+  if (mState == RESERVED_BY_REMOTE) {
+    // pushed streams needs to wait until headers have
+    // arrived to open up their window
+    LOG3(("Http2Stream::SetAllHeadersReceived %p state OPEN from reserved\n", this));
+    mState = OPEN;
+    AdjustInitialWindow();
+  }
+
   mAllHeadersReceived = 1;
   if (mIsTunnel) {
     MapStreamToHttpConnection();
