@@ -25,7 +25,7 @@ from mozbuild.util import (
     StrictOrderingOnAppendList,
 )
 import mozpack.path as mozpath
-from .sandbox_symbols import FinalTargetValue
+from .context import FinalTargetValue
 
 
 class TreeMetadata(object):
@@ -49,42 +49,46 @@ class ReaderSummary(TreeMetadata):
         self.total_emitter_execution_time = total_emitter_execution_time
 
 
-class SandboxDerived(TreeMetadata):
-    """Build object derived from a single MozbuildSandbox instance.
+class ContextDerived(TreeMetadata):
+    """Build object derived from a single Context instance.
 
-    It holds fields common to all sandboxes. This class is likely never
-    instantiated directly but is instead derived from.
+    It holds fields common to all context derived classes. This class is likely
+    never instantiated directly but is instead derived from.
     """
 
     __slots__ = (
         'objdir',
         'relativedir',
-        'sandbox_all_paths',
-        'sandbox_path',
+        'context_all_paths',
+        'context_path',
         'srcdir',
         'topobjdir',
         'topsrcdir',
     )
 
-    def __init__(self, sandbox):
+    def __init__(self, context):
         TreeMetadata.__init__(self)
 
-        # Capture the files that were evaluated to build this sandbox.
-        self.sandbox_main_path = sandbox.main_path
-        self.sandbox_all_paths = sandbox.all_paths
+        # Capture the files that were evaluated to fill this context.
+        self.context_main_path = context.main_path
+        self.context_all_paths = context.all_paths
 
         # Basic directory state.
-        self.topsrcdir = sandbox['TOPSRCDIR']
-        self.topobjdir = sandbox['TOPOBJDIR']
+        self.topsrcdir = context.config.topsrcdir
+        self.topobjdir = context.config.topobjdir
 
-        self.relativedir = sandbox['RELATIVEDIR']
-        self.srcdir = sandbox['SRCDIR']
-        self.objdir = sandbox['OBJDIR']
+        self.relativedir = context.relsrcdir
+        self.srcdir = context.srcdir
+        self.objdir = context.objdir
 
-        self.config = sandbox.config
+        self.config = context.config
+
+    @property
+    def relobjdir(self):
+        return mozpath.relpath(self.objdir, self.topobjdir)
 
 
-class DirectoryTraversal(SandboxDerived):
+class DirectoryTraversal(ContextDerived):
     """Describes how directory traversal for building should work.
 
     This build object is likely only of interest to the recursive make backend.
@@ -98,27 +102,17 @@ class DirectoryTraversal(SandboxDerived):
     """
     __slots__ = (
         'dirs',
-        'parallel_dirs',
-        'tool_dirs',
         'test_dirs',
-        'test_tool_dirs',
-        'tier_dirs',
-        'tier_static_dirs',
     )
 
-    def __init__(self, sandbox):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context):
+        ContextDerived.__init__(self, context)
 
         self.dirs = []
-        self.parallel_dirs = []
-        self.tool_dirs = []
         self.test_dirs = []
-        self.test_tool_dirs = []
-        self.tier_dirs = OrderedDict()
-        self.tier_static_dirs = OrderedDict()
 
 
-class BaseConfigSubstitution(SandboxDerived):
+class BaseConfigSubstitution(ContextDerived):
     """Base class describing autogenerated files as part of config.status."""
 
     __slots__ = (
@@ -127,8 +121,8 @@ class BaseConfigSubstitution(SandboxDerived):
         'relpath',
     )
 
-    def __init__(self, sandbox):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context):
+        ContextDerived.__init__(self, context)
 
         self.input_path = None
         self.output_path = None
@@ -143,7 +137,7 @@ class HeaderFileSubstitution(BaseConfigSubstitution):
     """Describes a header file that will be generated using substitutions."""
 
 
-class VariablePassthru(SandboxDerived):
+class VariablePassthru(ContextDerived):
     """A dict of variables to pass through to backend.mk unaltered.
 
     The purpose of this object is to facilitate rapid transitioning of
@@ -154,11 +148,11 @@ class VariablePassthru(SandboxDerived):
     """
     __slots__ = ('variables')
 
-    def __init__(self, sandbox):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context):
+        ContextDerived.__init__(self, context)
         self.variables = {}
 
-class XPIDLFile(SandboxDerived):
+class XPIDLFile(ContextDerived):
     """Describes an XPIDL file to be compiled."""
 
     __slots__ = (
@@ -166,20 +160,20 @@ class XPIDLFile(SandboxDerived):
         'source_path',
     )
 
-    def __init__(self, sandbox, source, module):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, source, module):
+        ContextDerived.__init__(self, context)
 
         self.source_path = source
         self.basename = mozpath.basename(source)
         self.module = module
 
-class Defines(SandboxDerived):
-    """Sandbox container object for DEFINES, which is an OrderedDict.
+class Defines(ContextDerived):
+    """Context derived container object for DEFINES, which is an OrderedDict.
     """
     __slots__ = ('defines')
 
-    def __init__(self, sandbox, defines):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, defines):
+        ContextDerived.__init__(self, context)
         self.defines = defines
 
     def get_defines(self):
@@ -191,88 +185,112 @@ class Defines(SandboxDerived):
             else:
                 yield('-D%s=%s' % (define, shell_quote(value)))
 
-class Exports(SandboxDerived):
-    """Sandbox container object for EXPORTS, which is a HierarchicalStringList.
+    def update(self, more_defines):
+        if isinstance(more_defines, Defines):
+            self.defines.update(more_defines.defines)
+        else:
+            self.defines.update(more_defines)
 
-    We need an object derived from SandboxDerived for use in the backend, so
+class Exports(ContextDerived):
+    """Context derived container object for EXPORTS, which is a
+    HierarchicalStringList.
+
+    We need an object derived from ContextDerived for use in the backend, so
     this object fills that role. It just has a reference to the underlying
     HierarchicalStringList, which is created when parsing EXPORTS.
     """
     __slots__ = ('exports', 'dist_install')
 
-    def __init__(self, sandbox, exports, dist_install=True):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, exports, dist_install=True):
+        ContextDerived.__init__(self, context)
         self.exports = exports
         self.dist_install = dist_install
 
-class Resources(SandboxDerived):
-    """Sandbox container object for RESOURCE_FILES, which is a HierarchicalStringList,
-    with an extra ``.preprocess`` property on each entry.
+class TestHarnessFiles(ContextDerived):
+    """Sandbox container object for TEST_HARNESS_FILES,
+    which is a HierarchicalStringList.
+
+    We need an object derived from ContextDerived for use in the backend, so
+    this object fills that role. It just has a reference to the underlying
+    HierarchicalStringList, which is created when parsing TEST_HARNESS_FILES.
+    """
+    __slots__ = ('srcdir_files', 'srcdir_pattern_files', 'objdir_files')
+
+    def __init__(self, context, srcdir_files, srcdir_pattern_files, objdir_files):
+        ContextDerived.__init__(self, context)
+        self.srcdir_files = srcdir_files
+        self.srcdir_pattern_files = srcdir_pattern_files
+        self.objdir_files = objdir_files
+
+class Resources(ContextDerived):
+    """Context derived container object for RESOURCE_FILES, which is a
+    HierarchicalStringList, with an extra ``.preprocess`` property on each
+    entry.
 
     The local defines plus anything in ACDEFINES are stored in ``defines`` as a
     dictionary, for any files that need preprocessing.
     """
     __slots__ = ('resources', 'defines')
 
-    def __init__(self, sandbox, resources, defines=None):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, resources, defines=None):
+        ContextDerived.__init__(self, context)
         self.resources = resources
         defs = {}
-        defs.update(sandbox.config.defines)
+        defs.update(context.config.defines)
         if defines:
             defs.update(defines)
         self.defines = defs
 
 
-class IPDLFile(SandboxDerived):
+class IPDLFile(ContextDerived):
     """Describes an individual .ipdl source file."""
 
     __slots__ = (
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class WebIDLFile(SandboxDerived):
+class WebIDLFile(ContextDerived):
     """Describes an individual .webidl source file."""
 
     __slots__ = (
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class GeneratedEventWebIDLFile(SandboxDerived):
+class GeneratedEventWebIDLFile(ContextDerived):
     """Describes an individual .webidl source file."""
 
     __slots__ = (
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class TestWebIDLFile(SandboxDerived):
+class TestWebIDLFile(ContextDerived):
     """Describes an individual test-only .webidl source file."""
 
     __slots__ = (
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class PreprocessedTestWebIDLFile(SandboxDerived):
+class PreprocessedTestWebIDLFile(ContextDerived):
     """Describes an individual test-only .webidl source file that requires
     preprocessing."""
 
@@ -280,24 +298,24 @@ class PreprocessedTestWebIDLFile(SandboxDerived):
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class PreprocessedWebIDLFile(SandboxDerived):
+class PreprocessedWebIDLFile(ContextDerived):
     """Describes an individual .webidl source file that requires preprocessing."""
 
     __slots__ = (
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
-class GeneratedWebIDLFile(SandboxDerived):
+class GeneratedWebIDLFile(ContextDerived):
     """Describes an individual .webidl source file that is generated from
     build rules."""
 
@@ -305,27 +323,70 @@ class GeneratedWebIDLFile(SandboxDerived):
         'basename',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.basename = path
 
 
-class ExampleWebIDLInterface(SandboxDerived):
+class ExampleWebIDLInterface(ContextDerived):
     """An individual WebIDL interface to generate."""
 
     __slots__ = (
         'name',
     )
 
-    def __init__(self, sandbox, name):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, name):
+        ContextDerived.__init__(self, context)
 
         self.name = name
 
 
-class BaseProgram(SandboxDerived):
-    """Sandbox container object for programs, which is a unicode string.
+class LinkageWrongKindError(Exception):
+    """Error thrown when trying to link objects of the wrong kind"""
+
+
+class Linkable(ContextDerived):
+    """Generic context derived container object for programs and libraries"""
+    __slots__ = (
+        'defines',
+        'linked_libraries',
+        'linked_system_libs',
+    )
+
+    def __init__(self, context):
+        ContextDerived.__init__(self, context)
+        self.linked_libraries = []
+        self.linked_system_libs = []
+        self.defines = Defines(context, {})
+
+    def link_library(self, obj):
+        assert isinstance(obj, BaseLibrary)
+        if isinstance(obj, SharedLibrary) and obj.variant == obj.COMPONENT:
+            raise LinkageWrongKindError(
+                'Linkable.link_library() does not take components.')
+        if obj.KIND != self.KIND:
+            raise LinkageWrongKindError('%s != %s' % (obj.KIND, self.KIND))
+        self.linked_libraries.append(obj)
+        obj.refs.append(self)
+
+    def link_system_library(self, lib):
+        # The '$' check is here as a special temporary rule, allowing the
+        # inherited use of make variables, most notably in TK_LIBS.
+        if not lib.startswith('$') and not lib.startswith('-'):
+            if self.config.substs.get('GNU_CC'):
+                lib = '-l%s' % lib
+            else:
+                lib = '%s%s%s' % (
+                    self.config.import_prefix,
+                    lib,
+                    self.config.import_suffix,
+                )
+        self.linked_system_libs.append(lib)
+
+class BaseProgram(Linkable):
+    """Context derived container object for programs, which is a unicode
+    string.
 
     This class handles automatically appending a binary suffix to the program
     name.
@@ -335,56 +396,152 @@ class BaseProgram(SandboxDerived):
     """
     __slots__ = ('program')
 
-    def __init__(self, sandbox, program, bin_suffix):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, program, is_unit_test=False):
+        Linkable.__init__(self, context)
 
-        bin_suffix = bin_suffix or ''
+        bin_suffix = context.config.substs.get(self.SUFFIX_VAR, '')
         if not program.endswith(bin_suffix):
             program += bin_suffix
         self.program = program
+        self.is_unit_test = is_unit_test
 
 
 class Program(BaseProgram):
-    """Sandbox container object for PROGRAM"""
+    """Context derived container object for PROGRAM"""
+    SUFFIX_VAR = 'BIN_SUFFIX'
+    KIND = 'target'
 
 
 class HostProgram(BaseProgram):
-    """Sandbox container object for HOST_PROGRAM"""
+    """Context derived container object for HOST_PROGRAM"""
+    SUFFIX_VAR = 'HOST_BIN_SUFFIX'
+    KIND = 'host'
 
 
 class SimpleProgram(BaseProgram):
-    """Sandbox container object for each program in SIMPLE_PROGRAMS"""
+    """Context derived container object for each program in SIMPLE_PROGRAMS"""
+    SUFFIX_VAR = 'BIN_SUFFIX'
+    KIND = 'target'
 
 
 class HostSimpleProgram(BaseProgram):
-    """Sandbox container object for each program in HOST_SIMPLE_PROGRAMS"""
+    """Context derived container object for each program in
+    HOST_SIMPLE_PROGRAMS"""
+    SUFFIX_VAR = 'HOST_BIN_SUFFIX'
+    KIND = 'host'
 
 
-class LibraryDefinition(SandboxDerived):
-    """Partial definition for a library
-
-    The static_libraries member tracks the list of relative directory and
-    library names of static libraries that are meant to be linked into
-    the library defined by an instance of this class.
-    """
+class BaseLibrary(Linkable):
+    """Generic context derived container object for libraries."""
     __slots__ = (
         'basename',
-        'static_libraries',
-        'refcount',
+        'lib_name',
+        'import_name',
+        'refs',
     )
 
-    def __init__(self, sandbox, basename):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, basename):
+        Linkable.__init__(self, context)
 
+        self.basename = self.lib_name = basename
+        if self.lib_name:
+            self.lib_name = '%s%s%s' % (
+                context.config.lib_prefix,
+                self.lib_name,
+                context.config.lib_suffix
+            )
+            self.import_name = self.lib_name
+
+        self.refs = []
+
+
+class Library(BaseLibrary):
+    """Context derived container object for a library"""
+    KIND = 'target'
+    __slots__ = (
+        'is_sdk',
+    )
+
+    def __init__(self, context, basename, real_name=None, is_sdk=False):
+        BaseLibrary.__init__(self, context, real_name or basename)
         self.basename = basename
-        self.refcount = 0
-        self.static_libraries = []
-
-    def link_static_lib(self, objdir, basename):
-        self.static_libraries.append((objdir, basename))
+        self.is_sdk = is_sdk
 
 
-class TestManifest(SandboxDerived):
+class StaticLibrary(Library):
+    """Context derived container object for a static library"""
+    __slots__ = (
+        'link_into',
+    )
+
+    def __init__(self, context, basename, real_name=None, is_sdk=False,
+        link_into=None):
+        Library.__init__(self, context, basename, real_name, is_sdk)
+        self.link_into = link_into
+
+
+class SharedLibrary(Library):
+    """Context derived container object for a shared library"""
+    __slots__ = (
+        'soname',
+        'variant',
+    )
+
+    FRAMEWORK = 1
+    COMPONENT = 2
+    MAX_VARIANT = 3
+
+    def __init__(self, context, basename, real_name=None, is_sdk=False,
+            soname=None, variant=None):
+        assert(variant in range(1, self.MAX_VARIANT) or variant is None)
+        Library.__init__(self, context, basename, real_name, is_sdk)
+        self.variant = variant
+        self.lib_name = real_name or basename
+        assert self.lib_name
+
+        if variant == self.FRAMEWORK:
+            self.import_name = self.lib_name
+        else:
+            self.import_name = '%s%s%s' % (
+                context.config.import_prefix,
+                self.lib_name,
+                context.config.import_suffix,
+            )
+            self.lib_name = '%s%s%s' % (
+                context.config.dll_prefix,
+                self.lib_name,
+                context.config.dll_suffix,
+            )
+        if soname:
+            self.soname = '%s%s%s' % (
+                context.config.dll_prefix,
+                soname,
+                context.config.dll_suffix,
+            )
+        else:
+            self.soname = self.lib_name
+
+
+class ExternalLibrary(object):
+    """Empty mixin for libraries built by an external build system."""
+
+
+class ExternalStaticLibrary(StaticLibrary, ExternalLibrary):
+    """Context derived container for static libraries built by an external
+    build system."""
+
+
+class ExternalSharedLibrary(SharedLibrary, ExternalLibrary):
+    """Context derived container for shared libraries built by an external
+    build system."""
+
+
+class HostLibrary(BaseLibrary):
+    """Context derived container object for a host library"""
+    KIND = 'host'
+
+
+class TestManifest(ContextDerived):
     """Represents a manifest file containing information about tests."""
 
     __slots__ = (
@@ -431,9 +588,9 @@ class TestManifest(SandboxDerived):
         'dupe_manifest',
     )
 
-    def __init__(self, sandbox, path, manifest, flavor=None,
+    def __init__(self, context, path, manifest, flavor=None,
             install_prefix=None, relpath=None, dupe_manifest=False):
-        SandboxDerived.__init__(self, sandbox)
+        ContextDerived.__init__(self, context)
 
         self.path = path
         self.directory = mozpath.dirname(path)
@@ -449,32 +606,32 @@ class TestManifest(SandboxDerived):
         self.external_installs = set()
 
 
-class LocalInclude(SandboxDerived):
+class LocalInclude(ContextDerived):
     """Describes an individual local include path."""
 
     __slots__ = (
         'path',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.path = path
 
-class GeneratedInclude(SandboxDerived):
+class GeneratedInclude(ContextDerived):
     """Describes an individual generated include path."""
 
     __slots__ = (
         'path',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.path = path
 
 
-class PerSourceFlag(SandboxDerived):
+class PerSourceFlag(ContextDerived):
     """Describes compiler flags specified for individual source files."""
 
     __slots__ = (
@@ -482,14 +639,14 @@ class PerSourceFlag(SandboxDerived):
         'flags',
     )
 
-    def __init__(self, sandbox, file_name, flags):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, file_name, flags):
+        ContextDerived.__init__(self, context)
 
         self.file_name = file_name
         self.flags = flags
 
 
-class JARManifest(SandboxDerived):
+class JARManifest(ContextDerived):
     """Describes an individual JAR manifest file and how to process it.
 
     This class isn't very useful for optimizing backends yet because we don't
@@ -500,13 +657,13 @@ class JARManifest(SandboxDerived):
         'path',
     )
 
-    def __init__(self, sandbox, path):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, path):
+        ContextDerived.__init__(self, context)
 
         self.path = path
 
 
-class JavaScriptModules(SandboxDerived):
+class JavaScriptModules(ContextDerived):
     """Describes a JavaScript module."""
 
     __slots__ = (
@@ -514,15 +671,15 @@ class JavaScriptModules(SandboxDerived):
         'flavor',
     )
 
-    def __init__(self, sandbox, modules, flavor):
-        super(JavaScriptModules, self).__init__(sandbox)
+    def __init__(self, context, modules, flavor):
+        super(JavaScriptModules, self).__init__(context)
 
         self.modules = modules
         self.flavor = flavor
 
 
-class SandboxWrapped(SandboxDerived):
-    """Generic sandbox container object for a wrapped rich object.
+class ContextWrapped(ContextDerived):
+    """Generic context derived container object for a wrapped rich object.
 
     Use this wrapper class to shuttle a rich build system object
     completely defined in moz.build files through the tree metadata
@@ -533,8 +690,8 @@ class SandboxWrapped(SandboxDerived):
         'wrapped',
     )
 
-    def __init__(self, sandbox, wrapped):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context, wrapped):
+        ContextDerived.__init__(self, context)
 
         self.wrapped = wrapped
 
@@ -569,7 +726,7 @@ class JavaJarData(object):
         self.javac_flags = list(javac_flags)
 
 
-class InstallationTarget(SandboxDerived):
+class InstallationTarget(ContextDerived):
     """Describes the rules that affect where files get installed to."""
 
     __slots__ = (
@@ -579,13 +736,13 @@ class InstallationTarget(SandboxDerived):
         'enabled'
     )
 
-    def __init__(self, sandbox):
-        SandboxDerived.__init__(self, sandbox)
+    def __init__(self, context):
+        ContextDerived.__init__(self, context)
 
-        self.xpiname = sandbox.get('XPI_NAME', '')
-        self.subdir = sandbox.get('DIST_SUBDIR', '')
-        self.target = sandbox['FINAL_TARGET']
-        self.enabled = not sandbox.get('NO_DIST_INSTALL', False)
+        self.xpiname = context.get('XPI_NAME', '')
+        self.subdir = context.get('DIST_SUBDIR', '')
+        self.target = context['FINAL_TARGET']
+        self.enabled = not context.get('NO_DIST_INSTALL', False)
 
     def is_custom(self):
         """Returns whether or not the target is not derived from the default
@@ -596,6 +753,22 @@ class InstallationTarget(SandboxDerived):
             DIST_SUBDIR=self.subdir)) == self.target
 
 
+class FinalTargetFiles(ContextDerived):
+    """Sandbox container object for FINAL_TARGET_FILES, which is a
+    HierarchicalStringList.
+
+    We need an object derived from ContextDerived for use in the backend, so
+    this object fills that role. It just has a reference to the underlying
+    HierarchicalStringList, which is created when parsing FINAL_TARGET_FILES.
+    """
+    __slots__ = ('files', 'target')
+
+    def __init__(self, sandbox, files, target):
+        ContextDerived.__init__(self, sandbox)
+        self.files = files
+        self.target = target
+
+
 class ClassPathEntry(object):
     """Represents a classpathentry in an Android Eclipse project."""
 
