@@ -22,12 +22,15 @@ from mozbuild.frontend.data import (
     ReaderSummary,
     Resources,
     SimpleProgram,
+    StaticLibrary,
+    TestHarnessFiles,
     TestManifest,
     VariablePassthru,
 )
 from mozbuild.frontend.emitter import TreeMetadataEmitter
 from mozbuild.frontend.reader import (
     BuildReader,
+    BuildReaderError,
     SandboxValidationError,
 )
 
@@ -41,10 +44,19 @@ data_path = mozpath.join(data_path, 'data')
 
 
 class TestEmitterBasic(unittest.TestCase):
+    def setUp(self):
+        self._old_env = dict(os.environ)
+        os.environ.pop('MOZ_OBJDIR', None)
+
+    def tearDown(self):
+        os.environ.clear()
+        os.environ.update(self._old_env)
+
     def reader(self, name):
         config = MockConfig(mozpath.join(data_path, name), extra_substs=dict(
             ENABLE_TESTS='1',
             BIN_SUFFIX='.prog',
+            OS_TARGET='WINNT',
         ))
 
         return BuildReader(config)
@@ -79,50 +91,43 @@ class TestEmitterBasic(unittest.TestCase):
 
         for o in objs:
             self.assertIsInstance(o, DirectoryTraversal)
-            self.assertEqual(o.parallel_dirs, [])
-            self.assertEqual(o.tool_dirs, [])
             self.assertEqual(o.test_dirs, [])
-            self.assertEqual(o.test_tool_dirs, [])
-            self.assertEqual(len(o.tier_dirs), 0)
-            self.assertEqual(len(o.tier_static_dirs), 0)
-            self.assertTrue(os.path.isabs(o.sandbox_main_path))
-            self.assertEqual(len(o.sandbox_all_paths), 1)
+            self.assertTrue(os.path.isabs(o.context_main_path))
+            self.assertEqual(len(o.context_all_paths), 1)
 
         reldirs = [o.relativedir for o in objs]
         self.assertEqual(reldirs, ['', 'foo', 'foo/biz', 'bar'])
 
+        self.assertEqual(objs[3].affected_tiers, {'misc'})
+
         dirs = [o.dirs for o in objs]
-        self.assertEqual(dirs, [['foo', 'bar'], ['biz'], [], []])
+        self.assertEqual(dirs, [
+            [
+                mozpath.join(reader.config.topsrcdir, 'foo'),
+                mozpath.join(reader.config.topsrcdir, 'bar')
+            ], [
+                mozpath.join(reader.config.topsrcdir, 'foo', 'biz')
+            ], [], []])
 
     def test_traversal_all_vars(self):
         reader = self.reader('traversal-all-vars')
         objs = self.read_topsrcdir(reader, filter_common=False)
-        self.assertEqual(len(objs), 6)
+        self.assertEqual(len(objs), 3)
 
         for o in objs:
             self.assertIsInstance(o, DirectoryTraversal)
 
         reldirs = set([o.relativedir for o in objs])
-        self.assertEqual(reldirs, set(['', 'parallel', 'regular', 'test',
-            'test_tool', 'tool']))
+        self.assertEqual(reldirs, set(['', 'regular', 'test']))
 
         for o in objs:
             reldir = o.relativedir
 
             if reldir == '':
-                self.assertEqual(o.dirs, ['regular'])
-                self.assertEqual(o.parallel_dirs, ['parallel'])
-                self.assertEqual(o.test_dirs, ['test'])
-                self.assertEqual(o.test_tool_dirs, ['test_tool'])
-                self.assertEqual(o.tool_dirs, ['tool'])
-
-    def test_tier_simple(self):
-        reader = self.reader('traversal-tier-simple')
-        objs = self.read_topsrcdir(reader, filter_common=False)
-        self.assertEqual(len(objs), 4)
-
-        reldirs = [o.relativedir for o in objs]
-        self.assertEqual(reldirs, ['', 'foo', 'foo/biz', 'bar'])
+                self.assertEqual(o.dirs, [
+                    mozpath.join(reader.config.topsrcdir, 'regular')])
+                self.assertEqual(o.test_dirs, [
+                    mozpath.join(reader.config.topsrcdir, 'test')])
 
     def test_config_file_substitution(self):
         reader = self.reader('config-file-substitution')
@@ -146,43 +151,31 @@ class TestEmitterBasic(unittest.TestCase):
         self.assertEqual(len(objs), 1)
         self.assertIsInstance(objs[0], VariablePassthru)
 
-        wanted = dict(
-            ASFILES=['fans.asm', 'tans.s'],
-            CMMSRCS=['fans.mm', 'tans.mm'],
-            CSRCS=['fans.c', 'tans.c'],
-            CPP_UNIT_TESTS=['foo.cpp'],
-            DISABLE_STL_WRAPPING=True,
-            EXPORT_LIBRARY=True,
-            EXTRA_COMPONENTS=['fans.js', 'tans.js'],
-            EXTRA_PP_COMPONENTS=['fans.pp.js', 'tans.pp.js'],
-            EXTRA_JS_MODULES=['bar.jsm', 'foo.jsm'],
-            EXTRA_PP_JS_MODULES=['bar.pp.jsm', 'foo.pp.jsm'],
-            FAIL_ON_WARNINGS=True,
-            FORCE_SHARED_LIB=True,
-            HOST_CPPSRCS=['fans.cpp', 'tans.cpp'],
-            HOST_CSRCS=['fans.c', 'tans.c'],
-            HOST_LIBRARY_NAME='host_fans',
-            IS_COMPONENT=True,
-            LIBS=['fans.lib', 'tans.lib'],
-            LIBXUL_LIBRARY=True,
-            MSVC_ENABLE_PGO=True,
-            NO_DIST_INSTALL=True,
-            OS_LIBS=['foo.so', '-l123', 'aaa.a'],
-            SDK_LIBRARY=['fans.sdk', 'tans.sdk'],
-            SSRCS=['bans.S', 'fans.S'],
-            VISIBILITY_FLAGS='',
-            DELAYLOAD_LDFLAGS=['-DELAYLOAD:foo.dll', '-DELAYLOAD:bar.dll'],
-            USE_DELAYIMP=True,
-            RCFILE='foo.rc',
-            RESFILE='bar.res',
-            RCINCLUDE='bar.rc',
-            DEFFILE='baz.def',
-            USE_STATIC_LIBS=True,
-            MOZBUILD_CFLAGS=['-fno-exceptions', '-w'],
-            MOZBUILD_CXXFLAGS=['-fcxx-exceptions', '-include foo.h'],
-            MOZBUILD_LDFLAGS=['-framework Foo', '-x'],
-            WIN32_EXE_LDFLAGS=['-subsystem:console'],
-        )
+        wanted = {
+            'ASFILES': ['fans.asm', 'tans.s'],
+            'CMMSRCS': ['fans.mm', 'tans.mm'],
+            'CSRCS': ['fans.c', 'tans.c'],
+            'DISABLE_STL_WRAPPING': True,
+            'EXTRA_COMPONENTS': ['fans.js', 'tans.js'],
+            'EXTRA_PP_COMPONENTS': ['fans.pp.js', 'tans.pp.js'],
+            'FAIL_ON_WARNINGS': True,
+            'HOST_CPPSRCS': ['fans.cpp', 'tans.cpp'],
+            'HOST_CSRCS': ['fans.c', 'tans.c'],
+            'MSVC_ENABLE_PGO': True,
+            'NO_DIST_INSTALL': True,
+            'SSRCS': ['bans.S', 'fans.S'],
+            'VISIBILITY_FLAGS': '',
+            'RCFILE': 'foo.rc',
+            'RESFILE': 'bar.res',
+            'RCINCLUDE': 'bar.rc',
+            'DEFFILE': 'baz.def',
+            'USE_STATIC_LIBS': True,
+            'MOZBUILD_CFLAGS': ['-fno-exceptions', '-w'],
+            'MOZBUILD_CXXFLAGS': ['-fcxx-exceptions', '-include foo.h'],
+            'MOZBUILD_LDFLAGS': ['-framework Foo', '-x', '-DELAYLOAD:foo.dll',
+                                 '-DELAYLOAD:bar.dll'],
+            'WIN32_EXE_LDFLAGS': ['-subsystem:console'],
+        }
 
         variables = objs[0].variables
         maxDiff = self.maxDiff
@@ -197,40 +190,48 @@ class TestEmitterBasic(unittest.TestCase):
         self.assertEqual(len(objs), 1)
         self.assertIsInstance(objs[0], Exports)
 
-        exports = objs[0].exports
-        self.assertEqual(exports.get_strings(), ['foo.h', 'bar.h', 'baz.h'])
+        expected = [
+            ('', ['foo.h', 'bar.h', 'baz.h']),
+            ('mozilla', ['mozilla1.h', 'mozilla2.h']),
+            ('mozilla/dom', ['dom1.h', 'dom2.h', 'dom3.h']),
+            ('mozilla/gfx', ['gfx.h']),
+            ('nspr/private', ['pprio.h', 'pprthred.h']),
+            ('overwrite', ['new.h']),
+            ('vpx', ['mem.h', 'mem2.h']),
+        ]
+        for (expect_path, expect_headers), (actual_path, actual_headers) in \
+                zip(expected, [(path, list(seq)) for path, seq in objs[0].exports.walk()]):
+            self.assertEqual(expect_path, actual_path)
+            self.assertEqual(expect_headers, actual_headers)
 
-        self.assertIn('mozilla', exports._children)
-        mozilla = exports._children['mozilla']
-        self.assertEqual(mozilla.get_strings(), ['mozilla1.h', 'mozilla2.h'])
+    def test_test_harness_files(self):
+        reader = self.reader('test-harness-files')
+        objs = self.read_topsrcdir(reader)
 
-        self.assertIn('dom', mozilla._children)
-        dom = mozilla._children['dom']
-        self.assertEqual(dom.get_strings(), ['dom1.h', 'dom2.h', 'dom3.h'])
+        self.assertEqual(len(objs), 1)
+        self.assertIsInstance(objs[0], TestHarnessFiles)
 
-        self.assertIn('gfx', mozilla._children)
-        gfx = mozilla._children['gfx']
-        self.assertEqual(gfx.get_strings(), ['gfx.h'])
+        expected = {
+            'mochitest': ['runtests.py', 'utils.py'],
+            'testing/mochitest': ['mochitest.py', 'mochitest.ini'],
+        }
 
-        self.assertIn('vpx', exports._children)
-        vpx = exports._children['vpx']
-        self.assertEqual(vpx.get_strings(), ['mem.h', 'mem2.h'])
+        for path, strings in objs[0].srcdir_files.iteritems():
+            self.assertTrue(path in expected)
+            basenames = sorted(mozpath.basename(s) for s in strings)
+            self.assertEqual(sorted(expected[path]), basenames)
 
-        self.assertIn('nspr', exports._children)
-        nspr = exports._children['nspr']
-        self.assertIn('private', nspr._children)
-        private = nspr._children['private']
-        self.assertEqual(private.get_strings(), ['pprio.h', 'pprthred.h'])
-
-        self.assertIn('overwrite', exports._children)
-        overwrite = exports._children['overwrite']
-        self.assertEqual(overwrite.get_strings(), ['new.h'])
+    def test_test_harness_files_root(self):
+        reader = self.reader('test-harness-files-root')
+        with self.assertRaisesRegexp(SandboxValidationError,
+            'Cannot install files to the root of TEST_HARNESS_FILES'):
+            objs = self.read_topsrcdir(reader)
 
     def test_resources(self):
         reader = self.reader('resources')
         objs = self.read_topsrcdir(reader)
 
-        expected_defines = reader.config.defines
+        expected_defines = dict(reader.config.defines)
         expected_defines.update({
             'FOO': True,
             'BAR': 'BAZ',
@@ -243,8 +244,8 @@ class TestEmitterBasic(unittest.TestCase):
         self.assertEqual(objs[1].defines, expected_defines)
 
         resources = objs[1].resources
-        self.assertEqual(resources.get_strings(), ['foo.res', 'bar.res', 'baz.res',
-                                                   'foo_p.res.in', 'bar_p.res.in', 'baz_p.res.in'])
+        self.assertEqual(resources._strings, ['foo.res', 'bar.res', 'baz.res',
+                                              'foo_p.res.in', 'bar_p.res.in', 'baz_p.res.in'])
         self.assertFalse(resources['foo.res'].preprocess)
         self.assertFalse(resources['bar.res'].preprocess)
         self.assertFalse(resources['baz.res'].preprocess)
@@ -254,8 +255,8 @@ class TestEmitterBasic(unittest.TestCase):
 
         self.assertIn('mozilla', resources._children)
         mozilla = resources._children['mozilla']
-        self.assertEqual(mozilla.get_strings(), ['mozilla1.res', 'mozilla2.res',
-                                                 'mozilla1_p.res.in', 'mozilla2_p.res.in'])
+        self.assertEqual(mozilla._strings, ['mozilla1.res', 'mozilla2.res',
+                                            'mozilla1_p.res.in', 'mozilla2_p.res.in'])
         self.assertFalse(mozilla['mozilla1.res'].preprocess)
         self.assertFalse(mozilla['mozilla2.res'].preprocess)
         self.assertTrue(mozilla['mozilla1_p.res.in'].preprocess)
@@ -263,25 +264,25 @@ class TestEmitterBasic(unittest.TestCase):
 
         self.assertIn('dom', mozilla._children)
         dom = mozilla._children['dom']
-        self.assertEqual(dom.get_strings(), ['dom1.res', 'dom2.res', 'dom3.res'])
+        self.assertEqual(dom._strings, ['dom1.res', 'dom2.res', 'dom3.res'])
 
         self.assertIn('gfx', mozilla._children)
         gfx = mozilla._children['gfx']
-        self.assertEqual(gfx.get_strings(), ['gfx.res'])
+        self.assertEqual(gfx._strings, ['gfx.res'])
 
         self.assertIn('vpx', resources._children)
         vpx = resources._children['vpx']
-        self.assertEqual(vpx.get_strings(), ['mem.res', 'mem2.res'])
+        self.assertEqual(vpx._strings, ['mem.res', 'mem2.res'])
 
         self.assertIn('nspr', resources._children)
         nspr = resources._children['nspr']
         self.assertIn('private', nspr._children)
         private = nspr._children['private']
-        self.assertEqual(private.get_strings(), ['pprio.res', 'pprthred.res'])
+        self.assertEqual(private._strings, ['pprio.res', 'pprthred.res'])
 
         self.assertIn('overwrite', resources._children)
         overwrite = resources._children['overwrite']
-        self.assertEqual(overwrite.get_strings(), ['new.res'])
+        self.assertEqual(overwrite._strings, ['new.res'])
 
     def test_program(self):
         reader = self.reader('program')
@@ -322,17 +323,6 @@ class TestEmitterBasic(unittest.TestCase):
         paths = sorted([k[len(o.directory)+1:] for k in o.installs.keys()])
         self.assertEqual(paths, ["foo.txt", "just-support.ini"])
 
-    def test_test_manifest_support_files_disabled_test(self):
-        """A test manifest with just disabled tests and support-files is supported."""
-        reader = self.reader('test-manifest-support-disabled-tests')
-
-        objs = self.read_topsrcdir(reader)
-        self.assertEqual(len(objs), 1)
-        o = objs[0]
-        self.assertEqual(len(o.installs), 2)
-        paths = sorted([k[len(o.directory)+1:] for k in o.installs.keys()])
-        self.assertEqual(paths, ["foo.txt", "support-disabled-tests.ini"])
-
     def test_test_manifest_absolute_support_files(self):
         """Support files starting with '/' are placed relative to the install root"""
         reader = self.reader('test-manifest-absolute-support')
@@ -391,7 +381,7 @@ class TestEmitterBasic(unittest.TestCase):
         objs = [o for o in self.read_topsrcdir(reader)
                 if isinstance(o, TestManifest)]
 
-        self.assertEqual(len(objs), 6)
+        self.assertEqual(len(objs), 8)
 
         metadata = {
             'a11y.ini': {
@@ -448,6 +438,14 @@ class TestEmitterBasic(unittest.TestCase):
                     'tail2': False,
                 },
             },
+            'reftest.list': {
+                'flavor': 'reftest',
+                'installs': {},
+            },
+            'crashtest.list': {
+                'flavor': 'crashtest',
+                'installs': {},
+            },
         }
 
         for o in objs:
@@ -479,23 +477,6 @@ class TestEmitterBasic(unittest.TestCase):
             'entry in generated-files not present elsewhere'):
             self.read_topsrcdir(reader),
 
-    # This test is only needed until all harnesses support filtering from
-    # manifests.
-    def test_test_manifest_inactive_ignored(self):
-        """Inactive tests should not be installed."""
-        reader = self.reader('test-manifest-inactive-ignored')
-
-        objs = [o for o in self.read_topsrcdir(reader)
-               if isinstance(o, TestManifest)]
-
-        self.assertEqual(len(objs), 1)
-
-        o = objs[0]
-
-        self.assertEqual(o.flavor, 'mochitest')
-        basenames = set(mozpath.basename(k) for k in o.installs.keys())
-        self.assertEqual(basenames, {'mochitest.ini', 'test_active.html'})
-
     def test_test_manifest_parent_support_files_dir(self):
         """support-files referencing a file in a parent directory works."""
         reader = self.reader('test-manifest-parent-support-files-dir')
@@ -520,6 +501,14 @@ class TestEmitterBasic(unittest.TestCase):
             'lists test that does not exist: test_missing.html'):
             self.read_topsrcdir(reader)
 
+    def test_test_manifest_missing_test_error_unfiltered(self):
+        """Missing test files should result in error, even when the test list is not filtered."""
+        reader = self.reader('test-manifest-missing-test-file-unfiltered')
+
+        with self.assertRaisesRegexp(SandboxValidationError,
+            'lists test that does not exist: missing.js'):
+            self.read_topsrcdir(reader)
+
     def test_ipdl_sources(self):
         reader = self.reader('ipdl_sources')
         objs = self.read_topsrcdir(reader)
@@ -604,6 +593,29 @@ class TestEmitterBasic(unittest.TestCase):
             reader = self.reader('xpidl-module-no-sources')
             self.read_topsrcdir(reader)
 
+    def test_missing_local_includes(self):
+        """LOCAL_INCLUDES containing non-existent directories should be rejected."""
+        with self.assertRaisesRegexp(SandboxValidationError, 'Path specified in '
+            'LOCAL_INCLUDES does not exist'):
+            reader = self.reader('missing-local-includes')
+            self.read_topsrcdir(reader)
+
+    def test_library_defines(self):
+        """Test that LIBRARY_DEFINES is propagated properly."""
+        reader = self.reader('library-defines')
+        objs = self.read_topsrcdir(reader)
+
+        libraries = [o for o in objs if isinstance(o,StaticLibrary)]
+        expected = {
+            'liba': '-DIN_LIBA',
+            'libb': '-DIN_LIBA -DIN_LIBB',
+            'libc': '-DIN_LIBA -DIN_LIBB',
+            'libd': ''
+        }
+        defines = {}
+        for lib in libraries:
+            defines[lib.basename] = ' '.join(lib.defines.get_defines())
+        self.assertEqual(expected, defines)
 
 if __name__ == '__main__':
     main()
