@@ -35,11 +35,11 @@ const CertVerifier::Flags CertVerifier::FLAG_MUST_BE_EV = 2;
 CertVerifier::CertVerifier(ocsp_download_config odc,
                            ocsp_strict_config osc,
                            ocsp_get_config ogc,
-                           pinning_enforcement_config pel)
+                           PinningMode pinningMode)
   : mOCSPDownloadEnabled(odc == ocsp_on)
   , mOCSPStrict(osc == ocsp_strict)
   , mOCSPGETEnabled(ogc == ocsp_get_enabled)
-  , mPinningEnforcementLevel(pel)
+  , mPinningMode(pinningMode)
 {
 }
 
@@ -81,95 +81,67 @@ IsCertBuiltInRoot(CERTCertificate* cert, bool& result) {
   return SECSuccess;
 }
 
-struct ChainValidationCallbackState
+Result
+CertListContainsExpectedKeys(const CERTCertList* certList,
+                             const char* hostname, Time time,
+                             CertVerifier::PinningMode pinningMode)
 {
-  const char* hostname;
-  const CertVerifier::pinning_enforcement_config pinningEnforcementLevel;
-  const SECCertificateUsage usage;
-  const Time time;
-};
+  if (pinningMode == CertVerifier::pinningDisabled) {
+    PR_LOG(gCertVerifierLog, PR_LOG_DEBUG,
+           ("Pinning is disabled; not checking keys."));
+    return Success;
+  }
 
-SECStatus chainValidationCallback(void* state, const CERTCertList* certList,
-                                  PRBool* chainOK)
-{
-  ChainValidationCallbackState* callbackState =
-    reinterpret_cast<ChainValidationCallbackState*>(state);
-
-  *chainOK = PR_FALSE;
-
-  PR_LOG(gCertVerifierLog, PR_LOG_DEBUG,
-         ("verifycert: Inside the Callback \n"));
-
-  // On sanity failure we fail closed.
   if (!certList) {
-    PR_LOG(gCertVerifierLog, PR_LOG_DEBUG,
-           ("verifycert: Short circuit, callback, sanity check failed \n"));
-    PR_SetError(PR_INVALID_STATE_ERROR, 0);
-    return SECFailure;
-  }
-  if (!callbackState) {
-    PR_LOG(gCertVerifierLog, PR_LOG_DEBUG,
-           ("verifycert: Short circuit, callback, no state! \n"));
-    PR_SetError(PR_INVALID_STATE_ERROR, 0);
-    return SECFailure;
+    return Result::FATAL_ERROR_INVALID_ARGS;
   }
 
-  if (callbackState->usage != certificateUsageSSLServer ||
-      callbackState->pinningEnforcementLevel == CertVerifier::pinningDisabled) {
-    PR_LOG(gCertVerifierLog, PR_LOG_DEBUG,
-           ("verifycert: Callback shortcut pel=%d \n",
-            callbackState->pinningEnforcementLevel));
-    *chainOK = PR_TRUE;
-    return SECSuccess;
+  CERTCertListNode* rootNode = CERT_LIST_TAIL(certList);
+  if (CERT_LIST_END(rootNode, certList)) {
+    return Result::FATAL_ERROR_INVALID_ARGS;
   }
 
-  for (CERTCertListNode* node = CERT_LIST_HEAD(certList);
-       !CERT_LIST_END(node, certList);
-       node = CERT_LIST_NEXT(node)) {
-    CERTCertificate* currentCert = node->cert;
-    if (CERT_LIST_END(CERT_LIST_NEXT(node), certList)) {
-      bool isBuiltInRoot = false;
-      SECStatus srv = IsCertBuiltInRoot(currentCert, isBuiltInRoot);
-      if (srv != SECSuccess) {
-        PR_LOG(gCertVerifierLog, PR_LOG_DEBUG, ("Is BuiltInRoot failure"));
-        return srv;
-      }
-      // If desired, the user can enable "allow user CA MITM mode", in which
-      // case key pinning is not enforced for certificates that chain to trust
-      // anchors that are not in Mozilla's root program
-      if (!isBuiltInRoot &&
-          (callbackState->pinningEnforcementLevel ==
-             CertVerifier::pinningAllowUserCAMITM)) {
-        *chainOK = PR_TRUE;
-        return SECSuccess;
-      }
-    }
+  bool isBuiltInRoot = false;
+  SECStatus srv = IsCertBuiltInRoot(rootNode->cert, isBuiltInRoot);
+  if (srv != SECSuccess) {
+    return MapPRErrorCodeToResult(PR_GetError());
+  }
+  // If desired, the user can enable "allow user CA MITM mode", in which
+  // case key pinning is not enforced for certificates that chain to trust
+  // anchors that are not in Mozilla's root program
+  if (!isBuiltInRoot && pinningMode == CertVerifier::pinningAllowUserCAMITM) {
+    return Success;
   }
 
-  bool enforceTestMode = (callbackState->pinningEnforcementLevel ==
-                          CertVerifier::pinningEnforceTestMode);
-  *chainOK = PublicKeyPinningService::
-    ChainHasValidPins(certList, callbackState->hostname, callbackState->time,
-                      enforceTestMode);
+  bool enforceTestMode = (pinningMode == CertVerifier::pinningEnforceTestMode);
+  if (PublicKeyPinningService::ChainHasValidPins(certList, hostname, time,
+                                                 enforceTestMode)) {
+    return Success;
+  }
 
-  return SECSuccess;
+  return Result::ERROR_KEY_PINNING_FAILURE;
 }
 
 static Result
-BuildCertChainForOneKeyUsage(TrustDomain& trustDomain, Input certDER,
+BuildCertChainForOneKeyUsage(NSSCertDBTrustDomain& trustDomain, Input certDER,
                              Time time, KeyUsage ku1, KeyUsage ku2,
                              KeyUsage ku3, KeyPurposeId eku,
                              const CertPolicyId& requiredPolicy,
-                             const Input* stapledOCSPResponse)
+                             const Input* stapledOCSPResponse,
+                             /*optional out*/ CertVerifier::OCSPStaplingStatus*
+                                                ocspStaplingStatus)
 {
+  trustDomain.ResetOCSPStaplingStatus();
   Result rv = BuildCertChain(trustDomain, certDER, time,
                              EndEntityOrCA::MustBeEndEntity, ku1,
                              eku, requiredPolicy, stapledOCSPResponse);
   if (rv == Result::ERROR_INADEQUATE_KEY_USAGE) {
+    trustDomain.ResetOCSPStaplingStatus();
     rv = BuildCertChain(trustDomain, certDER, time,
                         EndEntityOrCA::MustBeEndEntity, ku2,
                         eku, requiredPolicy, stapledOCSPResponse);
     if (rv == Result::ERROR_INADEQUATE_KEY_USAGE) {
+      trustDomain.ResetOCSPStaplingStatus();
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeEndEntity, ku3,
                           eku, requiredPolicy, stapledOCSPResponse);
@@ -178,6 +150,9 @@ BuildCertChainForOneKeyUsage(TrustDomain& trustDomain, Input certDER,
       }
     }
   }
+  if (ocspStaplingStatus) {
+    *ocspStaplingStatus = trustDomain.GetOCSPStaplingStatus();
+  }
   return rv;
 }
 
@@ -187,7 +162,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
                          const Flags flags,
             /*optional*/ const SECItem* stapledOCSPResponseSECItem,
         /*optional out*/ ScopedCERTCertList* builtChain,
-        /*optional out*/ SECOidTag* evOidPolicy)
+        /*optional out*/ SECOidTag* evOidPolicy,
+        /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus)
 {
   PR_LOG(gCertVerifierLog, PR_LOG_DEBUG, ("Top of VerifyCert\n"));
 
@@ -200,6 +176,13 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
   if (evOidPolicy) {
     *evOidPolicy = SEC_OID_UNKNOWN;
   }
+  if (ocspStaplingStatus) {
+    if (usage != certificateUsageSSLServer) {
+      PR_SetError(SEC_ERROR_INVALID_ARGS, 0);
+      return SECFailure;
+    }
+    *ocspStaplingStatus = OCSP_STAPLING_NEVER_CHECKED;
+  }
 
   if (!cert ||
       (usage != certificateUsageSSLServer && (flags & FLAG_MUST_BE_EV))) {
@@ -216,13 +199,6 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
     return SECFailure;
   }
 
-  ChainValidationCallbackState callbackState = {
-    hostname, mPinningEnforcementLevel, usage, time
-  };
-  CERTChainVerifyCallback callbackContainer;
-  callbackContainer.isChainValid = chainValidationCallback;
-  callbackContainer.isChainValidArg = &callbackState;
-
   NSSCertDBTrustDomain::OCSPFetching ocspFetching
     = !mOCSPDownloadEnabled ||
       (flags & FLAG_LOCAL_ONLY) ? NSSCertDBTrustDomain::NeverFetchOCSP
@@ -250,8 +226,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
       // XXX: We don't really have a trust bit for SSL client authentication so
       // just use trustEmail as it is the closest alternative.
       NSSCertDBTrustDomain trustDomain(trustEmail, ocspFetching, mOCSPCache,
-                                       pinArg, ocspGETConfig, nullptr,
-                                       builtChain);
+                                       pinArg, ocspGETConfig, pinningDisabled,
+                                       false, nullptr, builtChain);
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeEndEntity,
                           KeyUsage::digitalSignature,
@@ -276,14 +252,15 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
                       ocspFetching == NSSCertDBTrustDomain::NeverFetchOCSP
                         ? NSSCertDBTrustDomain::LocalOnlyOCSPForEV
                         : NSSCertDBTrustDomain::FetchOCSPForEV,
-                      mOCSPCache, pinArg, ocspGETConfig,
-                      &callbackContainer, builtChain);
+                      mOCSPCache, pinArg, ocspGETConfig, mPinningMode, true,
+                      hostname, builtChain);
         rv = BuildCertChainForOneKeyUsage(trustDomain, certDER, time,
                                           KeyUsage::digitalSignature,// (EC)DHE
                                           KeyUsage::keyEncipherment, // RSA
                                           KeyUsage::keyAgreement,    // (EC)DH
                                           KeyPurposeId::id_kp_serverAuth,
-                                          evPolicy, stapledOCSPResponse);
+                                          evPolicy, stapledOCSPResponse,
+                                          ocspStaplingStatus);
         if (rv == Success) {
           if (evOidPolicy) {
             *evOidPolicy = evPolicyOidTag;
@@ -300,22 +277,23 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
 
       // Now try non-EV.
       NSSCertDBTrustDomain trustDomain(trustSSL, ocspFetching, mOCSPCache,
-                                       pinArg, ocspGETConfig, &callbackContainer,
-                                       builtChain);
+                                       pinArg, ocspGETConfig, mPinningMode,
+                                       false, hostname, builtChain);
       rv = BuildCertChainForOneKeyUsage(trustDomain, certDER, time,
                                         KeyUsage::digitalSignature, // (EC)DHE
                                         KeyUsage::keyEncipherment, // RSA
                                         KeyUsage::keyAgreement, // (EC)DH
                                         KeyPurposeId::id_kp_serverAuth,
                                         CertPolicyId::anyPolicy,
-                                        stapledOCSPResponse);
+                                        stapledOCSPResponse,
+                                        ocspStaplingStatus);
       break;
     }
 
     case certificateUsageSSLCA: {
       NSSCertDBTrustDomain trustDomain(trustSSL, ocspFetching, mOCSPCache,
-                                       pinArg, ocspGETConfig, nullptr,
-                                       builtChain);
+                                       pinArg, ocspGETConfig, pinningDisabled,
+                                       false, nullptr, builtChain);
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeCA, KeyUsage::keyCertSign,
                           KeyPurposeId::id_kp_serverAuth,
@@ -325,8 +303,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
 
     case certificateUsageEmailSigner: {
       NSSCertDBTrustDomain trustDomain(trustEmail, ocspFetching, mOCSPCache,
-                                       pinArg, ocspGETConfig, nullptr,
-                                       builtChain);
+                                       pinArg, ocspGETConfig, pinningDisabled,
+                                       false, nullptr, builtChain);
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeEndEntity,
                           KeyUsage::digitalSignature,
@@ -340,8 +318,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
       // usage it is trying to verify for, and base its algorithm choices
       // based on the result of the verification(s).
       NSSCertDBTrustDomain trustDomain(trustEmail, ocspFetching, mOCSPCache,
-                                       pinArg, ocspGETConfig, nullptr,
-                                       builtChain);
+                                       pinArg, ocspGETConfig, pinningDisabled,
+                                       false, nullptr, builtChain);
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeEndEntity,
                           KeyUsage::keyEncipherment, // RSA
@@ -360,7 +338,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
     case certificateUsageObjectSigner: {
       NSSCertDBTrustDomain trustDomain(trustObjectSigning, ocspFetching,
                                        mOCSPCache, pinArg, ocspGETConfig,
-                                       nullptr, builtChain);
+                                       pinningDisabled, false, nullptr,
+                                       builtChain);
       rv = BuildCertChain(trustDomain, certDER, time,
                           EndEntityOrCA::MustBeEndEntity,
                           KeyUsage::digitalSignature,
@@ -372,8 +351,7 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
     case certificateUsageVerifyCA:
     case certificateUsageStatusResponder: {
       // XXX This is a pretty useless way to verify a certificate. It is used
-      // by the implementation of window.crypto.importCertificates and in the
-      // certificate viewer UI. Because we don't know what trust bit is
+      // by the certificate viewer UI. Because we don't know what trust bit is
       // interesting, we just try them all.
       mozilla::pkix::EndEntityOrCA endEntityOrCA;
       mozilla::pkix::KeyUsage keyUsage;
@@ -389,14 +367,15 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
       }
 
       NSSCertDBTrustDomain sslTrust(trustSSL, ocspFetching, mOCSPCache, pinArg,
-                                    ocspGETConfig, nullptr, builtChain);
+                                    ocspGETConfig, pinningDisabled, false,
+                                    nullptr, builtChain);
       rv = BuildCertChain(sslTrust, certDER, time, endEntityOrCA,
                           keyUsage, eku, CertPolicyId::anyPolicy,
                           stapledOCSPResponse);
       if (rv == Result::ERROR_UNKNOWN_ISSUER) {
         NSSCertDBTrustDomain emailTrust(trustEmail, ocspFetching, mOCSPCache,
-                                        pinArg, ocspGETConfig, nullptr,
-                                        builtChain);
+                                        pinArg, ocspGETConfig, pinningDisabled,
+                                        false, nullptr, builtChain);
         rv = BuildCertChain(emailTrust, certDER, time, endEntityOrCA,
                             keyUsage, eku, CertPolicyId::anyPolicy,
                             stapledOCSPResponse);
@@ -404,6 +383,7 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
           NSSCertDBTrustDomain objectSigningTrust(trustObjectSigning,
                                                   ocspFetching, mOCSPCache,
                                                   pinArg, ocspGETConfig,
+                                                  pinningDisabled, false,
                                                   nullptr, builtChain);
           rv = BuildCertChain(objectSigningTrust, certDER, time,
                               endEntityOrCA, keyUsage, eku,
@@ -419,7 +399,8 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
   }
 
   if (rv != Success) {
-    if (rv != Result::ERROR_KEY_PINNING_FAILURE) {
+    if (rv != Result::ERROR_KEY_PINNING_FAILURE &&
+        usage == certificateUsageSSLServer) {
       ScopedCERTCertificate certCopy(CERT_DupCertificate(cert));
       if (!certCopy) {
         return SECFailure;
@@ -433,13 +414,10 @@ CertVerifier::VerifyCert(CERTCertificate* cert, SECCertificateUsage usage,
         return SECFailure;
       }
       certCopy.forget(); // now owned by certList
-      PRBool chainOK = false;
-      srv = chainValidationCallback(&callbackState, certList, &chainOK);
-      if (srv != SECSuccess) {
-        return SECFailure;
-      }
-      if (!chainOK) {
-        rv = Result::ERROR_KEY_PINNING_FAILURE;
+      Result pinningResult = CertListContainsExpectedKeys(certList, hostname,
+                                                          time, mPinningMode);
+      if (pinningResult != Success) {
+        rv = pinningResult;
       }
     }
     PR_SetError(MapResultToPRErrorCode(rv), 0);
@@ -458,7 +436,8 @@ CertVerifier::VerifySSLServerCert(CERTCertificate* peerCert,
                                   bool saveIntermediatesInPermanentDatabase,
                                   Flags flags,
                  /*optional out*/ ScopedCERTCertList* builtChain,
-                 /*optional out*/ SECOidTag* evOidPolicy)
+                 /*optional out*/ SECOidTag* evOidPolicy,
+                 /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus)
 {
   PR_ASSERT(peerCert);
   // XXX: PR_ASSERT(pinarg)
@@ -478,18 +457,32 @@ CertVerifier::VerifySSLServerCert(CERTCertificate* peerCert,
   }
 
   ScopedCERTCertList builtChainTemp;
-  // CreateCertErrorRunnable assumes that CERT_VerifyCertName is only called
+  // CreateCertErrorRunnable assumes that CheckCertHostname is only called
   // if VerifyCert succeeded.
   SECStatus rv = VerifyCert(peerCert, certificateUsageSSLServer, time, pinarg,
                             hostname, flags, stapledOCSPResponse,
-                            &builtChainTemp, evOidPolicy);
+                            &builtChainTemp, evOidPolicy, ocspStaplingStatus);
   if (rv != SECSuccess) {
     return rv;
   }
 
-  rv = CERT_VerifyCertName(peerCert, hostname);
-  if (rv != SECSuccess) {
-    return rv;
+  Input peerCertInput;
+  Result result = peerCertInput.Init(peerCert->derCert.data,
+                                     peerCert->derCert.len);
+  if (result != Success) {
+    PR_SetError(MapResultToPRErrorCode(result), 0);
+    return SECFailure;
+  }
+  Input hostnameInput;
+  result = hostnameInput.Init(uint8_t_ptr_cast(hostname), strlen(hostname));
+  if (result != Success) {
+    PR_SetError(SEC_ERROR_INVALID_ARGS, 0);
+    return SECFailure;
+  }
+  result = CheckCertHostname(peerCertInput, hostnameInput);
+  if (result != Success) {
+    PR_SetError(MapResultToPRErrorCode(result), 0);
+    return SECFailure;
   }
 
   if (saveIntermediatesInPermanentDatabase) {
