@@ -34,6 +34,7 @@ namespace mozilla { namespace pkix {
 // Verify the given signed data using the given public key.
 Result VerifySignedData(const SignedDataWithSignature& sd,
                         Input subjectPublicKeyInfo,
+                        unsigned int minimumNonECCBits,
                         void* pkcs11PinArg);
 
 // Computes the SHA-1 hash of the data in the current item.
@@ -50,8 +51,10 @@ Result VerifySignedData(const SignedDataWithSignature& sd,
 Result DigestBuf(Input item, /*out*/ uint8_t* digestBuf,
                  size_t digestBufLen);
 
-// Checks, for RSA keys and DSA keys, that the modulus is at least 1024 bits.
-Result CheckPublicKey(Input subjectPublicKeyInfo);
+// Checks, for RSA keys and DSA keys, that the modulus is at least the given
+// number of bits.
+Result CheckPublicKey(Input subjectPublicKeyInfo,
+                      unsigned int minimumNonECCBits);
 
 Result MapPRErrorCodeToResult(PRErrorCode errorCode);
 PRErrorCode MapResultToPRErrorCode(Result result);
@@ -69,7 +72,8 @@ static const PRErrorCode ERROR_LIMIT = ERROR_BASE + 1000;
 enum ErrorCode {
   MOZILLA_PKIX_ERROR_KEY_PINNING_FAILURE = ERROR_BASE + 0,
   MOZILLA_PKIX_ERROR_CA_CERT_USED_AS_END_ENTITY = ERROR_BASE + 1,
-  MOZILLA_PKIX_ERROR_INADEQUATE_KEY_SIZE = ERROR_BASE + 2
+  MOZILLA_PKIX_ERROR_INADEQUATE_KEY_SIZE = ERROR_BASE + 2,
+  MOZILLA_PKIX_ERROR_V1_CERT_USED_AS_CA = ERROR_BASE + 3,
 };
 
 void RegisterErrorTable();
