@@ -31,12 +31,34 @@ class ErrorCodes(object):
     INVALID_RESPONSE = 53
     FRAME_SEND_NOT_INITIALIZED_ERROR = 54
     FRAME_SEND_FAILURE_ERROR = 55
+    FRAME_NOT_RESPONDING = 56
+    UNSUPPORTED_OPERATION = 405
     MARIONETTE_ERROR = 500
 
 class MarionetteException(Exception):
+    """Raised when a generic non-recoverable exception has occured."""
+
     def __init__(self, message=None,
                  status=ErrorCodes.MARIONETTE_ERROR, cause=None,
                  stacktrace=None):
+        """Construct new MarionetteException instance.
+
+        :param message: An optional exception message.
+
+        :param status: A WebDriver status code given as an integer.
+            By default the generic Marionette error code 500 will be
+            used.
+
+        :param cause: An optional tuple of three values giving
+            information about the root exception cause.  Expected
+            tuple values are (type, value, traceback).
+
+        :param stacktrace: Optional string containing a stacktrace
+            (typically from a failed JavaScript execution) that will
+            be displayed in the exception's string representation.
+
+        """
+
         self.msg = message
         self.status = status
         self.cause = cause
@@ -47,11 +69,14 @@ class MarionetteException(Exception):
         tb = None
 
         if self.cause:
-            msg += ", caused by %r" % self.cause[0]
-            tb = self.cause[2]
+            if type(self.cause) is tuple:
+                msg += ", caused by %r" % self.cause[0]
+                tb = self.cause[2]
+            else:
+                msg += ", caused by %s" % self.cause
         if self.stacktrace:
-            stack = "".join(["\t%s\n" % x for x in self.stacktrace.splitlines()])
-            msg += "\nstacktrace:\n%s" % stack
+            st = "".join(["\t%s\n" % x for x in self.stacktrace.splitlines()])
+            msg += "\nstacktrace:\n%s" % st
 
         return "".join(traceback.format_exception(self.__class__, msg, tb))
 
@@ -84,8 +109,10 @@ class ScriptTimeoutException(MarionetteException):
 
 class ElementNotVisibleException(MarionetteException):
     def __init__(self, message="Element is not currently visible and may not be manipulated",
-                status=ErrorCodes.ELEMENT_NOT_VISIBLE, stacktrace=None):
-        MarionetteException.__init__(self, message, status, stacktrace)
+                 status=ErrorCodes.ELEMENT_NOT_VISIBLE,
+                 stacktrace=None, cause=None):
+        super(ElementNotVisibleException, self).__init__(
+            message, status=status, cause=cause, stacktrace=stacktrace)
 
 class NoSuchFrameException(MarionetteException):
     pass
@@ -113,3 +140,6 @@ class FrameSendNotInitializedError(MarionetteException):
 
 class FrameSendFailureError(MarionetteException):
     pass
+
+class UnsupportedOperationException(MarionetteException):
+    pass
