@@ -24,8 +24,8 @@ let utils = {};
 loader.loadSubScript("chrome://marionette/content/EventUtils.js", utils);
 loader.loadSubScript("chrome://marionette/content/ChromeUtils.js", utils);
 
-function sendKeysToElement (document, element, keysToSend, successCallback, errorCallback, command_id) {
-  if (checkVisible(element)) {
+function sendKeysToElement (document, element, keysToSend, successCallback, errorCallback, command_id, context) {
+  if (context == "chrome" || checkVisible(element)) {
     element.focus();
     var value = keysToSend.join("");
     let hasShift = null;
@@ -33,7 +33,6 @@ function sendKeysToElement (document, element, keysToSend, successCallback, erro
     let hasAlt = null;
     let hasMeta = null;
     for (var i = 0; i < value.length; i++) {
-      let upper = value.charAt(i).toUpperCase();
       var keyCode = null;
       var c = value.charAt(i);
       switch (c) {
@@ -62,15 +61,15 @@ function sendKeysToElement (document, element, keysToSend, successCallback, erro
           break;
         case '\uE009':
           keyCode = "VK_CONTROL";
-          controlKey = !controlKey;
+          hasCtrl = !hasCtrl;
           break;
         case '\uE00A':
           keyCode = "VK_ALT";
-          altKey = !altKey;
+          hasAlt = !hasAlt;
           break;
         case '\uE03D':
           keyCode = "VK_META";
-          metaKey = !metaKey;
+          hasMeta = !hasMeta;
           break;
         case '\uE00B':
           keyCode = "VK_PAUSE";
@@ -202,7 +201,9 @@ function sendKeysToElement (document, element, keysToSend, successCallback, erro
           keyCode = "VK_F12";
           break;
       }
-      hasShift = value.charAt(i) == upper;
+      let charCode = c.charCodeAt(0);
+      let isUpper = charCode >= 65 && charCode <= 90;
+      hasShift = isUpper || hasShift;
       utils.synthesizeKey(keyCode || value[i],
                           { shiftKey: hasShift, ctrlKey: hasCtrl, altKey: hasAlt, metaKey: hasMeta },
                           document);
@@ -212,4 +213,4 @@ function sendKeysToElement (document, element, keysToSend, successCallback, erro
   else {
     errorCallback("Element is not visible", 11, null, command_id);
   }
-};
\ No newline at end of file
+};
