@@ -35,7 +35,6 @@ let test = asyncTest(function*() {
 });
 
 function wait(ms) {
-  let def = promise.defer();
-  setTimeout(def.resolve, ms);
-  return def.promise;
+  return new Promise(resolve =>
+    setTimeout(resolve, ms));
 }
