@@ -32,6 +32,7 @@ import os
 from StringIO import StringIO
 import subprocess
 import platform
+import mozinfo
 
 # List of libraries to shlibsign.
 SIGN_LIBS = [
@@ -80,6 +81,13 @@ class ToolLauncher(object):
         for e in extra_env:
             env[e] = extra_env[e]
 
+        # For VC12, make sure we can find the right bitness of pgort120.dll
+        if 'VS120COMNTOOLS' in env and not buildconfig.substs['HAVE_64BIT_BUILD']:
+            vc12dir = os.path.abspath(os.path.join(env['VS120COMNTOOLS'],
+                                                   '../../VC/bin'))
+            if os.path.exists(vc12dir):
+                env['PATH'] = vc12dir + ';' + env['PATH']
+
         # Work around a bug in Python 2.7.2 and lower where unicode types in
         # environment variables aren't handled by subprocess.
         for k, v in env.items():
@@ -138,14 +146,6 @@ def precompile_cache(formatter, source_path, gre_path, app_path):
     os.close(fd)
     os.remove(cache)
 
-    # For VC12, make sure we can find the right bitness of pgort120.dll
-    env = os.environ.copy()
-    if 'VS120COMNTOOLS' in env and not buildconfig.substs['HAVE_64BIT_OS']:
-      vc12dir = os.path.abspath(os.path.join(env['VS120COMNTOOLS'],
-                                             '../../VC/bin'))
-      if os.path.exists(vc12dir):
-        env['PATH'] = vc12dir + ';' + env['PATH']
-
     try:
         if launcher.launch(['xpcshell', '-g', gre_path, '-a', app_path,
                             '-f', os.path.join(os.path.dirname(__file__),
@@ -153,8 +153,7 @@ def precompile_cache(formatter, source_path, gre_path, app_path):
                             '-e', 'precompile_startupcache("resource://%s/");'
                                   % resource],
                            extra_linker_path=gre_path,
-                           extra_env={'MOZ_STARTUP_CACHE': cache,
-                                      'PATH': env['PATH']}):
+                           extra_env={'MOZ_STARTUP_CACHE': cache}):
             errors.fatal('Error while running startup cache precompilation')
             return
         from mozpack.mozjar import JarReader
@@ -296,11 +295,11 @@ def main():
     elif 'MOZ_OMNIJAR' in defines:
         del defines['MOZ_OMNIJAR']
 
-    binpath = ''
-    if 'BINPATH' in defines:
-        binpath = SimpleManifestSink.normalize_path(defines['BINPATH'])
-    while binpath.startswith('/'):
-        binpath = binpath[1:]
+    respath = ''
+    if 'RESPATH' in defines:
+        respath = SimpleManifestSink.normalize_path(defines['RESPATH'])
+    while respath.startswith('/'):
+        respath = respath[1:]
 
     if args.unify:
         def is_native(path):
@@ -351,18 +350,19 @@ def main():
             removals_in.name = args.removals
             removals = RemovedFiles(copier)
             preprocess(removals_in, removals, defines)
-            copier.add(mozpack.path.join(binpath, 'removed-files'), removals)
+            copier.add(mozpack.path.join(respath, 'removed-files'), removals)
 
     # shlibsign libraries
     if launcher.can_launch():
-        for lib in SIGN_LIBS:
-            libbase = mozpack.path.join(binpath, '%s%s') \
-                % (buildconfig.substs['DLL_PREFIX'], lib)
-            libname = '%s%s' % (libbase, buildconfig.substs['DLL_SUFFIX'])
-            if copier.contains(libname):
-                copier.add(libbase + '.chk',
-                           LibSignFile(os.path.join(args.destination,
-                                                    libname)))
+        if not mozinfo.isMac:
+            for lib in SIGN_LIBS:
+                libbase = mozpack.path.join(respath, '%s%s') \
+                    % (buildconfig.substs['DLL_PREFIX'], lib)
+                libname = '%s%s' % (libbase, buildconfig.substs['DLL_SUFFIX'])
+                if copier.contains(libname):
+                    copier.add(libbase + '.chk',
+                               LibSignFile(os.path.join(args.destination,
+                                                        libname)))
 
     # Setup preloading
     if args.jarlog and os.path.exists(args.jarlog):
@@ -378,7 +378,7 @@ def main():
     # Fill startup cache
     if isinstance(formatter, OmniJarFormatter) and launcher.can_launch() \
       and buildconfig.substs['MOZ_DISABLE_STARTUPCACHE'] != '1':
-        if buildconfig.substs['LIBXUL_SDK']:
+        if buildconfig.substs.get('LIBXUL_SDK'):
             gre_path = mozpack.path.join(buildconfig.substs['LIBXUL_DIST'],
                                          'bin')
         else:
@@ -395,7 +395,7 @@ def main():
 
     copier.copy(args.destination)
     generate_precomplete(os.path.normpath(os.path.join(args.destination,
-                                                       binpath)))
+                                                       respath)))
 
 
 if __name__ == '__main__':
