@@ -163,8 +163,9 @@ nsAndroidProtocolHandler::NewURI(const nsACString &aSpec,
 }
 
 NS_IMETHODIMP
-nsAndroidProtocolHandler::NewChannel(nsIURI* aURI,
-                                     nsIChannel* *aResult)
+nsAndroidProtocolHandler::NewChannel2(nsIURI* aURI,
+                                      nsILoadInfo* aLoadInfo,
+                                      nsIChannel** aResult)
 {
     nsCOMPtr<nsIChannel> channel = AndroidChannel::CreateChannel(aURI);
     if (!channel)
@@ -172,3 +173,10 @@ nsAndroidProtocolHandler::NewChannel(nsIURI* aURI,
     NS_ADDREF(*aResult = channel);
     return NS_OK;
 }
+
+NS_IMETHODIMP
+nsAndroidProtocolHandler::NewChannel(nsIURI* aURI,
+                                     nsIChannel* *aResult)
+{
+    return NewChannel2(aURI, nullptr, aResult);
+}
