@@ -31,6 +31,7 @@
 #include <utils/threads.h>
 
 #include "mozilla/layers/LayersSurfaces.h"
+#include "mozilla/layers/TextureClient.h"
 
 namespace android {
 // ----------------------------------------------------------------------------
@@ -38,7 +39,7 @@ namespace android {
 class GonkBufferQueue : public BnGraphicBufferProducer,
                     public BnGonkGraphicBufferConsumer,
                     private IBinder::DeathRecipient {
-    typedef mozilla::layers::SurfaceDescriptor SurfaceDescriptor;
+    typedef mozilla::layers::TextureClient TextureClient;
 
 public:
     enum { MIN_UNDEQUEUED_BUFFERS = 2 };
@@ -186,6 +187,16 @@ public:
     // will usually be the one obtained from dequeueBuffer.
     virtual void cancelBuffer(int buf, const sp<Fence>& fence);
 
+    // setSynchronousMode sets whether dequeueBuffer is synchronous or
+    // asynchronous. In synchronous mode, dequeueBuffer blocks until
+    // a buffer is available, the currently bound buffer can be dequeued and
+    // queued buffers will be acquired in order.  In asynchronous mode,
+    // a queued buffer may be replaced by a subsequently queued buffer.
+    //
+    // The default mode is synchronous.
+    // This should be called only during initialization.
+    virtual status_t setSynchronousMode(bool enabled);
+
     // connect attempts to connect a producer API to the GonkBufferQueue.  This
     // must be called before any other IGraphicBufferProducer methods are
     // called except for getAllocator.  A consumer must already be connected.
@@ -312,14 +323,11 @@ public:
     // dump our state in a String
     virtual void dump(String8& result, const char* prefix) const;
 
-    int getGeneration();
-    SurfaceDescriptor *getSurfaceDescriptorFromBuffer(ANativeWindowBuffer* buffer);
+     mozilla::TemporaryRef<TextureClient> getTextureClientFromBuffer(ANativeWindowBuffer* buffer);
+
+    int getSlotFromTextureClientLocked(TextureClient* client) const;
 
 private:
-    // releaseBufferFreeListUnlocked releases the resources in the freeList;
-    // this must be called with mMutex unlocked.
-    void releaseBufferFreeListUnlocked(nsTArray<SurfaceDescriptor>& freeList);
-
     // freeBufferLocked frees the GraphicBuffer and sync resources for the
     // given slot.
     //void freeBufferLocked(int index);
@@ -327,7 +335,7 @@ private:
     // freeAllBuffersLocked frees the GraphicBuffer and sync resources for
     // all slots.
     //void freeAllBuffersLocked();
-    void freeAllBuffersLocked(nsTArray<SurfaceDescriptor>& freeList);
+    void freeAllBuffersLocked();
 
     // setDefaultMaxBufferCountLocked sets the maximum number of buffer slots
     // that will be used if the producer does not override the buffer slot
@@ -366,8 +374,7 @@ private:
     struct BufferSlot {
 
         BufferSlot()
-        : mSurfaceDescriptor(SurfaceDescriptor()),
-          mBufferState(BufferSlot::FREE),
+        : mBufferState(BufferSlot::FREE),
           mRequestBufferCalled(false),
           mFrameNumber(0),
           mAcquireCalled(false),
@@ -378,8 +385,8 @@ private:
         // if no buffer has been allocated.
         sp<GraphicBuffer> mGraphicBuffer;
 
-        // mSurfaceDescriptor is the token to remotely allocated GraphicBuffer.
-        SurfaceDescriptor mSurfaceDescriptor;
+        // mTextureClient is a thin abstraction over remotely allocated GraphicBuffer.
+        mozilla::RefPtr<TextureClient> mTextureClient;
 
         // BufferState represents the different states in which a buffer slot
         // can be.  All slots are initially FREE.
@@ -502,6 +509,9 @@ private:
     // to NULL and is written by consumerConnect and consumerDisconnect.
     sp<IConsumerListener> mConsumerListener;
 
+    // mSynchronousMode whether we're in synchronous mode or not
+    bool mSynchronousMode;
+
     // mConsumerControlledByApp whether the connected consumer is controlled by the
     // application.
     bool mConsumerControlledByApp;
@@ -565,9 +575,6 @@ private:
 
     // mConnectedProducerToken is used to set a binder death notification on the producer
     sp<IBinder> mConnectedProducerToken;
-
-    // mGeneration is the current generation of buffer slots
-    uint32_t mGeneration;
 };
 
 // ----------------------------------------------------------------------------
