@@ -68,14 +68,7 @@ public:
    * Notifies IME of the notification (a request or an event).
    */
   static nsresult NotifyIME(nsWindow* aWindow,
-                            NotificationToIME aNotification);
-
-  /**
-   * Notifies IME of text change in the focused editable content.
-   */
-  static nsresult NotifyIMEOfTextChange(uint32_t aStart,
-                                        uint32_t aOldEnd,
-                                        uint32_t aNewEnd);
+                            const IMENotification& aIMENotification);
 
   /**
    * Returns update preferences.
@@ -119,14 +112,7 @@ public:
 
 private:
 #ifdef NS_ENABLE_TSF
-  typedef HRESULT (WINAPI *SetInputScopesFunc)(HWND windowHandle,
-                                               const InputScope *inputScopes,
-                                               UINT numInputScopes,
-                                               wchar_t **phrase_list,
-                                               UINT numPhraseList,
-                                               wchar_t *regExp,
-                                               wchar_t *srgs);
-  static SetInputScopesFunc sSetInputScopes;
+  static decltype(SetInputScopes)* sSetInputScopes;
   static void SetInputScopeForIMM32(nsWindow* aWindow,
                                     const nsAString& aHTMLInputType);
   static bool sIsInTSFMode;
@@ -136,6 +122,7 @@ private:
   static bool sPluginHasFocus;
 
   static bool IsTSFAvailable() { return (sIsInTSFMode && !sPluginHasFocus); }
+  static bool IsIMMActive();
 #endif // #ifdef NS_ENABLE_TSF
 };
 
